/*
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2007 - 2014.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */

(function(){
dojo.provide("com.ibm.rdm.web.model.types.RRMTypes");
dojo.require("com.ibm.rdm.web.common.bootstrap");
dojo.require("com.ibm.rdm.web.model.types.XmlNamespaces");
dojo.require("com.ibm.rdm.web.common.ErrorHandler");

var ErrorHandler = com.ibm.rdm.web.common.ErrorHandler;
var XmlNamespaces = com.ibm.rdm.web.model.types.XmlNamespaces;

com.ibm.rdm.web.model.types.RRMTypes = {
	///////////////////////////////////////////////////////////////////////////////////
	// PUBLIC FIELDS
	
	ETAG: "http://jazz.net/xmlns/foundation/1.0/etag",
	
	// Note: In order to retrieve the revision URL from the
	// View Service, you must make sure that the ETag has also been
	// requested. See Defect 64781 for details.
	REVISION: "http://www.ibm.com/xmlns/rdm/rdf/contentLocation",

	ATTRIBUTE_TYPES: {
		IntegerType:         XmlNamespaces.XML.uri + "int",
		FloatType:           XmlNamespaces.XML.uri + "float",
		BooleanType:         XmlNamespaces.XML.uri + "boolean",
		DateType:            XmlNamespaces.XML.uri + "date",
		TimeType:            XmlNamespaces.XML.uri + "time",
		DateTimeType:        XmlNamespaces.XML.uri + "dateTime",
		StringType:          XmlNamespaces.XML.uri + "string",
		DurationType:        XmlNamespaces.XML.uri + "duration",
		UserType:            XmlNamespaces.RRS30Types.uri + "UserAttributeType",
		ArtifactFormatsType: XmlNamespaces.RRS30Types.uri + "ArtifactFormats"
	},

	WELL_KNOWN_ATTRIBUTE_NAMES: {
		PrimaryText:		"PrimaryText",
		Title:				"Title",
		Description:		"Description",
		Creator:			"Creator",
		Created:			"Created",
		Contributor:		"Contributor",
		ModifiedOn:			"ModifiedOn",
		ArtifactFormat:		"ArtifactFormat",
		Identifier:			"Identifier",
		AlternateSpelling:	"AlternateSpelling"
	},
	
	RESERVED_NAMES: {
		Type:				"Type"
	},

	// Note by SPC: I'm not naming this WELL_KNOWN_PROCESS_AREA due to the fact
	// that this attribute, in server terms, is not a "well known" attribute
	// (but neither is "ofType", "isHeading" and "module" found below).  I
	// asked Mark what he considers such attributes to be and he told me that
	// they're considered to be just "RDF properties" ... so that's why I've
	// chosen the name "RDF_PROCESS_AREA" here.
	RDF_PROCESS_AREA: "http://www.ibm.com/xmlns/rdm/rdf/processArea",
	
	RDF_LOCK: "http://www.ibm.com/xmlns/rdm/rdf/lock",
	RDF_LOCKED_ON: "http://www.ibm.com/xmlns/rdm/rdf/lockedOn",
	RDF_LOCKED_BY: "http://www.ibm.com/xmlns/rdm/rdf/lockedBy", 
	RDF_CURRENT_USER: "http://www.ibm.com/xmlns/rdm/CurrentUser",
	
	
	MODULE_COPY_POLICIES: {
		DEFAULT_MODULE_COPY_POLICY: "http://www.ibm.com/xmlns/rdm/rdf/defaultCopyReuseFilter",
		DUPLICATE_ALL: "http://www.ibm.com/xmlns/rdm/rdf/copy/duplicateAll",
		DUPLICATE_NONE: "http://www.ibm.com/xmlns/rdm/rdf/copy/reuseAll",
		DUPLICATE_HEADINGS: "http://www.ibm.com/xmlns/rdm/rdf/copy/duplicateHeadings",
		REUSE_HEADINGS: "http://www.ibm.com/xmlns/rdm/rdf/copy/reuseHeadings"
	},
	
	// SW:  maybe this is not the best place for these ?
	MODULE_RELATIVE_TARGET_POSITION: {
		BEFORE: "http://www.ibm.com/xmlns/rdm/rdf/relativeTargetPosition/before",
		BELOW: "http://www.ibm.com/xmlns/rdm/rdf/relativeTargetPosition/below",
		AFTER: "http://www.ibm.com/xmlns/rdm/rdf/relativeTargetPosition/after"
	},
	
	WELL_KNOWN_OBJECT_TYPE: "http://www.ibm.com/xmlns/rdm/rdf/ofType",
	WELL_KNOWN_IS_HEADING: "http://www.ibm.com/xmlns/rdm/rdf/isHeading",
	WELL_KNOWN_HAS_GENERATED_TITLE: "http://www.ibm.com/xmlns/rdm/rdf/hasServerGeneratedTitle",
	
	WELL_KNOWN_MODULE_TYPE: "http://www.ibm.com/xmlns/rdm/rdf/module",
	
	WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES: {
		PrimaryText:		XmlNamespaces.RRS30Types.uri + "PrimaryText",
		ArtifactFormat:		XmlNamespaces.RRS30Types.uri + "ArtifactFormat",
		ArtifactFormats:    XmlNamespaces.RRS30Types.uri + "ArtifactFormats",
		AlternateSpelling:	XmlNamespaces.RRS30Types.uri + "AlternateSpelling",
		Title:				XmlNamespaces.DcTerms.uri + "title",
		Description:		XmlNamespaces.DcTerms.uri + "description",
		Creator:			XmlNamespaces.DcTerms.uri + "creator",
		Created:			XmlNamespaces.DcTerms.uri + "created",
		Contributor:		XmlNamespaces.DcTerms.uri + "contributor",
		ModifiedOn:			XmlNamespaces.DcTerms.uri + "modified",
		Identifier:			XmlNamespaces.DcTerms.uri + "identifier",
		WrappedResourceURI: "http://www.ibm.com/xmlns/rdm/rdf/wrappedResourceURI",
		WrappedResourceMimeType: "http://www.ibm.com/xmlns/rdm/rdf/wrappedResourceMimeType",
		WrappedResourceRevisionURI: "http://www.ibm.com/xmlns/rdm/rdf/wrappedResourceRevisionURI",
		//Parent for folders/folder members
		Parent: "http://com.ibm.rdm/navigation#parent",
		
		//exclusive module properties
		BoundArtifact:		XmlNamespaces.RRS30.uri + "boundArtifact",
		ParentBinding:		XmlNamespaces.RRS30.uri + "parentBinding",
		Module:				XmlNamespaces.RRS30.uri + "module",
		BookOrder:			XmlNamespaces.RRS30.uri + "bookOrder",
		ChildNumber:		XmlNamespaces.RRS30.uri + "childNumber",
		Depth:				XmlNamespaces.RRS30.uri + "depth",
		Section:			XmlNamespaces.RRS30.uri + "section",

		//exclusive module comments properties
		UnresolvedCommentCount: 		"http://www.ibm.com/xmlns/rdm/rdf/unresolvedCommentCount",
		ResolvedCommentCount:			"http://www.ibm.com/xmlns/rdm/rdf/resolvedCommentCount",
		RecentCommentCount:				"http://www.ibm.com/xmlns/rdm/rdf/recentCommentCount",
		MostRecentComment:				"http://www.ibm.com/xmlns/rdm/rdf/mostRecentComment",
		// This is not really an attribute, but is used to identify the column for sorting
		CommentCount:					"http://www.ibm.com/xmlns/rdm/rdf/commentCount",
		// This is not really an attribute, but is used to identify the column for sorting
		ReuseCount:					 	"http://www.ibm.com/xmlns/rdm/rdf/reuseCount",
		//for resource permissions:
		ProcessArea:					"http://www.ibm.com/xmlns/rdm/rdf/processArea"
	},
		
	WELL_KNOWN_OBJECT_TYPE_ROLES: {
		GlossaryTerm:		XmlNamespaces.RRS30Types.uri + "ObjectTypeRoles#GlossaryTerm",
		ModuleHeading:		XmlNamespaces.RRS30Types.uri + "ObjectTypeRoles#ModuleHeading"
	},
	
	EXPORT_FORMATS: {
		 PDF: "http://com.ibm.rdm/reports/formats/pdf",
		 WORD: "http://com.ibm.rdm/reports/formats/word",
		 HTML: "http://com.ibm.rdm/reports/formats/html",
		 XSLFO: "http://com.ibm.rdm/reports/formats/xslfo"
	},
	
	EXPORT_LAYOUTS: {
		BOOK: "http://com.ibm.rdm/reports/layouts/book",
		TABLE: "http://com.ibm.rdm/reports/layouts/table"
	},

	_WellKnownAttributeTypes: null,
	_ProjectsProperties: null,
	_ArtifactFormats: null,
	
	initialize: function(queryResponse) {
		
//				debugger;

				this._ProjectsProperties = {};
				for (var i=0; i<queryResponse.length; i++) {
					if (!this._ProjectsProperties[queryResponse[i].projectURI]) {
						this._ProjectsProperties[queryResponse[i].projectURI] = {};
						this._ProjectsProperties[queryResponse[i].projectURI].WellKnownAttributeDefinitions = [];
						this._ProjectsProperties[queryResponse[i].projectURI].ObjectTypeRoles = {};
					}
					switch (queryResponse[i].sameAs) {
						case com.ibm.rdm.web.model.types.RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.PrimaryText:
							this._ProjectsProperties[queryResponse[i].projectURI].PrimaryText = queryResponse[i].resourceURI;
							this._ProjectsProperties[queryResponse[i].projectURI].WellKnownAttributeDefinitions["PrimaryText"] = {};
							this._ProjectsProperties[queryResponse[i].projectURI].WellKnownAttributeDefinitions["PrimaryText"].reference = 
											XmlNamespaces.DcTerms.uri + "PrimaryText"; 
							this._ProjectsProperties[queryResponse[i].projectURI].WellKnownAttributeDefinitions["PrimaryText"].uri = 
											queryResponse[i].resourceURI;
							this._ProjectsProperties[queryResponse[i].projectURI].WellKnownAttributeDefinitions["PrimaryText"].sameAs =
											queryResponse[i].sameAs;
							this._ProjectsProperties[queryResponse[i].projectURI].WellKnownAttributeDefinitions["PrimaryText"].attributeType = 
											com.ibm.rdm.web.model.types.RRMTypes.ATTRIBUTE_TYPES.StringType;
							break;
						//server should fix this in M7
						case com.ibm.rdm.web.model.types.RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.ArtifactFormat:
							this._ProjectsProperties[queryResponse[i].projectURI].ArtifactFormat = queryResponse[i].resourceURI;
							this._ProjectsProperties[queryResponse[i].projectURI].WellKnownAttributeDefinitions["ArtifactFormat"] = {};
							this._ProjectsProperties[queryResponse[i].projectURI].WellKnownAttributeDefinitions["ArtifactFormat"].reference = 
											XmlNamespaces.DcTerms.uri + "ArtifactFormat";
							this._ProjectsProperties[queryResponse[i].projectURI].WellKnownAttributeDefinitions["ArtifactFormat"].uri = 
											queryResponse[i].resourceURI;
							this._ProjectsProperties[queryResponse[i].projectURI].WellKnownAttributeDefinitions["ArtifactFormat"].sameAs =
											queryResponse[i].sameAs;
							this._ProjectsProperties[queryResponse[i].projectURI].WellKnownAttributeDefinitions["ArtifactFormat"].attributeType = 
											com.ibm.rdm.web.model.types.RRMTypes.ATTRIBUTE_TYPES.ArtifactFormatsType;
							break;
						case com.ibm.rdm.web.model.types.RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.ArtifactFormats:
							this._ProjectsProperties[queryResponse[i].projectURI].ArtifactFormatsURI = queryResponse[i].resourceURI;
							this._ProjectsProperties[queryResponse[i].projectURI].ArtifactFormats =
								{
									"Text":				queryResponse[i].resourceURI + "#Text",
									"WrapperResource":	queryResponse[i].resourceURI + "#WrapperResource",
									"Collection":		queryResponse[i].resourceURI + "#Collection",
									"Sketch":		queryResponse[i].resourceURI + "#Sketch",
									"Storyboard":		queryResponse[i].resourceURI + "#Storyboard",
									"Part":		queryResponse[i].resourceURI + "#Part",
									"UseCaseDiagram":		queryResponse[i].resourceURI + "#UseCaseDiagram",
									"BusinessProcessDiagram":		queryResponse[i].resourceURI + "#BusinessProcessDiagram",
									"SimpleFlowDiagram":		queryResponse[i].resourceURI + "#SimpleFlowDiagram",
									"ScreenFlow":		queryResponse[i].resourceURI + "#ScreenFlow",
									"Module":		queryResponse[i].resourceURI + "#Module"									
								};
								if (!com.ibm.rdm.web.release.HIDE_2011_FEATURES) {
									this._ProjectsProperties[queryResponse[i].projectURI].ArtifactFormats["Glossary"] = queryResponse[i].resourceURI + "#Glossary";
									this._ProjectsProperties[queryResponse[i].projectURI].ArtifactFormats["Term"] = queryResponse[i].resourceURI + "#Term";
									this._ProjectsProperties[queryResponse[i].projectURI].ArtifactFormats["Document"] = queryResponse[i].resourceURI + "#Document";
								}
							break;
						case com.ibm.rdm.web.model.types.RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.Title:
							this._ProjectsProperties[queryResponse[i].projectURI].WellKnownAttributeDefinitions["Title"] = {};
							this._ProjectsProperties[queryResponse[i].projectURI].WellKnownAttributeDefinitions["Title"].uri = 
											queryResponse[i].resourceURI;
							this._ProjectsProperties[queryResponse[i].projectURI].WellKnownAttributeDefinitions["Title"].sameAs =
											queryResponse[i].sameAs;
							this._ProjectsProperties[queryResponse[i].projectURI].WellKnownAttributeDefinitions["Title"].attributeType = 
											com.ibm.rdm.web.model.types.RRMTypes.ATTRIBUTE_TYPES.StringType;
							break;
						case com.ibm.rdm.web.model.types.RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.Description:
							this._ProjectsProperties[queryResponse[i].projectURI].WellKnownAttributeDefinitions["Description"] = {};
							this._ProjectsProperties[queryResponse[i].projectURI].WellKnownAttributeDefinitions["Description"].uri = 
											queryResponse[i].resourceURI;
							this._ProjectsProperties[queryResponse[i].projectURI].WellKnownAttributeDefinitions["Description"].sameAs =
											queryResponse[i].sameAs;
							this._ProjectsProperties[queryResponse[i].projectURI].WellKnownAttributeDefinitions["Description"].attributeType = 
											com.ibm.rdm.web.model.types.RRMTypes.ATTRIBUTE_TYPES.StringType;
							break;
						case com.ibm.rdm.web.model.types.RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.Creator:
							this._ProjectsProperties[queryResponse[i].projectURI].WellKnownAttributeDefinitions["Creator"] = {};
							this._ProjectsProperties[queryResponse[i].projectURI].WellKnownAttributeDefinitions["Creator"].uri = 
											queryResponse[i].resourceURI;
							this._ProjectsProperties[queryResponse[i].projectURI].WellKnownAttributeDefinitions["Creator"].sameAs =
											queryResponse[i].sameAs;
							this._ProjectsProperties[queryResponse[i].projectURI].WellKnownAttributeDefinitions["Creator"].isSystemAttribute = true;
							this._ProjectsProperties[queryResponse[i].projectURI].WellKnownAttributeDefinitions["Creator"].attributeType = 
											com.ibm.rdm.web.model.types.RRMTypes.ATTRIBUTE_TYPES.UserType;
							break;
						case com.ibm.rdm.web.model.types.RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.Created:
							this._ProjectsProperties[queryResponse[i].projectURI].WellKnownAttributeDefinitions["Created"] = {};
							this._ProjectsProperties[queryResponse[i].projectURI].WellKnownAttributeDefinitions["Created"].uri = 
											queryResponse[i].resourceURI;
							this._ProjectsProperties[queryResponse[i].projectURI].WellKnownAttributeDefinitions["Created"].sameAs =
											queryResponse[i].sameAs;
							this._ProjectsProperties[queryResponse[i].projectURI].WellKnownAttributeDefinitions["Created"].isSystemAttribute = true;
							this._ProjectsProperties[queryResponse[i].projectURI].WellKnownAttributeDefinitions["Created"].attributeType = 
											com.ibm.rdm.web.model.types.RRMTypes.ATTRIBUTE_TYPES.DateTimeType;
							break;
						case com.ibm.rdm.web.model.types.RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.Contributor:
							this._ProjectsProperties[queryResponse[i].projectURI].WellKnownAttributeDefinitions["Contributor"] = {};
							this._ProjectsProperties[queryResponse[i].projectURI].WellKnownAttributeDefinitions["Contributor"].uri = 
											queryResponse[i].resourceURI;
							this._ProjectsProperties[queryResponse[i].projectURI].WellKnownAttributeDefinitions["Contributor"].sameAs =
											queryResponse[i].sameAs;
							this._ProjectsProperties[queryResponse[i].projectURI].WellKnownAttributeDefinitions["Contributor"].isSystemAttribute = true;
							this._ProjectsProperties[queryResponse[i].projectURI].WellKnownAttributeDefinitions["Contributor"].attributeType = 
											com.ibm.rdm.web.model.types.RRMTypes.ATTRIBUTE_TYPES.UserType;
							break;
						case com.ibm.rdm.web.model.types.RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.ModifiedOn:
							this._ProjectsProperties[queryResponse[i].projectURI].WellKnownAttributeDefinitions["ModifiedOn"] = {};
							this._ProjectsProperties[queryResponse[i].projectURI].WellKnownAttributeDefinitions["ModifiedOn"].uri = 
											queryResponse[i].resourceURI;
							this._ProjectsProperties[queryResponse[i].projectURI].WellKnownAttributeDefinitions["ModifiedOn"].sameAs =
											queryResponse[i].sameAs;
							this._ProjectsProperties[queryResponse[i].projectURI].WellKnownAttributeDefinitions["ModifiedOn"].isSystemAttribute = true;
							this._ProjectsProperties[queryResponse[i].projectURI].WellKnownAttributeDefinitions["ModifiedOn"].attributeType = 
											com.ibm.rdm.web.model.types.RRMTypes.ATTRIBUTE_TYPES.DateTimeType;
							break;
						case com.ibm.rdm.web.model.types.RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.Identifier:
							this._ProjectsProperties[queryResponse[i].projectURI].WellKnownAttributeDefinitions["Identifier"] = {};
							this._ProjectsProperties[queryResponse[i].projectURI].WellKnownAttributeDefinitions["Identifier"].uri = 
											queryResponse[i].resourceURI;
							this._ProjectsProperties[queryResponse[i].projectURI].WellKnownAttributeDefinitions["Identifier"].sameAs =
											queryResponse[i].sameAs;
							this._ProjectsProperties[queryResponse[i].projectURI].WellKnownAttributeDefinitions["Identifier"].isSystemAttribute = true;
							this._ProjectsProperties[queryResponse[i].projectURI].WellKnownAttributeDefinitions["Identifier"].attributeType = 
											com.ibm.rdm.web.model.types.RRMTypes.ATTRIBUTE_TYPES.InterType;
							break;
						case com.ibm.rdm.web.model.types.RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.AlternateSpelling:
							this._ProjectsProperties[queryResponse[i].projectURI].WellKnownAttributeDefinitions["AlternateSpelling"] = {};
							this._ProjectsProperties[queryResponse[i].projectURI].WellKnownAttributeDefinitions["AlternateSpelling"].uri = 
											queryResponse[i].resourceURI;
							this._ProjectsProperties[queryResponse[i].projectURI].WellKnownAttributeDefinitions["AlternateSpelling"].sameAs =
											queryResponse[i].sameAs;
							this._ProjectsProperties[queryResponse[i].projectURI].WellKnownAttributeDefinitions["AlternateSpelling"].attributeType = 
											com.ibm.rdm.web.model.types.RRMTypes.ATTRIBUTE_TYPES.StringType;
							break;

					}
				}
					
				this._WellKnownAttributeTypes = 
					{
						"int":					XmlNamespaces.XML.uri + "int",
						"float":				XmlNamespaces.XML.uri + "float",
						"boolean":				XmlNamespaces.XML.uri + "boolean",
						"date":					XmlNamespaces.XML.uri + "date",
						"time":					XmlNamespaces.XML.uri + "time",
						"dateTime":				XmlNamespaces.XML.uri + "dateTime",
						"string":				XmlNamespaces.XML.uri + "string",
						"duration":				XmlNamespaces.XML.uri + "duration",
						"user":					XmlNamespaces.RRS30Types.uri + 'UserAttributeType'
					};
						
				this._WellKnownRangeAttributeTypes = 
				{
					"int":					XmlNamespaces.XML.uri + "int",
					"float":				XmlNamespaces.XML.uri + "float",
					"date":					XmlNamespaces.XML.uri + "date",					
					"dateTime":				XmlNamespaces.XML.uri + "dateTime"	
				};
				
				this._WellKnownSameAsAttributeDefinitions = 
				[
					XmlNamespaces.RRS30Types.uri + "PrimaryText",
					XmlNamespaces.RRS30Types.uri + "ArtifactFormat",
					XmlNamespaces.RRS30Types.uri + "AlternateSpelling",
					XmlNamespaces.DcTerms.uri + "title",
					XmlNamespaces.DcTerms.uri + "description",
					XmlNamespaces.DcTerms.uri + "creator",
					XmlNamespaces.DcTerms.uri + "created",
					XmlNamespaces.DcTerms.uri + "contributor",
					XmlNamespaces.DcTerms.uri + "modified",
					XmlNamespaces.DcTerms.uri + "identifier"
				];
				this._WellKnownSameAsAttributeTypes =
				[
					XmlNamespaces.RRS30Types.uri + 'ArtifactFormats',
					XmlNamespaces.RRS30Types.uri + 'UserAttributeType',
					XmlNamespaces.XML.uri + "int",
					XmlNamespaces.XML.uri + "float",
					XmlNamespaces.XML.uri + "boolean",
					XmlNamespaces.XML.uri + "date",
					XmlNamespaces.XML.uri + "time",
					XmlNamespaces.XML.uri + "dateTime",
					XmlNamespaces.XML.uri + "string",
					XmlNamespaces.XML.uri + "duration"
				];
				this._WellKnownSameAsLinkTypes =
				[
					"http://www.ibm.com/xmlns/rdm/types/Embedding",
					"http://open-services.net/ns/rm#implementedBy",
					"http://open-services.net/ns/rm#affectedBy",
					"http://open-services.net/ns/rm#specifies",
					"http://open-services.net/ns/rm#validatedBy",
					"http://open-services.net/ns/rm#trackedBy",
					"http://www.ibm.com/xmlns/rdm/types/Decomposition",
					"http://purl.org/dc/terms/references",
					"http://www.ibm.com/xmlns/rdm/types/SynonymLink",
					"http://open-services.net/ns/rm#elaborates",
					"http://www.ibm.com/xmlns/rdm/types/ArtifactTermReferenceLink",
					"http://www.ibm.com/xmlns/rdm/types/Link",
					"http://www.ibm.com/xmlns/rdm/types/Extraction",
					"http://jazz.net/ns/dm/linktypes#derives",
					"http://open-services.net/ns/rm#elaboratedBy",
					"http://open-services.net/ns/rm#specifiedBy"
				];
	},

   
	///////////////////////////////////////////////////////////////////////////////////
	// API
	
	getArtifactFormatURI: function(projectURI) {
		this._verify(projectURI);
		return this._ProjectsProperties[projectURI].ArtifactFormat;
	},
	
	getPrimaryTextURI: function(projectURI) {
		this._verify(projectURI);
		return this._ProjectsProperties[projectURI].PrimaryText;
	},	
	
	getArtifactTypeDisplayName: function(projectURI, artifactType) {
		this._verify(projectURI);
		var artifactFormats = this.getArtifactFormats(projectURI);
		for(var key in artifactFormats)
		{
			if (artifactFormats[key] == artifactType){
				return key;
			}
		}
	},
	
	getWellKnownAttributeTypes: function() {
		return this._WellKnownAttributeTypes;
	},
	
	getWellKnownRangeAttributeTypes: function() {
		return this._WellKnownRangeAttributeTypes;
	},
	
	getArtifactFormats: function(projectURI) {
		if (this._ProjectsProperties && this._ProjectsProperties[projectURI]){
			return this._ProjectsProperties[projectURI].ArtifactFormats;
		}
		else{
			return null;
		}
	},
	
	getArtifactFormatsURI: function(projectURI) {
		this._verify(projectURI);
		return this._ProjectsProperties[projectURI].ArtifactFormatsURI;
	},
	
	isSameAsSystemAttribute: function(sameAsURI) {
		for (var i = 0; i < this._WellKnownSameAsAttributeDefinitions.length; i++) {
			if (this._WellKnownSameAsAttributeDefinitions[i] == sameAsURI) {
				return true;
			}
		}
		return false;
	},
	
	isSameAsSystemAttributeType: function(sameAsURI) {
		for (var i = 0; i < this._WellKnownSameAsAttributeTypes.length; i++) {
			if (this._WellKnownSameAsAttributeTypes[i] == sameAsURI) {
				return true;
			}
		}
		return false;
	},
	
	isSameAsLinkType: function(sameAsURI) {
		for (var i = 0; i < this._WellKnownSameAsLinkTypes.length; i++) {
			if (this._WellKnownSameAsLinkTypes[i] == sameAsURI) {
				return true;
			}
		}
		return false;
	},
	
	isSystemAttribute: function(attributeURI, projectURI) {
		this._verify(projectURI);
		for (var systemAttribute in this._ProjectsProperties[projectURI].WellKnownAttributeDefinitions) {
			if (this._ProjectsProperties[projectURI].WellKnownAttributeDefinitions[systemAttribute].uri == attributeURI && this._ProjectsProperties[projectURI].WellKnownAttributeDefinitions[systemAttribute].isSystemAttribute) {
				return true;
			}
		}
		return false;
	},
	
	isWellKnownAttributeDefinition: function(attributeURI, projectURI) {
		this._verify(projectURI);
		for (var wellKnownAttributeDefinition in this._ProjectsProperties[projectURI].WellKnownAttributeDefinitions) {
			if (this._ProjectsProperties[projectURI].WellKnownAttributeDefinitions[wellKnownAttributeDefinition].uri == attributeURI) {
				return true;
			}
		}
		if (this.getPrimaryTextURI(projectURI) == attributeURI || this.getArtifactFormatURI(projectURI) == attributeURI) {
			return true;
		}
//		return false;
	},
	
	isWellKnownStringTypeAttribute: function(projectURI, wellKnownAttributeName) {
		this._verify(projectURI);
		try {
			var type = this._ProjectsProperties[projectURI].WellKnownAttributeDefinitions[wellKnownAttributeName].attributeType;
			return type === com.ibm.rdm.web.model.types.RRMTypes.ATTRIBUTE_TYPES.StringType;
		} catch (e) {
			ErrorHandler.error("Invalid wellKnownAttributeName provided", wellKnownAttributeName, e);
			throw e;
		}
	},
	
	getWellKnownAttributeURI: function(projectURI, wellKnownAttributeName) {
		this._verify(projectURI);
		try {
			var uri = this._ProjectsProperties[projectURI].WellKnownAttributeDefinitions[wellKnownAttributeName].uri;
			return uri;
		} catch (e) {
			ErrorHandler.error("Invalid wellKnownAttributeName provided", wellKnownAttributeName, e);
			throw e;
		}
	},
	
	getWellKnownAttributeObjectFromURI: function(attributeURI, projectURI) {
		this._verify(projectURI);
		for (var wellKnownAttributeDefinition in this._ProjectsProperties[projectURI].WellKnownAttributeDefinitions) {
			if (this._ProjectsProperties[projectURI].WellKnownAttributeDefinitions[wellKnownAttributeDefinition].uri == attributeURI) {
				return this._ProjectsProperties[projectURI].WellKnownAttributeDefinitions[wellKnownAttributeDefinition];
			}
		}
		return null;
	},
	
	validateSystemAttributesForProjects: function(projectsMap) {
		var validRrcProjectsMap = {};
		for (var projectURI in projectsMap) {
			for (var systemAttributeName in this.WELL_KNOWN_ATTRIBUTE_NAMES) {
				try {
					if (!this._ProjectsProperties[projectURI].WellKnownAttributeDefinitions[systemAttributeName].uri) {
						ErrorHandler.error("Invalid or corrupt Rational DOORS Next Generation project");
						break;
					}
				}
				catch (e) {
					ErrorHandler.error("Invalid or corrupt Rational DOORS Next Generation project");
					break;
				}
				validRrcProjectsMap[projectURI] = projectsMap[projectURI];
			}
		}
		return validRrcProjectsMap;
	},
	
	
	
	/////////////////////////////////// 
	/// Private Methods
	
	_verify: function(projectURI) {
		if (!projectURI) {
			ErrorHandler.error("Null ProjectURI provided");
			if (console && console.trace) {console.trace();}
			throw new Error("Null ProjectURI provided");
		} else if (!this._ProjectsProperties) {
			ErrorHandler.error("No _ProjectsProperties defined");
			if (console && console.trace) {console.trace();}
			throw new Error("No _ProjectsProperties defined");
		} else if (!this._ProjectsProperties[projectURI]) {
			ErrorHandler.error("No _ProjectsProperties for this projectURI defined", projectURI);
			if (console && console.trace) {console.trace();}
			throw new Error("No _ProjectsProperties for this projectURI defined");
		}
	}
};

})();
