/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2014. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.rdm.web.pages.About");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("dojo.date");
dojo.require("dojo.date.locale");
dojo.require("dojo.date.stamp");
dojo.require("dojo.i18n");
dojo.require("dojo.string");

dojo.require("jazz.ui.Dialog");

dojo.require("com.ibm.team.repository.web.client.init.InitializationData");
dojo.require("com.ibm.team.repository.web.client.internal.licensing");
dojo.require("com.ibm.team.repository.web.client.internal.ServerStatusClient");
dojo.require("com.ibm.team.repository.web.ui.internal.Util");
dojo.require("com.ibm.rdm.web.server.client.RDMClient");
dojo.require("com.ibm.rdm.web.model.types.XmlNamespaces");


dojo.requireLocalization("com.ibm.rdm.web.pages", "About");

(function() {

var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var InitializationData = com.ibm.team.repository.web.client.init.InitializationData;
var LicenseClient = com.ibm.team.repository.web.client.internal.licensing;
var ServerStatusClient = com.ibm.team.repository.web.client.internal.ServerStatusClient;
var Util = com.ibm.team.repository.web.ui.internal.Util;
var RDMClient = com.ibm.rdm.web.server.client.RDMClient;
var XmlNamespaces = com.ibm.rdm.web.model.types.XmlNamespaces;
var PRODUCT_NAME = "Requirements Management - Core Libraries";
var JTS_PRODUCT_ID = "com.ibm.team.jazz.foundation.server";

dojo.declare("com.ibm.rdm.web.pages.About", [dijit._Widget, dijit._Templated], {
	
	prodInfo: null, //holds product name, version, and build information.
	templatePath: dojo.moduleUrl("com.ibm.rdm.web", "pages/templates/About.html"),
	
	
	constructor: function() {
		this.messages = dojo.i18n.getLocalization("com.ibm.rdm.web.pages", "About");
		this.forUpdatesAndInfoMsg = dojo.string.substitute(this.messages.forUpdatesAndInfo, ["<a href='http://jazz.net' target='_blank'>http://jazz.net</a>"]);
		
	},
	
	postCreate: function() {
		this.rationalLogo.src = dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/images/rtl_logo_sm.gif");
		this.rationalLogo.style.marginTop = "32px";
		this.jazzLogo.src = dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/about/images/jazz-mark.png");
		this.jazzLogo.style.marginTop = "18px";
		this.eclipseLogo.src = dojo.moduleUrl("com.ibm.team.repository.web","ui/internal/about/images/BuiltOnEclipse_Logo.png");
		this.eclipseLogo.style.marginTop = "15px";
		this.javaLogo.src = dojo.moduleUrl("com.ibm.team.repository.web","ui/internal/about/images/sun-java-comp-logo.png");
		this.licenseKeyManagementLink.href = net.jazz.ajax._contextRoot + "/admin#action=com.ibm.team.repository.admin.manageLicenses";
		if (!Util.userIsAdmin()) {
			this.licenseKeyManagementDiv.style.display = "none";
		}
		//Keep track of the # of calls going out (so we know when they are all completed)
		this.callsToExecute = 4;
		
		this.prodInfo=[];
		
		//Get Rational DOORS Requirements Professional Build Info from rootservices document
		var rdmBuildDeferred = this.processRRCBuildInfo();
		rdmBuildDeferred.addCallback(this, this.getRRCBuildInfo);  
		rdmBuildDeferred.addErrback(this, this.displayErrorDialog);
		
		var srh = new ServiceResponseHandler(this, "cbGetLicenseInfo", "displayErrorDialog");
		LicenseClient.getLicenseInfo(srh);
		
		var srh2 = new ServiceResponseHandler(this, "cbGetAssignedLicenses", "displayErrorDialog");
		LicenseClient.getAssignedLicenses(srh2, {userId: com.ibm.rdm.web.AUTHENTICATED_USER.userId});
		
		var shr3 = new ServiceResponseHandler(this,"cbGetAllProductInfo","displayErrorDialog");
		ServerStatusClient.getAllProductInfo(shr3);
	},
	
	processRRCBuildInfo: function(){
		
		//Format build Id if it comes in the form <VERSION>(<BUILD ID>)
		var formatBuildId = function(buildId){		
			var hasVersionInId =buildId.indexOf('(');
			if (hasVersionInId != -1) {
				var correctedId = buildId.substring(hasVersionInId + 1, buildId.indexOf(")") );
				return correctedId;
			}else {
				return buildId;
			}
		};
		
		var formatVersionId = function(buildId) {
			var hasVersionInId = buildId.indexOf('(');
			if (hasVersionInId != -1) {
				var correctedId = buildId.substring(0, hasVersionInId);
				return correctedId;
			} else {
				return buildId;
			}
		};
				
		var deferred = RDMClient.doGet({
			  uri: com.ibm.rdm.web.ROOT_SERVICE_DOCUMENT, 
			  skipCache: true,
			  parser: function(response) {
				    var productName = PRODUCT_NAME;
				    var buildVersionEntries = com.ibm.rdm.web.util.domutils.getElementsByTagNameNS(response.doc, "version", XmlNamespaces.OSLC_RM.uri);
					var buildIdEntries = com.ibm.rdm.web.util.domutils.getElementsByTagNameNS(response.doc, "buildVersion", XmlNamespaces.OSLC_RM.uri);
					var productVersion = formatVersionId(buildIdEntries[0].firstChild.nodeValue);
					var productBuildId  = formatBuildId(buildIdEntries[0].firstChild.nodeValue);	
					var info = {
							name:productName,
							version:productVersion,
							buildId:productBuildId
					};
					return info;		
				}		  
			  });
		 
		 return deferred;
		
	},
	cbGetAllProductInfo: function(prodInfo) {
		this.callsToExecute--;
		for(var i=0; i < prodInfo.length; i++){
			this.prodInfo.push({
				name: prodInfo[i].name,
				id: prodInfo[i].id,
				version: prodInfo[i].version,
				buildId: prodInfo[i].buildId,
				patches: prodInfo[i].patches || []
			});
		}		
		this.displayAboutDialog();
	},
	
	getRRCBuildInfo: function(prodInfo){
		this.callsToExecute--;
		this.prodInfo.push(prodInfo);
		this.displayAboutDialog();
	},
	
	cbGetLicenseInfo: function(licenseInfo) {
		this.callsToExecute--;
		this.licenseInfo = licenseInfo;
		this.displayAboutDialog();
	},
	
	cbGetAssignedLicenses: function(calAssignments) {
		this.callsToExecute--;
		this.assignmentIdMap = {};
		var self = this;
		dojo.forEach(calAssignments || [], function(cal) {
			self.assignmentIdMap[cal.id] = true;
		});
		this.displayAboutDialog();
	},
	
	sortProducts: function () {	
		return this.prodInfo.sort(function(a, b){
			var aName = a.name.toLowerCase();
			var bName = b.name.toLowerCase();
			return aName > bName ? 1 : -1;
		});
	},
	
	buildProductTable: function() {
		var tbody = this.productTbody;
		function col(row, node){
		    if (node === null || node === undefined) {
		        node = "-";
		    }
		    var td = document.createElement("td");
		    row.appendChild(td);
		     if (dojo.isString(node)) {
		        node = document.createTextNode(node);
		    }
		    td.appendChild(node);
		    return td;
		}
		this.prodInfo = this.sortProducts();
		var jtsRows = [];
		dojo.forEach(this.prodInfo, function(product) {
			var row = document.createElement("tr");
			col(row, product.name);
			col(row, product.version);
			col(row, product.buildId);
			if(product.id == JTS_PRODUCT_ID) {
				jtsRows.push(row);
			} else {
				tbody.appendChild(row);
			}
			dojo.forEach(product.patches, function(patch){
				var row = dojo.create("tr");
				dojo.toggleClass(row, "patch", true);
				var td = col(row, patch.name);
				dojo.toggleClass(td, "patchName", true);
				col(row, patch.version);
				col(row, patch.buildId);
				if(product.id == JTS_PRODUCT_ID) {
					jtsRows.push(row);
				} else {
					tbody.appendChild(row);
				}
			});
		});
		//Always show JTS libraries last
		dojo.forEach(jtsRows, function(row) {
			tbody.appendChild(row);
		});	
	},
	
	buildServerTable: function() {
		var tbody = this.serverTbody;
		function col(row, node){
		    if (node === null || node === undefined) {
		        node = "-";
		    }
		    var td = document.createElement("td");
		    row.appendChild(td);
		     if (dojo.isString(node)) {
		        node = document.createTextNode(node);
		    }
		    td.appendChild(node);
		    return td;
		}
		this.prodInfo = this.sortProducts();
		dojo.forEach(this.prodInfo, function(product) {
			if(product.id == JTS_PRODUCT_ID) {
				var row = document.createElement("tr");
				col(row, product.name);
				col(row, product.version);
				col(row, product.buildId);
				tbody.appendChild(row);	
				dojo.forEach(product.patches, function(patch){			
					var row = dojo.create("tr");
					dojo.toggleClass(row, "patch", true);
					var td = col(row, patch.name);
					dojo.toggleClass(td, "patchName", true);
					col(row, patch.version);
					col(row, patch.buildId);
					tbody.appendChild(row);
				});
			}
		});
	},
	
	displayAboutDialog: function () {
		// don't execute until we have data from all remote methods
		if(this.callsToExecute > 0) {
			return;
		}

		if(this.prodInfo) {
			this.buildProductTable();
		} else {
			this.serverSection.style.display = "none";
		}
		
		var info = this.licenseInfo;		
		var serverInfo = InitializationData.get("com.ibm.team.repository.service.serverInfoInitializer");
		
		if (info.serverLicense) {
			this.buildServerTable();
		} else {
			this.buildSection.style.display = "none";
		}
		
		if (info.types) {
			this.calSection.style.display = "block";
			this.licenseKeyHeader.appendChild(document.createTextNode(dojo.string.substitute(this.messages.licenseKeyHeader, [com.ibm.rdm.web.AUTHENTICATED_USER.name]))); //getAuthenticatedContributor().name])));
			var self = this;
			dojo.forEach(info.types, dojo.hitch(this, function (cal) {
				var isAssignedToUser = self.assignmentIdMap[cal.id];
				if (isAssignedToUser) {
					// Set up the row.
					var row = document.createElement("tr"),
						product = document.createElement("td"),
						version = document.createElement("td"),
						type = document.createElement("td"),
						variant = document.createElement("td");
					
					product.appendChild(document.createTextNode(cal.productName || ""));
					version.appendChild(document.createTextNode(cal.version || ""));
					type.appendChild(document.createTextNode(cal.name || ""));
					variant.appendChild(document.createTextNode(cal.variantName || ""));
					
					row.appendChild(product);
					row.appendChild(version);
					row.appendChild(type);
					row.appendChild(variant);
					
					this.installedCALBody.appendChild(row);
				}
			}));
		}
		this.showDialog();
		
	},
	
	displayErrorDialog: function(err) {
		if(this.errorIsDiplayed) {
			return;
		}
		this.errorIsDiplayed = true;
		var container = document.createElement("div");
		container.style.padding = "1em";
		container.className = "com-ibm-rdm-web-pages-about";
		var div = document.createElement("div");
		div.appendChild(document.createTextNode(this.messages.errorFetchingInformation));
		div.className = "errorMsg";
		container.appendChild(div);
		this.dialog = new jazz.ui.Dialog({
			contentNode: container,
			primaryTitle: this.messages.aboutJazzTeamServer,
			width: "600px"
		});
	},
	
	showDialog: function () {
		this.dialog = new jazz.ui.Dialog({
			contentNode: this.contentNode,
			primaryTitle: this.dialogTitle,
			width: "800px"
		});
	}
	
});

}) ();
	
	

