/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2014. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

(function() {
dojo.provide("com.ibm.rdm.web.server.client.utils.ResourceUtil");

dojo.require("jazz.util.html");

dojo.require("com.ibm.rdm.web.server.client.RDMClient");
dojo.require("com.ibm.rdm.web.common.ErrorHandler");
dojo.require("com.ibm.rdm.web.notifications.ChangeNotifier");
dojo.require("com.ibm.rdm.web.notifications.EventTopics");
dojo.require("com.ibm.rdm.web.server.XhrUtils");
dojo.require("com.ibm.rdm.web.common.MimeTypes");
dojo.require("com.ibm.rdm.web.common.MessageForm");
//CIRC: dojo.require("com.ibm.rdm.web.model.resources.ResourceManager");
dojo.require("com.ibm.rdm.web.util.queryutils");
dojo.require("com.ibm.rdm.web.util.domutils");
dojo.require("com.ibm.rdm.web.parsers.xml2json");
dojo.require("com.ibm.rdm.web.project.TaskProgressNotifier");
dojo.require("com.ibm.rdm.web.server.client.utils.LinkUtils");
dojo.require("com.ibm.rdm.web.links.LinkTypeUtils");
dojo.require("com.ibm.rdm.web.calm.CalmLinkUtils");
dojo.require("com.ibm.rdm.web.common.StatusDisplay");
dojo.require("com.ibm.rdm.web.ext.ArtifactExtension");
dojo.require("com.ibm.rdm.web.server.client.TaskTracker");
dojo.require("com.ibm.rdm.web.server.Headers");
dojo.require("com.ibm.rdm.web.preferences.UserPreferenceNames");
dojo.require("com.ibm.rdm.web.locks.LockManager");
dojo.require("com.ibm.rdm.web.common.MessageHandler");
dojo.require("com.ibm.rdm.web.model.types.RRMTypes");
dojo.require("com.ibm.rdm.web.common.TaskTracker");
dojo.require("com.ibm.rdm.web.common.UriHelper");
dojo.require("com.ibm.rdm.web.common.ProjectCache");
// CIRC dojo.require("com.ibm.rdm.web.server.client.utils.ArtifactTemplateUtil");
dojo.require("com.ibm.rdm.web.util.storage");
dojo.require("com.ibm.rdm.web.artifactgrid.ArtifactConstants");

dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.rdm.web.server.client.utils", "ResourceUtil");


var ChangeNotifier = com.ibm.rdm.web.notifications.ChangeNotifier;
var RDMClient = com.ibm.rdm.web.server.client.RDMClient;
var ErrorHandler = com.ibm.rdm.web.common.ErrorHandler;
var XhrUtils = com.ibm.rdm.web.server.XhrUtils;
//var ResourceManager = com.ibm.rdm.web.model.resources.ResourceManager;
var queryutils = com.ibm.rdm.web.util.queryutils;
var domutils = com.ibm.rdm.web.util.domutils;
var MimeTypes = com.ibm.rdm.web.common.MimeTypes;
var TaskProgressNotifier = com.ibm.rdm.web.project.TaskProgressNotifier;
var LinkUtils = com.ibm.rdm.web.server.client.utils.LinkUtils;
var LinkTypeUtils = com.ibm.rdm.web.links.LinkTypeUtils;
var CalmLinkUtils = com.ibm.rdm.web.calm.CalmLinkUtils;
var ArtifactExtension = com.ibm.rdm.web.ext.ArtifactExtension;
var TaskTracker = com.ibm.rdm.web.server.client.TaskTracker;
var Headers = com.ibm.rdm.web.server.Headers;
var RRMTypes = com.ibm.rdm.web.model.types.RRMTypes;
var UserPreferenceNames = com.ibm.rdm.web.preferences.UserPreferenceNames;
var UriHelper = com.ibm.rdm.web.common.UriHelper;
var ProjectCache = com.ibm.rdm.web.common.ProjectCache;
// var ArtifactTemplateUtil = com.ibm.rdm.web.server.client.utils.ArtifactTemplateUtil;

var MAP_PROJECT_URI_TO_MAPS_OF_COMPOSITE_TYPE_KEY_TO_ARTIFACT_TEMPLATE_URI = {};
// Note: I'm adding a "2" to the end of the following cookie name due to the
// fact that in M14 the cook tuples represent mappings like this:
//     artifactTypeURI -> artifactTemplateURI
// but in RC1 we need to use mappings like this:
//     artifactTypeURI/artifactFormatURI -> artifactTemplateURI
// and by boosting this number we can effectively ignore cookies produced
// by pre-RC1 installs.
var ARTIFACT_TEMPLATES_COOKIE_NAME_PREFIX =
	"com.ibm.rdm.web.server.client.utils.ResourceUtil._artifactTemplateCookie2";

var StatusDisplay = com.ibm.rdm.web.common.StatusDisplay;

com.ibm.rdm.web.server.client.utils.ResourceUtil = {
	///////////////////////////////////////////////////////////////////////////////////
	// PRIVATE FIELDS
	_messages: dojo.i18n.getLocalization("com.ibm.rdm.web.server.client.utils", "ResourceUtil"),
   
	///////////////////////////////////////////////////////////////////////////////////
	// API
	
	makeUniqueNamesAtFolder: function(projectURI, folderURI, currentNames){
		// summary:
		//		Given a FolderURI, this function will return a unique name
		//		within that folder using "Copy of {0}" or "Copy {0} of {1}"  
		// projectURI: URI
		//		The URI of the current Project.
		// folderURI: URI
		//		The URI of the folder in which to make the unique name.
		// currentNames: Array of Names
		//		The current names of the artifacts to be made unique.		
		var deferred = new dojo.Deferred();

//35107: Copying artifacts is very slow	
//		var nameQuery = ProjectCache.executeMultiFetch(this._makeUniqueNameQuery(projectURI, folderURI));
//		nameQuery.addBoth(this, function(entries){
			//Build a collection of existing entries.
			var existingNames = {};
//			for (var i in entries){
//				existingNames[entries[i].displayName] = true;
//			}
			//Now lets check and replace the names
			var len = currentNames.length;
			for (var i = 0; i < len; i++) {
				var potentialName = dojo.string.substitute(this._messages.copyOfNaming, [currentNames[i]]);
				var count = 2;
				while (existingNames[potentialName]){
					potentialName = dojo.string.substitute(this._messages.copyOfNamingN, [count, currentNames[i]]);
					count++;
				}
				currentNames[i] = potentialName;
				existingNames[potentialName] = true;
			}
			deferred.callback(currentNames);
//		});
		return deferred;
	},
	
		
	_makeUniqueNameQuery: function(projectURI, folderURI){
		// summary:
		//		Composes the Query Info needed to get the contents of a folder
		//		from the server.		
		if (!projectURI) {
			return null;
		}
		var query = null;
		var properties = 'name';
		query = 'parentFolder=<' + folderURI + '>';
		query += " and namespace in";
		query += "[<" + com.ibm.rdm.web.FRONTING_SERVICE_RESOURCES + ">]";
		var queryInfo = {
			query: query,
			properties: properties,
			projectURI: projectURI,
			extendedQueryMode: true,
			cancelDeferreds: false,
			skipCache: true,   //Because we are potentially checking uniqueness, we want to ensure this is up to date.
			disablePagination: false
		};
		return queryInfo;
	},
	
	deleteResource: function(resourceURI, projectURI, artifactName, wrappedResourceURI,
							 isInModule, isModule, moduleStructure, /*Callback?*/onDialogClose) {
		return this._performDeleteResource(
			resourceURI, projectURI, artifactName, wrappedResourceURI,
			com.ibm.rdm.web.notifications.EventTopics.RESOURCE_DELETED,
			false, false, isInModule, isModule, moduleStructure, onDialogClose
		);
	},
	
	deleteGLEResource: function(resourceURI, projectURI, artifactName, wrappedResourceURI) {
		return this._performDeleteResource(
			resourceURI, projectURI, artifactName, wrappedResourceURI,
			com.ibm.rdm.web.notifications.EventTopics.RESOURCE_DELETED,
			false, true
		);
	},
	
	deleteFolder: function(resourceURI, projectURI, artifactName) {
		return this._performDeleteResource(
			resourceURI, projectURI, artifactName, null, null, true, true
		);
	},
	
	executeDeleteResource: function(resourceURI, projectURI, wrappedResourceURI, notifyEventTopic, isFolder, 
			isInModule, isModule, associatedModuleTemplates, /*Callback?*/onDialogClose, responseDeferred, 
			deleteUnboundResource, trackerCallback) {
		var self = this;
		StatusDisplay.showStatusMessage(
				isInModule ?
						self._messages.removingArtifacts :
							self._messages.deletingArtifacts
		);
		self.deleteUnboundResource = deleteUnboundResource;
		
		var deleteAssociatedTemplates = function () {
			// this function gets called once the delete has taken place, see below.
			dojo.forEach(associatedModuleTemplates, function (graph) {
				com.ibm.rdm.web.server.client.utils.ArtifactTemplateUtil.deleteArtifactTemplate(
				graph.xmlAttributes.about,
				graph.ofType.xmlAttributes.resource, 
				graph.ArtifactFormat.xmlAttributes.resource,
				projectURI);
			});
		};
		
		var deferred = null;
		// For wrapper resources, the wrapped resource must be deleted, server will
		// automatically delete the wrapper
		var reportUnboundArtifacts = (isInModule || isModule);
		if (wrappedResourceURI) {
			deferred = self._performDeleteOperation(wrappedResourceURI, reportUnboundArtifacts);
		} else if (isFolder) {
			deferred = self._performDeleteFolder(resourceURI);
		} else {
			deferred = self._performDeleteOperation(resourceURI, reportUnboundArtifacts);
		}
		deferred.addCallback(self, function(response) {
			deleteAssociatedTemplates();
			StatusDisplay.hideStatusMessage();
			if (notifyEventTopic) {
				ChangeNotifier.notifyListeners({
					eventTopic: notifyEventTopic,
					response: response,
					resourceURI: resourceURI,
					projectURI: projectURI,
					crossWindow: true
				});
				if (resourceURI.indexOf("reviews") !== -1) {
					ChangeNotifier.notifyListeners({
						eventTopic: com.ibm.rdm.web.notifications.EventTopics.REVIEW_DELETED,
						resourceURI: resourceURI
					});
				}
			}
			// Note: The response code may be a 202 which means that the
			// caller is attempting to obtain intermediate progress
			// and will need the "Location" header found in the response.
			responseDeferred.callback(response);
			// For single artifact deletes of a module, we do not wait for the task tracker to complete before resolving
			// the deferred that we gave to our caller.  The client relies on iternal caching, while server is reindexing,
			// to to avoid the user having to wait for indexing.
			// However, for delete of a module or row in a module we must continue to monitor the tracker on the delete
			// operation so that we can learn of any base artifacts that are now unbound from all modules, and offer to
			// delete them.  
			if (reportUnboundArtifacts) {
				(new TaskTracker()).pollTracker({
					response: response,
					callback: dojo.hitch(this, trackerCallback)
				});
			}
		});
		deferred.addErrback(self, function(errorResponse) {
			StatusDisplay.hideStatusMessage();
			if (errorResponse && errorResponse.lockResponse) {
				com.ibm.rdm.web.common.MessageHandler.clearMessage();
				com.ibm.rdm.web.locks.LockManager.showLockedMessage(com.ibm.rdm.web.locks.LockManager.MSG_DELETE,
						errorResponse.lockResponse);
			} else {
				ErrorHandler.error(errorResponse);
			}
			responseDeferred.errback(errorResponse);
		});
		return deferred;
	},
	
	performUnboundDeletion: function(trackerResponse, isInModule, deleteUnboundResource, onDialogClose){
		// read info about any unbound artifacts, prompt the user and delete them. 
		var unboundArtifactURIs = [];
		this._gatherUnboundArtifactURIs(trackerResponse, unboundArtifactURIs);
		if (unboundArtifactURIs.length) {
			return this._deleteUnboundArtifacts(unboundArtifactURIs, isInModule, 1, 0, deleteUnboundResource, onDialogClose);
		}
		// _deleteUnboundArtifacts() returns a deferred that resolves when the delete operation
		// is finished or cancelled by the user.  However, we have already resolved the
		// deferred we gave to our caller and are running asynchronously, so no need to do any
		// more when the deletion has completed.
	},
	
	_performDeleteResource: function(
		resourceURI, projectURI, artifactName, wrappedResourceURI, notifyEventTopic,
		isFolder, disableConfirmationDialog, isInModule, isModule, moduleStructure, /*Callback?*/onDialogClose)
	{
		var self = this;
		var fakeDeferred = new dojo.Deferred(),
			associatedModuleTemplates = [];
		
		var confirmationDelete = dojo.hitch(this, function(deleteUnboundResource) {
			var trackerCallback = dojo.hitch(this, function(trackerResponse) {
				this.performUnboundDeletion(trackerResponse, isInModule, deleteUnboundResource, onDialogClose);
			});		
			this.executeDeleteResource(resourceURI, projectURI, wrappedResourceURI, notifyEventTopic, isFolder, isInModule, isModule, 
					associatedModuleTemplates, onDialogClose, fakeDeferred, deleteUnboundResource, trackerCallback);
		});
				
		if (!disableConfirmationDialog) {
			var showConfirm = dojo.hitch(this, function (hasAssociatedTemplate) {
				var confirmMsg;
				var dialogTitle = (isInModule) ? this._messages.confirmRemoveDialogTitle : this._messages.confirmDeleteDialogTitle;
				var confirmButtonText = (isInModule) ? this._messages.removeLabel : this._messages.deleteLabel;
				var dialogStyle = "yesNo";
				var yesButtonText = confirmButtonText;
				var noButtonText = this._messages.cancelLabel;
				var cancelButtonText;
				var dialogWidth = "350px";
				var onOkFunction = confirmationDelete;
				var onNoFunction;
				
				var escapedArtifactName = jazz.util.html.escape(artifactName);
				if (isFolder) {
					confirmMsg = dojo.string.substitute(
						this._messages.confirmDeleteFolderContent, [escapedArtifactName]
					);
				} else if (isModule && hasAssociatedTemplate) {
					confirmMsg = dojo.string.substitute(
						this._messages.confirmDeleteModuleWithTemplate, [escapedArtifactName]
					);
				}
				else if (isModule) {
					confirmMsg = dojo.string.substitute(
						this._messages.confirmDeleteModule, [escapedArtifactName]
					);
				}
				else if (isInModule) {
					if (moduleStructure.getArtifactByURI(resourceURI).children.length > 0) {
						confirmMsg = dojo.string.substitute(
								this._messages.confirmDeleteFromModule, [escapedArtifactName]
						);
					} else {
						confirmMsg = dojo.string.substitute(
								this._messages.confirmDeleteFromModuleNoChild, [escapedArtifactName]
						);
					}
					
					dialogStyle = "yesNoCancel";
					yesButtonText = this._messages.removeAndDeleteLabel;
					noButtonText = this._messages.removeLabel;
					cancelButtonText = this._messages.cancelLabel;
					dialogWidth = "450px";
					onOkFunction = function() {
						confirmationDelete(true);
					};
					onNoFunction = function() {
						confirmationDelete(false);
					};	
				}
				else {
					confirmMsg = dojo.string.substitute(
						this._messages.confirmDeleteArtifact, [escapedArtifactName]
					);
				}
				var msgForm = new com.ibm.rdm.web.common.MessageForm({
					confirmContent: confirmMsg,
					dialogTitle: dialogTitle,
					dialogStyle: dialogStyle,
					yesButtonText: 	yesButtonText,
					noButtonText:	noButtonText,
					cancelButtonText: cancelButtonText,
					dialogWidth:	dialogWidth,
					onOk: onOkFunction,
					onNo: onNoFunction,
					onDialogClose: function() {
						// resolve the callback for all ways of closing the dialog other than
						// the OK case.  Cancel button, X on dialog, Esc key.
						if (msgForm) {
							if (onNoFunction) {
								if (!msgForm.isHandlingNo() && !msgForm.isHandlingOk()) {
									fakeDeferred.callback();
								}
							}
							else {
								if (!msgForm.isHandlingOk()) {
									fakeDeferred.callback();
								}		
							}
						}						
					}
				});
				msgForm.show(onDialogClose);
			});
			
			if (isModule) {
				var associatedModuleTemplatesDeferred = this.getAssociatedModuleTemplates(resourceURI, projectURI);
				associatedModuleTemplatesDeferred.addCallback(this, function (response) {
					var parser = new com.ibm.rdm.web.parsers.xml2json(response.doc),
						parsedResponse = parser.parse();
					if (parsedResponse.RDF.Description && parsedResponse.RDF.Description.member) {
						// there is an associated template, we need to delete it.
						showConfirm(true);
						associatedModuleTemplates.push(parsedResponse.RDF.Description.member.ArtifactTemplateGraph);
					} else {
						showConfirm();
					}
				});
			} else {
				showConfirm();
			}
		}
		else {
			confirmationDelete();
		}
		return fakeDeferred;
	},
	
	deleteMultipleResources: function(artifacts, isInModule, moduleStructure) {
		//Summary: deletes multiple resources.
		// artifacts: an array of multiple resource objects, each with
		//			  resourceURI, _projectURI, displayName as properties.
		
		//Build the confirmation message
		var fakeDeferred = new dojo.Deferred();
		
		if (!artifacts || artifacts.length === 0) {
			return;
		}
		
		var modules = [], // a subset of the selection (artifacts) which is modules
		templatesToDelete = [], // a list of associated module templates to delete
		listOfDeferreds = [], // list of deferreds to check whether there is a template
		deferredList, // which we put in a deferred list.
		undeletedArtifacts = {},
		deleteAssociatedTemplates = function () {
			// Summary:
			//		This function gets called once the delete of the resource(s) has taken place.
			dojo.forEach(templatesToDelete, function (graph) {
				com.ibm.rdm.web.server.client.utils.ArtifactTemplateUtil.deleteArtifactTemplate(
					graph.xmlAttributes.about,
					graph.ofType.xmlAttributes.resource, 
					graph.ArtifactFormat.xmlAttributes.resource,
					graph.projectURI);
			});
		},		
		deleteResources = dojo.hitch(this, function(deleteUnboundResources) {
			// Handler for OK click on delete confirmation dialog.
			StatusDisplay.showStatusMessage(isInModule?this._messages.removingArtifacts:this._messages.deletingArtifacts);
					
			var resourceToWrapperMap = {};
			var deferred = this._performMultipleDeleteOperation(artifacts, isInModule, modules.length, undeletedArtifacts, resourceToWrapperMap, deleteUnboundResources);
			deferred.addCallback(this, function(response) {
				var numDeleted;
				deleteAssociatedTemplates();
				StatusDisplay.hideStatusMessage();
				
				//Check we actually deleted it
				artifacts = dojo.filter(artifacts, function (item) {
					var uri = item.resourceURI;
					if (resourceToWrapperMap[uri]) {
						uri = resourceToWrapperMap[uri];
					}
					return !undeletedArtifacts[uri];
				});
				
				numDeleted = artifacts.length;
				dojo.forEach(artifacts, function(item, idx) {
					
					ChangeNotifier.notifyListeners({
						eventTopic: com.ibm.rdm.web.notifications.EventTopics.RESOURCE_DELETED,
						response: response,
						isMultiple: true,
						deletedCount: numDeleted,
						deletedIdx: idx+1,
						resourceURI: item.resourceURI,
						projectURI : item.projectURI,
						crossWindow: true
					});
					
				});
				fakeDeferred.callback();
			});
			deferred.addErrback(this, function(errorResponse) {
				StatusDisplay.hideStatusMessage();
				ErrorHandler.error(errorResponse);
				fakeDeferred.errback();
			});
			return deferred;
		});
		
		
		var showConfirm = dojo.hitch(this, function (/*Boolean?*/moduleTemplateWarning) {
			// summary:
			//		This function pops up the "are you sure you want to do this" dialog.
			//		optional param says whether at least one of the things being deleted
			//		is the basis for a module template. 
			var tooMany = (artifacts.length > 2),
				dialogTitle = (isInModule) ? this._messages.confirmRemoveDialogTitle : this._messages.confirmDeleteDialogTitle,
				confirmButtonText = (isInModule) ? this._messages.removeLabel : this._messages.deleteLabel,
				confirmationMessage,
				dialogStyle = "yesNo",
				yesButtonText =	confirmButtonText,
				noButtonText = this._messages.cancelLabel,
				cancelButtonText,
				dialogWidth = "350px",
				onOkFunction = deleteResources,
				onNoFunction;
				
			if (moduleTemplateWarning) {
				confirmationMessage = tooMany ? this._messages.confirmMultiDeleteManyModuleWithTemplate : this._messages.confirmMultiDeleteTwoModuleWithTemplate;
			} else if (isInModule) {
				if (moduleStructure.getArtifactByURI(artifacts[0].resourceURI).children.length > 0) {
					confirmationMessage = tooMany ? this._messages.confirmDeleteManyFromModule : this._messages.confirmDeleteTwoFromModule;
				} else {
					confirmationMessage = tooMany ? this._messages.confirmDeleteManyFromModuleNoChild : this._messages.confirmDeleteTwoFromModuleNoChild;
				}
				
				dialogStyle = "yesNoCancel";
				yesButtonText = this._messages.removeAndDeleteLabel;
				noButtonText = this._messages.removeLabel;
				cancelButtonText = this._messages.cancelLabel;
				dialogWidth = "450px";
				onOkFunction = function() {
					deleteResources(true);
				};
				onNoFunction = function() {
					deleteResources(false);
				};	
			}
			else {
				 confirmationMessage = tooMany ? this._messages.confirmDeleteManyArtifacts : this._messages.confirmDeleteTwoArtifacts;
			}
			
			var msgForm = new com.ibm.rdm.web.common.MessageForm(
			{
				confirmContent: dojo.string.substitute(confirmationMessage, [artifacts[0].getDisplayTitle(), artifacts.length - 1]),
				dialogTitle: 	dialogTitle,
				dialogStyle: 	dialogStyle,
				yesButtonText: 	yesButtonText,
				noButtonText:	noButtonText,
				cancelButtonText: cancelButtonText,
				dialogWidth:	dialogWidth,
				onOk: 			onOkFunction,
				onNo:			onNoFunction
			});   
			msgForm.show();
		});

		// filter the list of artifacts to get the ones that are modules.
		modules = dojo.filter(artifacts, dojo.hitch(this, function(artifact) {
			return this.isModule(artifact.getEntityType());
		}));

		if (modules && modules.length > 0) {
			// if there are modules, we need to check whether these are the source of
			// a template...
			dojo.forEach(modules, dojo.hitch(this, function (module) {
				var resourceURI = module.resourceURI,
					projectURI = module.projectURI,
					associatedModuleTemplatesDeferred = this.getAssociatedModuleTemplates(resourceURI, projectURI);
					
				associatedModuleTemplatesDeferred.then(dojo.hitch(this, function (response) {
					var parser = new com.ibm.rdm.web.parsers.xml2json(response.doc),
						parsedResponse = parser.parse(),
						thingToDelete;
						
					if (parsedResponse.RDF.Description && parsedResponse.RDF.Description.member) {
						// this module is the source of a template, so extract from the response
						// what we will need to delete it if the user says go ahead.
						 thingToDelete = parsedResponse.RDF.Description.member.ArtifactTemplateGraph;
						 thingToDelete.projectURI = projectURI;
						// there is an associated template, we need to delete it.
						templatesToDelete.push(thingToDelete);
					} 
				}));
				
				listOfDeferreds.push(associatedModuleTemplatesDeferred);
			}));
			
			deferredList = new dojo.DeferredList(listOfDeferreds);
			deferredList.then(dojo.hitch(this, function () {
				// wait until we have checked all the modules in the selection.
				showConfirm(templatesToDelete.length > 0);
			}));

		} else {
			showConfirm();
		}

		return fakeDeferred;
	},
	
	_performDeleteOperation: function(resourceURI, /*?boolean*/ reportUnboundArtifacts) {
		// summary:
		//		Deletes the given resource.
		// reportUnboundArtifacts:
		//		If provided and true, the delete operation reports (through its tracker) any artifacts that are
		//		no longer bound to any modules as a result of the operation.
		
		if (reportUnboundArtifacts) {
			resourceURI += "?reportUnboundArtifacts=true";
		}
		
		//delete resource
		var deleteDeferred = RDMClient.doDelete({
			uri:resourceURI,
			delayCallbackUntilDataIndexed: false //allow for potential forced indexing for extensions
		});
		return deleteDeferred;
	},
	
	moveMultipleResources: function(artifacts, newFolderURI) {
		//Summary: moves multiple resources.
		// artifacts: 
		//  an array of multiple resource objects, each with
		//	resourceURI, _projectURI, displayName as properties.
		// newFolderURI:
		//	The URI of the folder to which artifacts should be moved.
		
		//Build the confirmation message
		if (!artifacts || artifacts.length === 0) {
			var fakeDeferred = new dojo.Deferred();
			fakeDeferred.callback();
			return fakeDeferred;
		}
		
		var self = this;
		var deferredArray = [];
		dojo.forEach(artifacts, function(item) {
			deferredArray.push(self.moveToFolder(item.projectURI, item.resourceURI, newFolderURI));
		});
		return new dojo.DeferredList(deferredArray);
	},
	
//	{
//		targetURI: targetURI,
//		resourceURIs: artifactUris,
//		projectURI: projectURI
//	}
	createReusedArtifacts: function(args){
		//Serially copies a number of resources
		var count = 0,
			skippedResources = false,
			resourceURIs = args.resourceURIs,
			targetURI = args.targetURI,
			projectURI = args.projectURI,
			createdResources = [],
			where = args.where,
			processCreate,
			relativeSiblingURI = args.relativeSiblingURI,
			errorHandler;

		var createReusedDeferred = new dojo.Deferred();

		//Reverse the order for AFTER so server puts in 'correct' order
		if (resourceURIs.length > 1 && where === com.ibm.rdm.web.artifactgrid.ArtifactConstants.AFTER) {
			resourceURIs = resourceURIs.reverse();
		}				
		
		//We need to chain the deferred together to create the artifacts one by one.
		var createResources = dojo.hitch(this, function(response) {
			var createDeferred = this.createReusedArtifact({
					targetURI: targetURI,
					resourceURI: resourceURIs[count],
					projectURI: projectURI,
					delayCallbackUntilDataIndexed: (count == resourceURIs.length-1),
					where: where,
					relativeSiblingURI: relativeSiblingURI
				});
			createDeferred.addCallback(this, processCreate);		
			createDeferred.addErrback(this, errorHandler);
			
			return response;
		});
		
		processCreate = dojo.hitch(this, function(response) {
			//copy complete - either loop or finish.
			count++;
			if(response.skipped) {
				// Artifact was skipped because it was a module or collection.
				skippedResources = true;
			} else {
				createdResources.push(response.ioArgs.xhr.getResponseHeader("Location"));
			}
			if (count < resourceURIs.length) {
				//round again
				createResources();
			} else {
				//finished
				if(createdResources.length > 0){
					//Reverse the order (again) for AFTER, so server and client match (by having reversed lists!)
					if (createdResources.length > 1 && where === com.ibm.rdm.web.artifactgrid.ArtifactConstants.AFTER) {
						createdResources = createdResources.reverse();
					}				
					createReusedDeferred.callback(createdResources);
				} else {
					// We've accomplished nothing.
					createReusedDeferred.errback();
					StatusDisplay.hideStatusMessage();
				}
				
				if(skippedResources){
					com.ibm.rdm.web.common.MessageHandler.showWarning(7685);
				}
			}		
			return response;
		});
		
		errorHandler = function(errorResponse) {
			ErrorHandler.error(errorResponse);

			//an error
			createReusedDeferred.errback(errorResponse);

			return errorResponse;
		};
		
		//Start the ball rolling
		createResources();
		
		return createReusedDeferred;
	},
	
//	{
//	targetURI: targetURI,
//	resourceURI: artifactUri,
//	projectURI: projectURI
//}
	createReusedArtifact: function(args){
		
		var objectXML = [],
			resourceURI = args.resourceURI,
			targetURI = args.targetURI,
			projectURI = args.projectURI,
			where = args.where,
			relativeSiblingURI = args.relativeSiblingURI;

		
		// WI: 75167, in order to figure out whether this artifact should be
		// a heading, we need to get it's type, so this function will load the resource.
		var loadResource = function() {
			return com.ibm.rdm.web.model.resources.ResourceManager.get({
				resourceURI: resourceURI
			});
		};
		
		// which we will then pass to this function to fetch the type
		var getType = function(resource) {
			return com.ibm.rdm.web.model.types.TypesManager.getObjectType({
				projectURI: args.projectURI,
				typeURI: resource.getType()
			});
		};
		
		// finally, we'll use the type from above to figure out if we need to pass the isHeading flag
		// when we create the reused artifact.
		var doCreate = function(type) {
			var format = type.getDefaultFormat();
			if(this.isModule(format) || this.isCollection(format)) {
				// Don't allow adding a module or collection to a module
				// Returning a "skipped" value so we can continue adding other resources (if we have anymore)
				return {"skipped": true};
			}
			
			var positionParam;
			
			objectXML.push('<rdf:RDF\n'+
			'xmlns:dc="http://purl.org/dc/elements/1.1/"\n'+
			'xmlns:dcterms="http://purl.org/dc/terms/"\n'+
			'xmlns:h="http://www.w3.org/TR/REC-html40"\n'+
			'xmlns:jfs="http://jazz.net/xmlns/foundation/1.0/"\n'+
			'xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"\n'+
			'xmlns:rm="http://www.ibm.com/xmlns/rdm/rdf/"\n'+
			'xmlns:rrmCollection="http://jazz.net/xmlns/alm/rm/Collection/v1.0/"\n'+
			'xmlns:xs="http://www.w3.org/2001/XMLSchema#">\n'+
			'<rm:Artifact rdf:about="">\n'+
			'<rm:children>\n'+
			'<rdf:Seq/>\n'+
			'</rm:children>');
			objectXML.push('<rm:boundArtifact rdf:resource="' + resourceURI + '"/>');
			if (type.isModuleHeading()) {
				objectXML.push('<rm:isHeading rdf:datatype="http://www.w3.org/2001/XMLSchema#boolean">true</rm:isHeading>');
			}
			
			
			objectXML.push('</rm:Artifact>\n'+
				'</rdf:RDF>');
			
			// as per new server API, we now always post to the parent
			// so if the target (as per ModuleStructure constant) is
			// before OR below then we want to use the beforeSibling
			// parameter.  Below is now "before first child". 
			// If there are no children, then the "where" parameter
			// is not supplied and so we pass up no sibling or position
			// which means "make it a child".
			if (relativeSiblingURI && where) {
				if (where === "before") {
					positionParam = "beforeSibling";
				} else {
					positionParam = "afterSibling";
				}
							
				if (targetURI.indexOf("?") === -1) {
					targetURI += "?"+positionParam+"=" + relativeSiblingURI;
				} else {
					targetURI += "&"+positionParam+"=" + relativeSiblingURI;
				}
			}
	
			return RDMClient.doPost({
				uri: targetURI,
				projectURI: projectURI,
				postData: objectXML.join(''),
				xhrArgs: {
					headers: {
						"X-com-ibm-rdm-reuse-mode": "uncontrolled",
						"Content-Type": "application/rdf+xml"
					}
				},
				delayCallbackUntilDataIndexed: args.delayCallbackUntilDataIndexed
			});
		};
				
		return loadResource().addCallback(getType).addCallback(dojo.hitch(this, doCreate));
	},
	
	
	moveToFolder:function(projectURI, resourceURI, newFolderURI)
	{
		// summary:
		//		This method moves a resource to a new location within
		//		the folder hierarchy of the current project.
		// projectURI: URI
		//		The URI of the current Project.
		// resourceURI: URI
		//		The URI of the resource to be moved.
		// newFolderURI: URI
		//		The URI of the new folder to which the resource should be moved.
		var deferred = this._performMoveOperation(projectURI, resourceURI, newFolderURI);
		deferred.addBoth(this, function(moveResponse){
			ChangeNotifier.notifyListeners({
					eventTopic: com.ibm.rdm.web.notifications.EventTopics.RESOURCE_MOVED,    				
					resourceURI: resourceURI,
					projectURI : projectURI,
					resourceUriScope: resourceURI,
					folderURI: moveResponse.ioArgs.url,
					notifyResource: moveResponse,
					xhrArgs: {
						headers: {
							"Content-Type": "application/rdf+xml"
						}
					}
				});
		});
		return deferred;
	},
	
	_performMoveOperation: function(projectURI, resourceURI, newFolderURI, delayCallbackUntilDataIndexed/*optional*/)
	{
		// summary:
		//		Perform the move of a resource to a new location within
		//		the folder hierarchy of the current project.
		// projectURI: URI
		//		The URI of the current Project.
		// resourceURI: URI
		//		The URI of the resource to be moved.
		// newFolderURI: URI
		//		The URI of the new folder to which the resource should be moved.	
		//		returns:
		//		A deferred to which callbacks can be attached to fire once the move has been
		//		performed.
		var moveToFolderXML = '<?xml version="1.0" encoding="UTF-8"?>\n' +
			'<rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#">\n' +
			'<ns:containedResources xmlns:ns="http://com.ibm.rdm/navigation#" rdf:about="">\n' +
			'<ns:containedResource rdf:resource="' + resourceURI + '" xmlns:ns="http://com.ibm.rdm/navigation#"/>\n' +
			'</ns:containedResources></rdf:RDF>';
		if (delayCallbackUntilDataIndexed === false) {
			delayCallbackUntilDataIndexed = false;
		}
		else {
			delayCallbackUntilDataIndexed = true;
		}	
		var moveToFolderDeferred = RDMClient.doPost({
			uri: newFolderURI + "?containedResources",
			projectURI: projectURI,
			postData: moveToFolderXML,
			delayCallbackUntilDataIndexed: delayCallbackUntilDataIndexed
		});
		
		return moveToFolderDeferred;
	},
	
	copyResourcesToFolder: function(copyParams) {
		// summary:
		//		This method copies resources to a new location within
		//		the folder hierarchy of the current project.
		// resources: resources
		//		The resources to be copied.
		// folderURI: URI
		//		The URI of the folder in which the copy should be made.
		// doDuplicateLinks: Boolean
		//		When set to true, the links will be copied over to the new
		//		artifact.
		// copyName: String
		//		The new name for the copied artifact
		// createLinkType: String
		//		When set, a new link should be created from original to copied artifact
		// isCancelled: function
		//		A function that checks if the copy operation has been cancelled
		// updateProgress: function
		//		A function callback to update the progress of the copy operation.
		var deferred = new dojo.Deferred();
		var names = [];
		var dialog;
		for (var i = 0; i<copyParams.resources.length; i++) {
			names.push(copyParams.resources[i].getTitle());
		}
		var newNamesDeferred = this.makeUniqueNamesAtFolder(copyParams.resources[0].getProjectURI(), copyParams.folderURI, names);
		newNamesDeferred.addBoth(this, function(newNames){
			var copyOperations = [];
			dojo.forEach(copyParams.resources, function(item, count) {
				var contentType = null,
					objectType = null,
					wrappedResourceURI = null;

				if (item.getEntityType() === MimeTypes.getArtifactFormat(MimeTypes.WRAPPER_RESOURCE_NAME, item.getProjectURI())) {
					contentType = MimeTypes.getArtifactFormat(MimeTypes.WRAPPER_RESOURCE_NAME, item.getProjectURI());
					wrappedResourceURI = item.getWrappedResourceURI();
					var objectTypes = item.getTypes();
					for (var type in objectTypes) {
						objectType = type;
						break;
					}
				}
				else {
					contentType = item.getEntityType();
				}
				copyOperations.push({
					resourceURI: item.getResourceURI(),
					projectSnapshotURI: item.getSnapshotURI(),
					projectURI: item.getProjectURI(),
					fromArtifactName: item.getTitle(),
					fromArtifactIdentifier: item.getID(),
					copiedName: newNames[count],
					folderURI: copyParams.folderURI,
					revision: item.getRevision(),
					reviewURI: null,
					contentType: contentType,
					wrappedResourceURI: wrappedResourceURI,
					objectType: objectType,
					delayCallbackUntilDataIndexed: false,
					doDuplicateLinks: copyParams.doDuplicateLinks,
					moduleCopyPolicy: copyParams.moduleCopyPolicy,
					createLinkType: copyParams.createLinkType
				});
			}, this);
			if(copyOperations.length === 1) {
				this.copyToFolder(copyOperations[0]).then(
					function(){
						deferred.callback();
					},
					function(response){
						deferred.errback(response);
					});
			} else {
				this.copyMultipleToFolder(copyOperations, 
					copyParams.isCancelled, 
					copyParams.updateProgress).then(
					function(){
						deferred.callback();
					},
					function(response){
						deferred.errback(response);
					});
			}
		});
		return deferred;
	},
	
	copyToFolder: function(copyParams){  
		// summary:
		//		This method copies a resource with the given name to a new location within
		//		the folder hierarchy of the current project.
		// resourceURI: URI
		//		The URI of the resource to be copied.
		// projectSnapshotURI: URI
		//		The URI of the current project snapshot. Null if there isn't one.
		// projectURI: URI
		//		The URI of the current project.
		// copiedName: String
		//		The name of the copy to be created.
		// fromArtifactName: String
		//		The name of the artifact that is to be copied.
		// fromArtifactIdentifier: String
		//		The artifact ID for the artifact that is to be copied..
		// folderURI: URI
		//		The URI of the folder in which the copy should be made.
		// revision: String
		//		The revision of the resource being created.
		// reviewURI: URI
		//		The URI of the review. Null if no review exists.
		// doDuplicateLinks: Boolean
		//		When set to true, the links will be copied over to the new
		//		artifact.
		// createLinkType: String
		//		When set, a new link should be created from original to copied artifact
		// contentType: String
		//		The Content Type of the Artifact to be copied.

		var deferred = null;

		if (this.isModule(copyParams.contentType)) {
			return this._performModuleCopy(copyParams);
		} 
		
		switch(copyParams.contentType)
		{
			case MimeTypes.getArtifactFormat(MimeTypes.WRAPPER_RESOURCE_NAME, copyParams.projectURI):
				deferred = this._performWrapperResourceCopy(copyParams);
				break;
			default:
				deferred = this._performNewRequirementCopy(copyParams);
		}		
		return deferred;
	},
	
	isModule: function(contentType) {
		// summary: 
		//		Is the supplied contentType a module.
		//		This must be performed in several other places 
		//		throughout the code...
		var url = new dojo._Url(contentType);
			
		return (url.fragment && url.fragment === MimeTypes.MODULE_NAME);
						
	},
	
	isCollection: function(contentType) {
		// summary: 
		//		Is the supplied contentType a collection.
		//		This must be performed in several other places 
		//		throughout the code...
		var url = new dojo._Url(contentType);
			
		return (url.fragment && url.fragment === MimeTypes.COLLECTION_NAME);
						
	},
	
	_performModuleCopy: function(copyParams) {
		// Sorry about this... The new TaskTracker creates a whole nest of 
		// dojo.hitch logic that isn't even worth the hassle
		var self = this; 
		var doneCopyDeferred = new dojo.Deferred();

		var failFastHasOccurred = false;
		var artifactWasCreated = false;
		var onCopyFailed = function(){
			if (!failFastHasOccurred) {
				failFastHasOccurred = true;
				doneCopyDeferred.errback({
					copyIndex: 1,
					artifactWasCreated: artifactWasCreated,
					numArtifactsRequestedForCopying: 1,
					artifactInfoForArtifactThatCouldNotBeCompletelyCopied: {
						artifactName: copyParams.fromArtifactName,
						artifactIdentifier: copyParams.fromArtifactIdentifier
					}
				});
			}
		};
		var onCopySucceeded = function(moduleResourceURI){
			doneCopyDeferred.callback(moduleResourceURI);
		};
		
		// 1st we need to load the module resource into the right kind of object
		com.ibm.rdm.web.model.resources.ResourceManager.get({
			resourceURI: copyParams.resourceURI,
			revision: copyParams.revision,
			projectSnapshotURI: copyParams.projectSnapshotURI,
			reviewURI: copyParams.reviewURI
		}).then(
			function(moduleResource){
				
				var onCopyPolicyUpdateSucceeded = dojo.hitch(this, function(){
					// TODO, we could bundle this all up inside the resource itself.
					var headers = {},
						copyDeferred;
					
					var postData = "targetFolder="+encodeURIComponent(copyParams.folderURI);				 
					postData += "&title="+encodeURIComponent(copyParams.copiedName);
					postData += "&copyLinks="+ (copyParams.doDuplicateLinks ? "true" : "false");
					
					var copyPolicyUrl = moduleResource.getCopyPolicy().about;
					
					//For copying from a baseline, the snapshot URI needs to be on the URL itslef, not in the postdata.
					if (copyParams.projectSnapshotURI) {
						copyPolicyUrl += "?baseline="+encodeURIComponent(copyParams.projectSnapshotURI);				 
					}
					headers[Headers.CONTENT_TYPE] = MimeTypes.CONTENT_TYPE_FORM_URLENCODED;
					RDMClient.doPost({
						uri: copyPolicyUrl,
						projectURI: copyParams.projectURI,
						postData: postData,
						xhrArgs: {
							headers: headers
						}
					}).then(
						function(response){	
							new com.ibm.rdm.web.common.TaskTracker({
								location: XhrUtils.getResponseLocation(response),
								onProgress: function(){
								},
								onComplete: function(severity, rdfJson){
									if (dojo.getObject("RDF.Description.result.severity.xmlAttributes.resource", false, rdfJson)
										=== com.ibm.rdm.web.common.TaskTracker.SEVERITY.ERROR
										) {
										onCopyFailed();
									} else {
										artifactWasCreated = true;
										onCopySucceeded(moduleResource.resourceURI);
										var copiedModuleURI = rdfJson.RDF.Description.taskResult.xmlAttributes.resource;
										com.ibm.rdm.web.model.resources.ResourceManager.get({
											resourceURI: copiedModuleURI
										}).then(
											function(copiedResource){
												if(copyParams.createLinkType) {
													self._linkCopied(
														copyParams.createLinkType,
														copiedResource.getResourceURI(),
														copyParams.resourceURI,
														copyParams.projectURI
													);
												}
												ChangeNotifier.notifyListeners({
													eventTopic: com.ibm.rdm.web.notifications.EventTopics.RESOURCE_COPIED,
													resourceURI: copiedResource.getResourceURI(),
													projectURI: copyParams.projectURI,
													resourceUriScope: copyParams.resourceUriScope,
													resourceObject: copiedResource
												});
											}
										);
									}
								},
								onError: function(){
									onCopyFailed();
								}
							});
						},
						function(){
							onCopyFailed();
						}
					);
				});
				
				if (moduleResource.getCopyPolicy().copyOnReuseFilter !== copyParams.moduleCopyPolicy) {
					// exposing the copy policy, which we need to now set..
					moduleResource.setCopyPolicyReuseFilter(copyParams.moduleCopyPolicy);
					
					// PUT that back
					moduleResource.save().then(
						function(){
							onCopyPolicyUpdateSucceeded();
						},
						function(){
							onCopyFailed();
						}
					);
				} else {
					onCopyPolicyUpdateSucceeded();
				}
			},
			function(){
				onCopyFailed();
			}
		);
		
		return doneCopyDeferred;
	},
	
	copyMultipleToFolder: function(
		/*array*/artifacts,
		/*Optional Function*/isCancelled,
		/*Optional Function*/updateCopyProgress)
	{
		//Serially copies a number of resources
		var self = this;
		var count = 0;
		
		var copyMultipleDeferred = new dojo.Deferred();
		var processCopy, errorHandler;
		
		//We need to chain the deferred together to copy the artifacts one by one.
		if (updateCopyProgress){
			updateCopyProgress((count+1), artifacts.length);
		}
		var copyResources = function(response) {
			if (!isCancelled || !isCancelled()){
				var toCopy = artifacts[count],
					copyDeferred = self.copyToFolder(toCopy);
				copyDeferred.addCallback(this, processCopy);
				copyDeferred.addErrback(this, errorHandler);
			}
			else {
				count = artifacts.length;
				processCopy(response);
			}
			return response;
		};
		
		processCopy = function(response) {
			//copy complete - either loop or finish.
			count++;
			if (count < artifacts.length) {
				//round again
				if (updateCopyProgress){
					updateCopyProgress((count+1), artifacts.length);
				}
				copyResources();
			} else {
				//finished
				copyMultipleDeferred.callback(response);
			}
			return response;
		};
		
		errorHandler = function(errorResponse) {
			var copyIndex = errorResponse.artifactWasCreated ? count+1 : count;
			copyMultipleDeferred.errback({
				copyIndex: copyIndex,
				numArtifactsRequestedForCopying: artifacts.length,
				artifactInfoForArtifactThatCouldNotBeCompletelyCopied: {
					artifactName: artifacts[count].fromArtifactName,
					artifactIdentifier: artifacts[count].fromArtifactIdentifier
				}
			});
			return errorResponse;
		};
		
		//Start the ball rolling
		copyResources();
		
		return copyMultipleDeferred;
	},
	
	_performWrapperResourceCopy: function(copyParams) {
		// summary:
		//		This method performs the copy of a resource with the given name to a new location within
		//		the folder hierarchy of the current project.
		// resourceURI: URI
		//		The URI of the resource to be copied.
		// projectSnapshotURI: URI
		//		The URI of the current project snapshot. Null if there isn't one.
		// projectURI: URI
		//		The URI of the current project.
		// copiedName: String
		//		The name of the copy to be created.
		// fromArtifactName: String
		//		The name of the artifact that is to be copied.
		// fromArtifactIdentifier: String
		//		The artifact ID for the artifact that is to be copied..
		// folderURI: URI
		//		The URI of the folder in which the copy should be made.
		// revision: String
		//		The revision of the resource being created.
		// reviewURI: URI
		//		The URI of the review. Null if no review exists.
		// contentType: String
		//		The Content Type of the Artifact to be copied.		
		// doDuplicateLinks: Boolean
		//		When set to true, the links will be copied over to the new
		//		artifact.
		// createLinkType: String
		//		When set, a new link should be created from original to copied artifact
		// returns:
		//		A deferred to which callbacks can be attached when the copy is completed.
		var deferred = new dojo.Deferred();
		var postURI = com.ibm.rdm.web.FRONTING_SERVICE_WRAPPED_RESOURCES
					+ "?sourceResourceUri="	+ encodeURIComponent(copyParams.wrappedResourceURI)
					+ "&artifactType="		+ encodeURIComponent(copyParams.objectType); // needed for checking permissions
		
		var failFastHasOccurred = false;
		var artifactWasCreated = false;
		var onCopyFailed = function(){
			if (!failFastHasOccurred) {
				failFastHasOccurred = true;
				deferred.errback({
					copyIndex: 1,
					artifactWasCreated: artifactWasCreated,
					numArtifactsRequestedForCopying: 1,
					artifactInfoForArtifactThatCouldNotBeCompletelyCopied: {
						artifactName: copyParams.fromArtifactName,
						artifactIdentifier: copyParams.fromArtifactIdentifier
					}
				});
			}
		};
		var onCopySucceeded = function(savedResource){
			//Notify with the save response...
			ChangeNotifier.notifyListeners({
				eventTopic: com.ibm.rdm.web.notifications.EventTopics.RESOURCE_COPIED,
				resourceURI: savedResource.getResourceURI(),
				projectURI : copyParams.projectURI,
				resourceUriScope: copyParams.resourceUriScope,
				resourceObject: savedResource
			});
			deferred.callback(savedResource.getResourceURI());	    
		};
		// Note by SPC on 10/10/2012: I made an attempt at changing this to not
		// be two steps:
		// 1) POST to create wrapped resource based on original binary content
		// 2) POST to move newly created wrapped resource to the target folder
		// to instead be just one step:
		// 1) POST to create a wrapped resource in a folder based on the
		//    original binary content
		// Unfortunately, however, after talking with Erik Craig it appears that
		// this is not possible with the existing "POST to folder API" ... so
		// I'm leaving this as a two step process (unlike all our other artifact
		// types where I've changed this copy behavior to do these two steps in
		// a single step).
		var copyDeferred = RDMClient.doPost({
			uri: postURI,
			projectURI: copyParams.projectURI,
			delayCallbackUntilDataIndexed: copyParams.delayCallbackUntilDataIndexed
		});
		copyDeferred.addCallback(this, function(response) {
			artifactWasCreated = true;
			var locationURI = response.ioArgs.xhr.getResponseHeader("Location");
			var rdf = response.doc;
			var deferredArray = [];
			var moveDeferred = 
				// Note: We're passing 'true' as the last param because we will
				// next perform a ResourceManager.get, and this call assumes
				// that the resource to be gotten has already been indexed.
				this._performMoveOperation(
					copyParams.projectURI, locationURI, copyParams.folderURI, true
				);
			moveDeferred.addErrback(this, function(response){
				onCopyFailed();
				return response;
			});
			deferredArray.push(moveDeferred);
			if (copyParams.doDuplicateLinks) {
				var copyLinksDeferred =
					this._copyLinks(
						locationURI, copyParams.resourceURI, copyParams.projectURI
					);
				copyLinksDeferred.addErrback(this, function(response){
					onCopyFailed();
					return response;
				});
				deferredArray.push(copyLinksDeferred);
			}
			if (copyParams.createLinkType) {
				var linkCopiedDeferred = 
					this._linkCopied(
						copyParams.createLinkType,
						locationURI,
						copyParams.resourceURI,
						copyParams.projectURI
					);
				linkCopiedDeferred.addErrback(this, function(response){
					onCopyFailed();
					return response;
				});
				deferredArray.push(linkCopiedDeferred);
			}
			
			var deferredList = new dojo.DeferredList(deferredArray);
			deferredList.addCallback(this, function(response) {
				var originalResourceDeferred = com.ibm.rdm.web.model.resources.ResourceManager.get({
					resourceURI: copyParams.resourceURI,
					revision: copyParams.revision,
					projectSnapshotURI: copyParams.snapshotURI,
					reviewURI: copyParams.reviewURI
				});
				originalResourceDeferred.addCallback(this, function(originalResource) {
					var resourceDeferred = com.ibm.rdm.web.model.resources.ResourceManager.get({
						resourceURI: locationURI
					});
					resourceDeferred.addCallback(this, function(resource) {
						var newlyCreatedArtifactETag = resource.getETag();
						resource.loadFromXml(rdf);
						originalResource.clone(resource);
						resource.setETag(newlyCreatedArtifactETag);
						resource.setTitle(copyParams.copiedName);
						resource.addType(copyParams.objectType);
						var saveResourceDeferred = resource.save({
							refreshAfterSave: true,
							softRefresh: true,
							forceDelayCallback: true
						}).then(
							function(savedResource){
								onCopySucceeded(savedResource);
							},
							function(){
								onCopyFailed();
							}
						);
					});
					resourceDeferred.addErrback(this, onCopyFailed);
				});
				originalResourceDeferred.addErrback(this, onCopyFailed);
			});
			deferredList.addErrback(this, onCopyFailed);
		});
		copyDeferred.addErrback(this, onCopyFailed);
		return deferred;
	},

	_performNewRequirementCopy: function(copyParams){
		// summary:
		//		This method performs the copy of a resource with the given name
		//		to a new location within the folder hierarchy of the current
		//		project.
		// resourceURI: URI
		//		The URI of the resource to be copied.
		// projectSnapshotURI: URI
		//		The URI of the current project snapshot. Null if there isn't one.
		// projectURI: URI
		//		The URI of the current project.
		// copiedName: String
		//		The name of the copy to be created.
		// fromArtifactName: String
		//		The name of the artifact that is to be copied.
		// fromArtifactIdentifier: String
		//		The artifact ID for the artifact that is to be copied..
		// folderURI: URI
		//		The URI of the folder in which the copy should be made.
		// revision: String
		//		The revision of the resource being created.
		// reviewURI: URI
		//		The URI of the review. Null if no review exists.
		// contentType: String
		//		The Content Type of the Artifact to be copied.
		// doDuplicateLinks: Boolean
		//		When set to true, the links will be copied over to the new
		//		artifact.
		// createLinkType: String
		//		When set, a new link should be created from original to copied artifact
		// returns:
		//		A deferred to which callbacks can be attached when the copy is completed.

		var deferred = new dojo.Deferred();
		var artifactWasCreated = false;
		var onCopyFailed = function(){
			deferred.errback({
				copyIndex: 1,
				artifactWasCreated: artifactWasCreated,
				numArtifactsRequestedForCopying: 1,
				artifactInfoForArtifactThatCouldNotBeCompletelyCopied: {
					artifactName: copyParams.fromArtifactName,
					artifactIdentifier: copyParams.fromArtifactIdentifier
				}
			});
		};
		var onCopySucceeded = function(savedResource){
			//Notify with the save response...
			ChangeNotifier.notifyListeners({
				eventTopic: com.ibm.rdm.web.notifications.EventTopics.RESOURCE_COPIED,
				resourceURI: savedResource.getResourceURI(),
				projectURI: copyParams.projectURI,
				resourceUriScope: copyParams.resourceUriScope,
				resourceObject: savedResource
			});
			deferred.callback(savedResource.getResourceURI());
		};
		
		if (copyParams.delayCallbackUntilDataIndexed === false) {
			copyParams.delayCallbackUntilDataIndexed = false;
		}
		else {
			copyParams.delayCallbackUntilDataIndexed = true;
		}
	   
		var createCopy = function(loadedResource) {
			//Creates a copy from the existing resource and saves it
			//with a new name.
			var copiedObject = com.ibm.rdm.web.model.resources.ResourceManager.copy(loadedResource);
			copiedObject.setTitle(copyParams.copiedName);
			copiedObject.save({
				refreshAfterSave: true,
				softRefresh: true,
				createInFolderURI: copyParams.folderURI
			}).then(
				dojo.hitch(this, function(savedResource){
					artifactWasCreated = true;
					// The artifact has been copied ... now try to copy the links.
					if (copyParams.doDuplicateLinks) {
						this._copyLinks(
							savedResource.getResourceURI(),
							copyParams.resourceURI,
							copyParams.projectURI
						).then(
							dojo.hitch(this, function(){
								if (copyParams.createLinkType) {
									this._linkCopied(
										copyParams.createLinkType,
										savedResource.getResourceURI(),
										copyParams.resourceURI,
										copyParams.projectURI
									).then(
										function(){ onCopySucceeded(savedResource);	},
										function(){	onCopyFailed();	}
									);
								} else {
									onCopySucceeded(savedResource);
								}
							}),
							function(){
								onCopyFailed();
							}
						);
					} else if (copyParams.createLinkType) {
						this._linkCopied(
							copyParams.createLinkType,
							savedResource.getResourceURI(),
							copyParams.resourceURI,
							copyParams.projectURI
						).then(
							function(){ onCopySucceeded(savedResource);	},
							function(){	onCopyFailed();	}
						);
					} else {
						onCopySucceeded(savedResource);
					}
				}),
				function(){
					onCopyFailed();
				}
			);
		};
		
		com.ibm.rdm.web.model.resources.ResourceManager.get(
			{
				resourceURI: copyParams.resourceURI,
				revision: copyParams.revision,
				projectSnapshotURI: copyParams.snapshotURI,
				reviewURI: copyParams.reviewURI
			},
			dojo.hitch(this, createCopy)
		);
		return deferred;
	},

	_linkCopied: function(linkTypeURI, toResourceURI, fromResourceURI, projectURI) {
		var isObject = linkTypeURI.indexOf("/object") !== -1;
		var linkTypeURI = linkTypeURI.substring(0, linkTypeURI.lastIndexOf("/"));
		var link = new com.ibm.rdm.web.links.Link({
			object: (isObject) ? fromResourceURI : toResourceURI,
			subject: (isObject) ? toResourceURI : fromResourceURI,
			linkType: linkTypeURI
		});
		return LinkUtils.saveLink(
			projectURI, link, false, true
		);
	},
	
	_copyLinks: function(toResourceURI, fromResourceURI, projectURI) {
		// summary:
		//		Copy the links from fromResourceURI to the newly created
		//		toResourceURI.
		// description:
		//		Returns a deferred that will be called back when the
		//		toResourceURI contains all of the newly copied links.
		var deferred = new dojo.Deferred();
		var existingLinksDeferred =
			LinkUtils.getLinksForResource({
				sourceAndTargetURI: fromResourceURI,
				projectURI: projectURI,
				skipCache: true
			});
		existingLinksDeferred.addCallback(this, function(args) {
			var links = args.resourceInfos;
			var linkTypesDeferred = LinkTypeUtils.getLinkTypesForProject(projectURI);
			linkTypesDeferred.addCallback(this, function(linkTypes) {
				var failFastHasOccurred = false;
				// Create a linksToCreate array and fill it with copies of the links.
				var linksToCreate = [];
				var calmLinksToCreate = [];
				var saveDeferreds = [];
				for (var i = 0; i < links.length; i++) {
					var link = links[i];
					var copyLink = true;
					// only copy content links that originate from this resource
					if (link.isElementLink()) {
						copyLink = false;
						var isSubject = link.getSubject() == fromResourceURI;  
						if (isSubject) {
							if (link.isSubjectElementLink()) {
								copyLink = true;
							}
						} else {
							if (link.isObjectElementLink()) {
								copyLink = true;
							} 
						}
					}
					if (copyLink) {
						var linkValue = link.getValue();
						var linkTitle = link.getTitle();
						var linkSubject = (
							link.getSubject() === fromResourceURI ?
							toResourceURI :
							link.getSubject()
						);
						var linkObject = (
							link.getObject() === fromResourceURI ?
							toResourceURI :
							link.getObject()
						);
						var link = new com.ibm.rdm.web.links.Link({
							value: linkValue ? linkValue : "",
							subject: linkSubject,
							object: linkObject,
							linkType: link.getLinkType(),
							rationale: link.getRationale(),
							title: linkTitle ? linkTitle : "",
							subjectLocationId: link.getSubjectLocationId(),
							subjectDescription: link.getSubjectDescription(),
							objectLocationId: link.getObjectLocationId(),
							objectDescription: link.getObjectDescription()
						});
						var linkType = null;
						for (var x = 0; x < linkTypes.length; x++) {
							if (link.getLinkType() == linkTypes[x].getResourceURI()) {
								linkType = linkTypes[x];
								break;
							}
						}
						if (linkType != null && LinkTypeUtils.isCalmLink(linkType.getSameAs())) {
							calmLinksToCreate.push(link);
						} else {
							linksToCreate.push(link);
						}
					}
				}
				// Ask the server to persist these new linksToCreate.
				// Note: In this saveLinks call we pass true as the last argument
				// as we do NOT want to send out unnecessary notifications.  Things
				// are slow enough already, due to the fact the server really
				// should be supporting things like "artifact copy" so that we
				// don't have to do this client side ...).
				// Also, eventually there WILL be a "copy artifact" service and
				// when we make use of it will will not additionally be sending
				// out link creation notifications.
				var saveLinksDeferred = LinkUtils.saveLinks(
					projectURI, linksToCreate, true, true
				);
				saveLinksDeferred.addErrback(this, function(response){
					if (!failFastHasOccurred) {
						deferred.errback(response);
						failFastHasOccurred = true;
					}
					return response;
				});
				saveDeferreds.push(saveLinksDeferred);
				var calmSaveLinksDeferred = CalmLinkUtils.createLinks(
					projectURI, calmLinksToCreate, true
				);
				calmSaveLinksDeferred.addErrback(this, function(response){
					if (!failFastHasOccurred) {
						deferred.errback(response);
						failFastHasOccurred = true;
					}
					return response;
				});
				saveDeferreds.push(calmSaveLinksDeferred);
				var deferredList = new dojo.DeferredList(saveDeferreds);
				deferredList.addCallback(this, function(response) {
					if (!failFastHasOccurred) {
						deferred.callback();
					}
					return response;
				});
				deferredList.addErrback(this, function(response) {
					if (!failFastHasOccurred) {
						deferred.errback(response);
						failFastHasOccurred = true;
					}
					return response;
				});
			});
			linkTypesDeferred.addErrback(this, function(response) {
				deferred.errback(response);
				return response;
			});
		});
		existingLinksDeferred.addErrback(this, function(response) {
			deferred.errback(response);
			return response;
		});
		return deferred;
	},

	renameArtifact: function(renameParams){  
		//summary:
		// 		This method renames a resource with the given name to a new location within
		//		the folder hierarchy of the current project.
		//resourceURI: URI
		//		The URI of the resource to be renamed.
		//projectURI: URI
		//		The URI of the current project.
		//revision: String
		//		The revision of the resource being renamed.
		//newName: String
		//		The new name for the resource
		//newDescription: String
		//		The new description for the resource		
		//contentType: String
		//		The Content Type of the Artifact to be renamed.
		var deferred = this._performRename(renameParams);//null;
		return deferred;
	},

	_performRename: function(renameParams){
		var deferred  = new dojo.Deferred();
		var errorHandler = function(errorResponse) {
			ErrorHandler.error(errorResponse);
		};
		
		var saveCallback = function(saveResource){
			//Notify with the save response...
			ChangeNotifier.notifyListeners({
				notifyResource: saveResource,
				eventTopic: com.ibm.rdm.web.notifications.EventTopics.RESOURCE_RENAMED,
				resourceURI: saveResource.getResourceURI(),
				projectURI : renameParams.projectURI,
				resourceUriScope: renameParams.resourceURI,
				xhrArgs: {
					headers: {
						"Content-Type": "application/rdf+xml"
					}
				}
			});
			deferred.callback(saveResource.getResourceURI());				
		};
		
		var renameFunction = function(loadedResource) {
			//sets the new title and description on the resource.
			loadedResource.setTitle(renameParams.newName);
			loadedResource.setDescription(renameParams.newDescription);
			loadedResource.setPrimaryText(loadedResource.getPrimaryText());
			if (renameParams.contentType === MimeTypes.getArtifactFormat(MimeTypes.COLLECTION_NAME, loadedResource.getProjectURI())){
				loadedResource.setArtifactURIs(loadedResource.getArtifactURIs());
				loadedResource.setSnapshot(loadedResource.getSnapshot());
			}
			var savedResource = loadedResource.save({refreshAfterSave: true, softRefresh: true});
			savedResource.addCallback(this, saveCallback);
			savedResource.addErrback(this, errorHandler);
		};		
				
		com.ibm.rdm.web.model.resources.ResourceManager.get(
			{
				resourceURI: renameParams.resourceURI,
				revision: renameParams.revision
			},
			dojo.hitch(this, renameFunction)
		);
		return deferred;		
	},

	secureArtifact: function(secureParams) {
		// summary:
		//		This method is used to assign an artifact's ownership to a process
		//		area (i.e. either a projectURI or a teamAreaURI).
		// resourceURI: URI
		//		The URI of the artifact to be changed.
		// projectURI: URI
		//		The URI of the current project.
		// processAreaURI: URI
		//		The URI of a "process area".  This URI should either be a projectURI
		//		or a teamAreaURI.
		return this._performSecureArtifact({
			processAreaURI: secureParams.processAreaURI,
			resourceURI: secureParams.resourceURI,
			projectURI: secureParams.projectURI
		});
	},
	
	renameFolder: function(renameParams){
		// summary:
		//		This method is used to rename a folder.
		// folderURI: URI
		//		The URI of the folder to be changed.
		// projectURI: URI
		//		The URI of the current project.
		// folderTitle: String
		//		The title for the folder
		// folderDescription: String
		//		The description for the folder				
		var renameFolder = this._performFolderOperation({
			projectURI: renameParams.projectURI,
			folderURI: renameParams.folderURI,
			folderTitle: domutils.getXMLEncodedText(renameParams.folderTitle),
			folderDescription: domutils.getXMLEncodedText(renameParams.folderDescription)
		});
		renameFolder.addBoth(this, function(){
			ChangeNotifier.notifyListeners({
					eventTopic: com.ibm.rdm.web.notifications.EventTopics.FOLDER_RENAMED,    				
					resourceURI: renameParams.folderURI,
					projectURI : renameParams.projectURI,
					newName: renameParams.folderTitle,
					newDescription: renameParams.folderDescription,
					xhrArgs: {
						headers: {
							"Content-Type": "application/rdf+xml"
						}
					}
				});	
		});
		return renameFolder;
	},
	
	moveFolder: function(moveParams){
		// summary:
		//		This method is used to move a folder to a new parent.
		// folderURI: URI
		//		The URI of the folder to be changed.
		// projectURI: URI
		//		The URI of the current project.
		// newParentURI: URI
		//		The parent URI of the folder. (This should be the identifier part (i.e. all after last / of URL	
		var moveFolder = this._performFolderOperation({
			projectURI: moveParams.projectURI,
			folderURI: moveParams.folderURI,
			parentURI: moveParams.newParentURI
		});		
		moveFolder.addBoth(this, function(moveResponse){
			ChangeNotifier.notifyListeners({
					eventTopic: com.ibm.rdm.web.notifications.EventTopics.FOLDER_MOVED,    				
					resourceURI: moveParams.folderURI,
					projectURI : moveParams.projectURI,
					newFolderURI: moveParams.newParentURI,
					xhrArgs: {
						headers: {
							"Content-Type": "application/rdf+xml"
						}
					}
				});	
		});
		return moveFolder;
	},

	secureFolder: function(secureParams) {
		// summary:
		//		This method is used to assign a folder's ownership to a process
		//		area (i.e. either a projectURI or a teamAreaURI).
		// folderURI: URI
		//		The URI of the folder to be changed.
		// projectURI: URI
		//		The URI of the current project.
		// processAreaURI: URI
		//		The URI of a "process area".  This URI should either be a projectURI
		//		or a teamAreaURI.
		// doUpdateProcessAreaForSubFolders: Boolean
		//		Passed as true when the sub-folders of folderURI are to ALSO be
		//		assigned new ownership (to the same passed processAreaURI).
		var secureFolderDeferred = this._performFolderOperation({
			projectURI: secureParams.projectURI,
			folderURI: secureParams.folderURI,
			processAreaURI: secureParams.processAreaURI,
			doUpdateProcessAreaForSubFolders: secureParams.doUpdateProcessAreaForSubFolders
		});
		return secureFolderDeferred;
	},
	
	createFolder: function(createParams){
		//summary:
		// 		This method is used to create a new folder.
		//projectURI: URI
		//		The URI of the current project.
		//parentURI: URI
		//		The parent URI of the folder. (This should be the identifier part (i.e. all after last / of URL
		//folderTitle: String
		//		The title for the folder
		//folderDescription: String
		//		The description for the folder
		var createFolder = this._performCreateFolder(createParams);
		
		createFolder.addBoth(this, function(createResponse){
			ChangeNotifier.notifyListeners({		
				eventTopic: com.ibm.rdm.web.notifications.EventTopics.FOLDER_CREATED,    				
				resourceURI: createResponse.ioArgs.xhr.getResponseHeader("Location"),
				projectURI : createParams.projectURI,
				newName: createParams.folderTitle,
				newDescription: createParams.folderDescription,
				newParent: createParams.parentURI,
				xhrArgs: {
					headers: {
						"Content-Type": "application/rdf+xml"
					}
				}
			});			
		});
		
		return createFolder;
	},
	
	_performDeleteFolder: function(resourceURI){
		return RDMClient.doPost({
			uri: resourceURI +"?deleteRecursive",
			skipErrors: true
		});
	},
	
	_performCreateFolder: function(createParams){
		//summary:
		// 		This method is used to create a new folder.
		//projectURI: URI
		//		The URI of the current project.
		//parentURI: URI
		//		The parent URI of the folder. (This should be the identifier part (i.e. all after last / of URL
		//folderTitle: String
		//		The title for the folder
		//folderDescription: String
		//		The description for the folder

		var deferred = new dojo.Deferred();		
		var postBody = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>" +
						"<rdf:RDF xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\">" +
						"<ns:folder xmlns:ns=\"http://com.ibm.rdm/navigation#\" rdf:about=\"\">" + 
						"<ns:title xmlns:ns=\"http://purl.org/dc/terms/\">" +
						domutils.getXMLEncodedText(createParams.folderTitle) +
						"</ns:title>" +
						"<ns:description xmlns:ns=\"http://purl.org/dc/terms/\">" +
						domutils.getXMLEncodedText(createParams.folderDescription) +
						"</ns:description>" +
						"<ns:parent rdf:resource=\"" + createParams.parentURI + "\" xmlns:ns=\"http://com.ibm.rdm/navigation#\"/>" +
						"</ns:folder>" +
						"</rdf:RDF>";
		var folderOperationDeferred = RDMClient.doPost({
			uri: com.ibm.rdm.web.FRONTING_SERVICE_FOLDERS,
			projectURI: createParams.projectURI,
			postData: postBody,
			xhrArgs: {
				headers: {
					"Content-Type": "application/rdf+xml"
				}
			}
			
		});		
		folderOperationDeferred.addCallback(this, function(response){
			deferred.callback(response);
		});
		folderOperationDeferred.addErrback(this, function(response){
			deferred.errback(response);
		});		
		
		return deferred;
	},
	
	_performFolderOperation: function(folderParams){
		// summary:
		//		This method is used to rename or move a folder or to assign a
		//		folder's ownership to a process area.
		// folderURI: URI
		//		The URI of the folder to be changed.
		// projectURI: URI
		//		The URI of the current project.
		// parentURI: URI
		//		The parent URI of the folder.
		//		(This should be the identifier part (i.e. all after last / of URL).)
		// folderTitle: String
		//		The title for the folder
		// folderDescription: String
		//		The description for the folder
		// processAreaURI: URI
		//		The URI of a project or team area.
		// doUpdateProcessAreaForSubFolders: Boolean
		//		Passed as true when the sub-folders of folderURI are to ALSO be
		//		assigned new ownership (to the same passed processAreaURI).
		var deferred = new dojo.Deferred();
		var performFolderPut = function(result){
			var folderNode = domutils.getSingleElementByTagNameNS(result.doc, "folder", queryutils.NAV_NAMESPACE);
			var titleNode = domutils.getSingleElementByTagNameNS(folderNode, "title", queryutils.DCTERMS_NAMESPACE);
			var title = (titleNode.firstChild ? jazz.util.html.escape(titleNode.firstChild.nodeValue) : "");
			var descriptionNode = domutils.getSingleElementByTagNameNS(folderNode, "description", queryutils.DCTERMS_NAMESPACE);
			var description = (descriptionNode.firstChild ? descriptionNode.firstChild.nodeValue : "");
			var parentNode = domutils.getSingleElementByTagNameNS(folderNode, "parent", queryutils.NAV_NAMESPACE);
			var parentURI = domutils.getAttributeNS(parentNode, "resource", queryutils.RDF_NAMESPACE, queryutils.RDF_QUALIFIER);

			var putTitle = folderParams.folderTitle || title;
			var putDescription = folderParams.folderDescription || description;
			var putParent = folderParams.parentURI || parentURI;
			var putProcessArea = folderParams.processAreaURI || "";
			if (!putProcessArea) {
				var processAreaNode = domutils.getSingleElementByTagNameNS(folderNode, "processArea", queryutils.RM_NAMESPACE);
				if (processAreaNode) {
					putProcessArea = domutils.getAttributeNS(processAreaNode, "resource", queryutils.RDF_NAMESPACE, queryutils.RDF_QUALIFIER);
				}
			}

			var uri = folderParams.folderURI;
			if (folderParams.doUpdateProcessAreaForSubFolders) {
				uri += "?updateRecursive&processArea=" + putProcessArea;
			}
			
			var putBody = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>" +
							"<rdf:RDF xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\">" +
							"<ns:folder xmlns:ns=\"http://com.ibm.rdm/navigation#\" rdf:about=\"\">" + 
							"<ns:title xmlns:ns=\"http://purl.org/dc/terms/\">" +
							putTitle +
							"</ns:title>" +
							(putProcessArea ?
							 "<ns:processArea rdf:resource=\"" + putProcessArea + "\" xmlns:rdf=\"http://www.ibm.com/xmlns/rdm/rdf/\"/>" :
							 "") +
							"<ns:description xmlns:ns=\"http://purl.org/dc/terms/\">" +
							putDescription +
							"</ns:description>" +
							"<ns:parent rdf:resource=\"" + putParent + "\" xmlns:ns=\"http://com.ibm.rdm/navigation#\"/>" +
							"</ns:folder>" +
							"</rdf:RDF>";
			var putOrPostParams = {
				uri: uri,
				projectURI: folderParams.projectURI,
				postData: putBody,
				xhrArgs: {
					headers: {
						"Content-Type": "application/rdf+xml"
					}
				}
			};
			var folderOperationDeferred = (
				folderParams.doUpdateProcessAreaForSubFolders ?
				RDMClient.doPost(putOrPostParams) :
				RDMClient.doPut(putOrPostParams)
			);
			
			folderOperationDeferred.addBoth(this, function(response){
				if (folderParams.doUpdateProcessAreaForSubFolders){
					ProjectCache.clearFolderCache();
				}
				else{
					ProjectCache.invalidateCachedFolder(folderParams.folderURI);
				}
				return response;
			});
			folderOperationDeferred.addCallback(this, function(response){
				deferred.callback(response);
			}); 
			folderOperationDeferred.addErrback(this, function(response){
				deferred.errback(response);
			});
			
		};

		var getFolder = RDMClient.doGet({
			uri: folderParams.folderURI,
			skipCache: true
		});		
		getFolder.addBoth(this, performFolderPut);
		
		return deferred;
	},
	
	_performMultipleDeleteOperation: function(artifacts, isInModule, moduleCount, undeletedArtifactsGatherer, resourceToWrapperMap,
		deleteUnboundResources) {
		// summary:
		//		Serially deletes a number of resources.
		// artifacts:
		//		An array of resources to delete.  May contain wrapper resources.
		// isInModule:
		//		set true to indicate that the deletion is within a module.
		// moduleCount:
		//		If provided, indicates the number of modules that are included in the deletion.
		// undeletedArtifactsGatherer
		//		Gathers URIs of artifacts we failed to delete
		// resourceToWrapperMap
		//		Maps resource URIs to their wrapped resource URI, is changed
		// deleteUnboundResources
		//		Set to true to delete the resources if they are not bound to any modules
		// returns:
		//		A deferred that resolves when all artifacts have been deleted.
		var resourceURIsToDelete = [];
		// Convert artifacts to an array of URIs, getting the proper URI for wrapper resources.
		dojo.forEach(artifacts, function(artifact){
			// wrapped resource needs to be deleted for wrapper resources, server will
			// delete the wrapper resource. But if we are deleting from a module, it is the wrapper that we should delete - in this
			// case the wrapped resource is still being used by some other wrapper outside of the module.
			if (!isInModule && (artifact.getEntityType() === MimeTypes.getArtifactFormat(MimeTypes.WRAPPER_RESOURCE_NAME, artifact.getProjectURI()))) {
				// delete the wrapped resource
				resourceURIsToDelete.push(artifact.getWrappedResourceURI());
				resourceToWrapperMap[artifact.resourceURI] = artifact.getWrappedResourceURI();
			} else {
				//delete the resource itself.
				resourceURIsToDelete.push(artifact.resourceURI);
			}
		});
		
		
		// 67335 - If we're deleting from a module, we have to use the legacy code until we can safely update
		// the server code to handle multi-request delete for module artifacts. ~Jason
		if (isInModule) {
			return this._performLegacyMultipleDeleteOperationByUri(resourceURIsToDelete, isInModule, moduleCount, undeletedArtifactsGatherer, deleteUnboundResources);
		} else {
			return this._performMultipleDeleteOperationByUri(resourceURIsToDelete, isInModule, moduleCount, undeletedArtifactsGatherer);
		}
	},
	
	_performLegacyMultipleDeleteOperationByUri: function(artifactURIs, isInModule, /*?Integer*/moduleCount, undeletedArtifactsGatherer, deleteUnboundResources) {
		// summary:
		//		Serially deletes a number of resources.
		// artifactURIs:
		//		An array of URIs of the resources to delete.  Note that this must contain
		//		the actual URIs to delete.  It should not include any wrapper resource URIs - 
		//		They must all be converted to the corresponding wrapped resource URIs before
		//		calling this method.
		// isInModule:
		//		set true to indicate that the deletion is within a module.
		// moduleCount:
		//		If provided, indicates the number of modules that are included in the deletion.
		// undeletedArtifactsGatherer
		//		If provided, a collection for the URIs of artifacts we didn't delete 
		// deleteUnboundResources
		//		Set to true to delete the resources if they are not bound to any modules
		// returns:
		//		A deferred that resolves when all artifacts have been deleted.

		var count = 0,
			unboundArtifactURIs = [],
			multipleDeleteDeferred = new dojo.Deferred(),
			lockedArtifactsCount = 0,
			processDelete,
			errorHandler;
		
		//We need to chain the deferred together to delete the artifacts one by one.
		
		var deleteResource = dojo.hitch(this, function(response) {
			var deleteURI = artifactURIs[count];
			if (isInModule || moduleCount) {
				// For deletion in module, ask for info on base artifacts that are now no longer bound to any module.
				deleteURI += "?reportUnboundArtifacts=true";
			}
			var deleteDeferred = RDMClient.doDelete({
				uri:deleteURI,
				delayCallbackUntilDataIndexed: false //deletion should ALWAYS be handled on the client //(count < artifacts.length-1)?false:true
			});
			
			//Multiple delete needs to rely on returned task as per defect 56211
			if (isInModule || moduleCount > 0) {
				deleteDeferred.addErrback(this, errorHandler);
				deleteDeferred.addCallback(this, function(response){
					if (response.locked) {
						//If it was locked, don't expect any unbound artifact URIs
						processDelete(response);
					} else {
						(new TaskTracker()).pollTracker({
							response: response,
							callback: dojo.hitch(this, function(trackerResponse){
								this._gatherUnboundArtifactURIs(trackerResponse, unboundArtifactURIs);
								processDelete(response);
							})
						});
					}
				});		
				
			}
			else {
				//Add errback first so we can correct if possible
				deleteDeferred.addErrback(this, errorHandler);
				deleteDeferred.addCallback(this, processDelete);		
			}
			return response;
		});
		
		var showLocked = dojo.hitch(this, function(all, okFunction){
			
			var msg;
			if (all) {
				msg = this._messages.allWereLocked;
			} else {
				msg = this._messages.someWereLocked;
			}
			
			var msgForm = new com.ibm.rdm.web.common.MessageForm({
				confirmContent: msg,
				dialogTitle: 	this._messages.showLockedFeedbackDialogTitle,
				dialogStyle: 	"ok",
				dialogWidth:	"350px",
				onOk:			okFunction
			});   
			
			msgForm.show();
		});
		
		processDelete = dojo.hitch(this, function(response) {
		
			//delete complete - either loop or finish.
			count++;
			if (count < artifactURIs.length) {
				//round again
				deleteResource();
			} else {
				//finished
				// If we have removed the last module references to any artifacts, ask user if they should be deleted.
				if (lockedArtifactsCount === count) {
					//All artifacts were locked - tell the user. No need to do anything else
					showLocked(true, dojo.hitch(multipleDeleteDeferred, multipleDeleteDeferred.callback, response));
				} else {
					if (unboundArtifactURIs.length) {
						var deleteUnboundArtifactsDeferred = this._deleteUnboundArtifacts(unboundArtifactURIs, isInModule, moduleCount, lockedArtifactsCount, deleteUnboundResources);
						deleteUnboundArtifactsDeferred.addBoth(this, function(){
							multipleDeleteDeferred.callback(response);
						});
					} else {
						//No unbound artifacts - need a message if any could not be deleted
						if (lockedArtifactsCount > 0) {
							showLocked(false, dojo.hitch(multipleDeleteDeferred, multipleDeleteDeferred.callback, response));
						} else {
							multipleDeleteDeferred.callback(response);
						}
					}
				}
			}		
			return response;
		});
		
		errorHandler = function(errorResponse) {
			if (errorResponse.lockResponse) {
				//An artifact is locked - just remember how many were locked
				com.ibm.rdm.web.common.MessageHandler.clearMessage();
				lockedArtifactsCount++;
				//Remember we didn't delete it
				if (undeletedArtifactsGatherer) {
					undeletedArtifactsGatherer[artifactURIs[count]]=true;
				}
				//Return so it goes to callback
				return {locked: true};
			} else {
				ErrorHandler.error(errorResponse);

				//an error
				multipleDeleteDeferred.errback(errorResponse);
			}
			//throw so it goes to further errbacks
			throw errorResponse;
		};
		
		//Start the ball rolling
		deleteResource();
		
		return multipleDeleteDeferred;
	},	
	
	_performMultipleDeleteOperationByUri: function(artifactURIs, isInModule, /*?Integer*/moduleCount, undeletedArtifactsGatherer) {
		// summary:
		//		Serially deletes a number of resources.
		// artifactURIs:
		//		An array of URIs of the resources to delete.  Note that this must contain
		//		the actual URIs to delete.  It should not include any wrapper resource URIs - 
		//		They must all be converted to the corresponding wrapped resource URIs before
		//		calling this method.
		// isInModule:
		//		set true to indicate that the deletion is within a module.
		// moduleCount:
		//		If provided, indicates the number of modules that are included in the deletion.
		// undeletedArtifactsGatherer
		//		If provided, a collection for the URIs of artifacts we didn't delete 
		// returns:
		//		A deferred that resolves when all artifacts have been deleted.

		var unboundArtifactURIs = [],
			multipleDeleteDeferred = new dojo.Deferred(),
			nonDeletedArtifactsCount = 0,
			showLocked,
			errorHandler;
		
		var deleteResource = dojo.hitch(this, function() {			
			artifactURIs = dojo.map(artifactURIs, function(uri) {
				if (isInModule || moduleCount) {
					uri += "?reportUnboundArtifacts=true";
				}
				
				return uri;
			});
			
			var multiRequestArgs = [];
			
			dojo.forEach(artifactURIs, function(uri) {
				multiRequestArgs.push({
					method: "DELETE",
					uri: uri
				});
			}, this);
			
			var deleteDeferred = RDMClient.doMultiRequest(multiRequestArgs);
			
			deleteDeferred.addCallback(this, function(results) {
				var unboundDeferreds = [];
				
				dojo.forEach(results, function(result) {
					if (result.statusCode === 200 || result.statusCode === 410) {
						// OK or GONE.  Delete succeeded, or the artifact was already gone for
						// some reason.  Success
						return;
					} else if (result.statusCode === 403) {
						var lockResponse = RDMClient.error403RDFParser(result.doc);
						if (lockResponse) {
							result.lockResponse = lockResponse;
						}
						errorHandler(result);
					} else if (result.statusCode === 202) {
						unboundDeferreds.push(this._trackUnboundArtifacts(result.headers.Location, unboundArtifactURIs));						
					} else {
						// some other error
						errorHandler(result);
					}
				}, this);
				
				if (unboundDeferreds.length > 0) {
					var unboundDeferredList = new dojo.DeferredList(unboundDeferreds);
					
					unboundDeferredList.addCallback(this, function(response) {
						if (nonDeletedArtifactsCount === artifactURIs.length) {
							//All artifacts were locked - tell the user. No need to do anything else
							showLocked(true, dojo.hitch(multipleDeleteDeferred, multipleDeleteDeferred.callback, results));
						} else {
							if (unboundArtifactURIs.length) {
								var deleteUnboundArtifactsDeferred = this._deleteUnboundArtifacts(unboundArtifactURIs, isInModule, moduleCount, nonDeletedArtifactsCount);
								deleteUnboundArtifactsDeferred.addBoth(this, function(){
									multipleDeleteDeferred.callback(response);
								});
							} else {
								//No unbound artifacts - need a message if any could not be deleted
								if (nonDeletedArtifactsCount > 0) {
									showLocked(false, dojo.hitch(multipleDeleteDeferred, multipleDeleteDeferred.callback, results));
								} else {
									multipleDeleteDeferred.callback(response);
								}
							}
						}						
					});
					
					unboundDeferredList.addErrback(this, errorHandler);				
				} else {
					if (nonDeletedArtifactsCount === artifactURIs.length) {
						//All artifacts were locked - tell the user. No need to do anything else
						showLocked(true, dojo.hitch(multipleDeleteDeferred, multipleDeleteDeferred.callback, results));
					} else {
						if (unboundArtifactURIs.length) {
							var deleteUnboundArtifactsDeferred = this._deleteUnboundArtifacts(unboundArtifactURIs, isInModule, moduleCount, nonDeletedArtifactsCount);
							deleteUnboundArtifactsDeferred.addBoth(this, function(){
								multipleDeleteDeferred.callback(results);
							});
						} else {
							//No unbound artifacts - need a message if any could not be deleted
							if (nonDeletedArtifactsCount > 0) {
								showLocked(false, dojo.hitch(multipleDeleteDeferred, multipleDeleteDeferred.callback, results));
							} else {
								multipleDeleteDeferred.callback(results);
							}
						}
					}
				}
				
				return results;
			});
		});
		
		showLocked = dojo.hitch(this, function(all, okFunction){
			
			var msg;
			if (all) {
				msg = this._messages.allWereLockedOrInModule;
			} else {
				msg = this._messages.someWereLockedOrInModule;
			}
			
			var msgForm = new com.ibm.rdm.web.common.MessageForm({
				confirmContent: msg,
				dialogTitle: 	this._messages.showLockedFeedbackDialogTitle,
				dialogStyle: 	"ok",
				dialogWidth:	"350px",
				onOk:			okFunction
			});   
			
			msgForm.show();
		});
		
		errorHandler = function(errorResponse) {
			if (errorResponse.reasonPhrase && (errorResponse.reasonPhrase.indexOf("CRRRS8683") >= 0)) {
				// server says the artifact was not deleted because it is used in a module.
				// Record that we didn't delete it. 
				nonDeletedArtifactsCount++;
				if (undeletedArtifactsGatherer && errorResponse.url) {
					undeletedArtifactsGatherer[UriHelper.stripURIParams(errorResponse.url)] = true;
				}
				return {inModule: true};
			} else if (errorResponse.lockResponse) {
				//An artifact is locked - just remember how many were locked
				com.ibm.rdm.web.common.MessageHandler.clearMessage();
				nonDeletedArtifactsCount++;
				//Remember we didn't delete it
				if (undeletedArtifactsGatherer && errorResponse.url) {
					undeletedArtifactsGatherer[UriHelper.stripURIParams(errorResponse.url)] = true;
				}
				//Return so it goes to callback
				return {locked: true};
			} else {
				ErrorHandler.error(errorResponse);

				//an error
				multipleDeleteDeferred.errback(errorResponse);
			}
			//throw so it goes to further errbacks
			throw errorResponse;
		};
		
		//Start the ball rolling
		deleteResource();
		
		return multipleDeleteDeferred;
	},
	
	_gatherUnboundArtifactURIs: function(trackerResponse, unboundArtifactURIs) {
		// summary:
		//		Add any unbound Artifact URIs found in the gived tracker response in the
		//		given unboundArtifactURIs array.
		var label, unboundArtifact, artifactURI;
		if (trackerResponse.unboundArtifacts) {
			var unboundArtifacts = trackerResponse.unboundArtifacts;
			for (label in unboundArtifacts) {
				if (label.indexOf("unboundArtifact") === 0) {
					unboundArtifact = unboundArtifacts[label];
					
					// If this is a wrapper resource, gather the wrapped resource URI, rather than
					// the wrapper, since that is what we need to delete - we delete the wrapped
					// and the wrapper is deleted automagically by the server.
					artifactURI = unboundArtifact.wrappedResourceURI ? unboundArtifact.wrappedResourceURI.xmlAttributes.resource :
																	   unboundArtifact.artifact.xmlAttributes.resource;
					if (artifactURI) {
						unboundArtifactURIs.push(artifactURI);
					}
				}
			}
		}
	},
	
	_trackUnboundArtifacts: function(trackerURI, unboundArtifactURIs) {
		var trackingDeferred = new dojo.Deferred();
  
		trackingDeferred.addCallback(this, function(response) {
			this._gatherUnboundArtifactURIs(response, unboundArtifactURIs);
		});
		
		setTimeout(dojo.hitch(this, this._pollUnboundArtifacts, trackerURI, trackingDeferred), 500);
		
		return trackingDeferred;
	},
	
	_pollUnboundArtifacts: function(trackerURI, trackingDeferred) {
		var pollingDeferred = RDMClient.doGet({
			uri: trackerURI,
			skipCache: true
		});
		
		pollingDeferred.addCallback(this, function(response) {
			var xmlParser = new com.ibm.rdm.web.parsers.xml2json(response.doc);
			var tracker = xmlParser.parse().RDF.Description;

			if (tracker.taskError) {
				StatusDisplay.hideStatusMessage();
				com.ibm.rdm.web.common.MessageHandler.showError(
					7661, [], response
				);
				
				return response;
			} else if (tracker.taskProgress && tracker.taskProgress.value) {
				StatusDisplay.hideStatusMessage();
				if (tracker.taskProgress.value.toLowerCase().indexOf("error") >= 0) {
					com.ibm.rdm.web.common.MessageHandler.showError(
						7661, [], response
					);
					
					return response;
				}
			} 
			
			if (tracker.workDone && tracker.totalWorkRequired && parseInt(tracker.workDone.value, 10) >= parseInt(tracker.totalWorkRequired.value, 10)) {
				trackingDeferred.callback(tracker);
			} else {
				setTimeout(dojo.hitch(this, this._pollUnboundArtifacts, trackerURI, trackingDeferred), 500);
			}
			
			return response;
		});
			
	},
	
	_deleteUnboundArtifacts: function(unboundArtifactURIs, isInModule, moduleCount, nonDeletedArtifactCount,
									  deleteUnboundArtifact, /*Callback?*/onDialogClose) {
		// summary:
		//		Prompts for confirmation that unbound artifacts are to be
		//		removed, then removes them.
		// unboundArtifactURIs:
		//		An array indicating the artifacts to be removed.
		// moduleCount:
		//		If provided, indicates the number of modules from which the unbound artifacts have come.
		// isInModule:
		//		true if the original deletion was within a module (rather than a delete including a
		//		whole module.
		// nonDeletedArtifactCount:
		//		the number of artifacts that were not deleted due to locks, use in module, permissions etc.
		//		and so couldn't be deleted originally.
		// deleteUnboundArtifact
		//		If set to true, overrides prompt for confirmation and removes the unbound artifacts.
		//		If set to false, overrides prompt for confirmation and skips the removal of unbound artifacts.
		// returns:
		//		A deferred whose callback is called when the artifacts have
		//		been removed.
		var deleteUnboundArtifactsDeferred = new dojo.Deferred(),
			confirmationMessages = [];
		
		var deleteUnboundResources = function() {
			//console.debug("deleting unbound artifacts");			
			StatusDisplay.showStatusMessage(this._messages.deletingArtifacts);
			var multiDeleteDeferred = this._performMultipleDeleteOperationByUri(unboundArtifactURIs);
			multiDeleteDeferred.then(function(){
				deleteUnboundArtifactsDeferred.callback();
				return;
			}, function(error){
				deleteUnboundArtifactsDeferred.errback(error);
				return error;
			});
			multiDeleteDeferred.addBoth(function(){
				StatusDisplay.hideStatusMessage();
			});
		};
		
		var cancelDelete = function() {
			//console.debug("NOT deleting unbound artifacts");
			deleteUnboundArtifactsDeferred.callback();
		};
		
		var skipConfirmationPropertyName;
		
		var showConfirmation = dojo.hitch(this, function(deleteUnboundArtifact){
			if (isInModule) {
				if (deleteUnboundArtifact) {
					// No need to show confirmation dialog.  Response has already been given.					 
					var deleteUnboundedResourcesFunction = dojo.hitch(this, deleteUnboundResources);
					deleteUnboundedResourcesFunction();
				}
				else if (!deleteUnboundArtifact) {
					var cancelDeleteFunction = dojo.hitch(this, cancelDelete);
					cancelDeleteFunction();					
				}
			}
			else {
				confirmationMessages.push(this._messages.cannotUndo);
				var confirmationMessageNode = this._concatConfirmationMessages(confirmationMessages);

				var msgForm = new com.ibm.rdm.web.common.MessageForm(
						{
							confirmContent: confirmationMessageNode,
							dialogTitle: 	this._messages.confirmDeleteDialogTitle,
							dialogStyle: 	"yesNo",
							yesButtonText: 	this._messages.deleteLabel,
							noButtonText:	this._messages.skipLabel,
							dialogWidth:	"350px",
							doNotAskAgainPropertyName: skipConfirmationPropertyName,
							onOk:			dojo.hitch(this, deleteUnboundResources),
							onCancel:		dojo.hitch(this, cancelDelete)
						});   
				msgForm.show(onDialogClose);
			}
		});

		if (nonDeletedArtifactCount > 0) {
			// start the confirmation with a message saying not everything was deleted or removed.
			confirmationMessages.push( isInModule ?	this._messages.someWereLockedModuleRemove : this._messages.someWereLockedOrInModule );
		}
		if (unboundArtifactURIs.length === 1) {
			// For just one unbound artifact get the name to include in the message
			var getNameDeferred = this._getArtifactDisplayName(unboundArtifactURIs[0]);
			getNameDeferred.addCallback(this, function(name){
				if (isInModule) { 
					skipConfirmationPropertyName = UserPreferenceNames.DO_NOT_ASK_DELETE_UNBOUND_IN_MODULE;
					confirmationMessages.push(dojo.string.substitute(this._messages.confirmDeleteUnboundArtifactThisModule, [name]));
				} else {
					skipConfirmationPropertyName = UserPreferenceNames.DO_NOT_ASK_DELETE_UNBOUND_WHOLE_MODULE;
					confirmationMessages.push(dojo.string.substitute(this._messages.confirmDeleteUnboundArtifactFromDeletedModule, [name]));
				}
				showConfirmation(deleteUnboundArtifact);
				return name;
			});
		} else {
			// For multiple unbound artifacts, just use the number in the message
			if (isInModule) {
				skipConfirmationPropertyName = UserPreferenceNames.DO_NOT_ASK_DELETE_UNBOUND_IN_MODULE;
				confirmationMessages.push(dojo.string.substitute(this._messages.confirmDeleteUnboundArtifactsThisModule, [unboundArtifactURIs.length]));
			} else {
				skipConfirmationPropertyName = UserPreferenceNames.DO_NOT_ASK_DELETE_UNBOUND_WHOLE_MODULE;
				if (moduleCount === 1) {
					confirmationMessages.push(dojo.string.substitute(this._messages.confirmDeleteUnboundArtifactsFromDeletedModule, [unboundArtifactURIs.length]));
				} else {
					confirmationMessages.push(dojo.string.substitute(this._messages.confirmDeleteUnboundArtifactsFromDeletedModules, [unboundArtifactURIs.length, moduleCount]));
				}
			}
			showConfirmation(deleteUnboundArtifact);
		}
		
		return deleteUnboundArtifactsDeferred;
	},
	
	_getArtifactDisplayName: function(artifactURI) {
		// summary:
		//		Gets the display name of an artifact.
		// artifactURI:
		//		Identifier for the artifact.
		// returns:
		//		A deferred whose callback is called with the display name or "unknown name" if
		//		there is some error reading the name.
		var displayNameDeferred = new dojo.Deferred();
		com.ibm.rdm.web.model.resources.ResourceManager.get(
			{
				resourceURI: artifactURI
			},
			dojo.hitch(this, function(loadedResource){
				// We get here for both success and error cases.
				var displayName = loadedResource.getDisplayTitle ? loadedResource.getDisplayTitle() : this._messages.unknownArtifactName;
				displayNameDeferred.callback(displayName);
			})
		);
		return displayNameDeferred;
	},
	
	_concatConfirmationMessages: function(messages) {
		// summary:
		//		Adds a trailing caveat to the message indicating that the operation cannot be undone.
		// messages:
		//		An array of message strings.  These are joined into a DIV that includes each with line break between.
		// returns:
		//		A DOM suitable for display in a MessageForm.
		var messageWrapper = dojo.create("div"),
			first = true;
		dojo.forEach(messages, function(message){
			if (!first) {
				dojo.create("br", null, messageWrapper);
				dojo.create("br", null, messageWrapper);
			}
			dojo.place(document.createTextNode(message), messageWrapper);
			first = false;
		});
		return messageWrapper;
	},
		
	_performSecureArtifact: function(secureParams){
		var deferred  = new dojo.Deferred();
		var errorHandler = function(errorResponse) {
			ErrorHandler.error(errorResponse);
		};
		var saveCallback = function(saveResource){
			ChangeNotifier.notifyListeners({
				eventTopic: com.ibm.rdm.web.notifications.EventTopics.RESOURCE_PROCESS_AREA_CHANGED,
				projectURI: secureParams.projectURI,
				resourceURI: secureParams.resourceURI,
				processAreaURI: secureParams.processAreaURI
			});
			deferred.callback(saveResource.getResourceURI());				
		};
		var secureFunction = function(loadedResource) {
			loadedResource.setProcessAreaURI(secureParams.processAreaURI);
			var savedResource = loadedResource.save({refreshAfterSave: true, softRefresh: true});
			savedResource.addCallback(this, saveCallback);
			savedResource.addErrback(this, errorHandler);
		};		
		com.ibm.rdm.web.model.resources.ResourceManager.get(
			{
				resourceURI: secureParams.resourceURI
				// revision: renameParams.revision
			},
			dojo.hitch(this, secureFunction)
		);
		return deferred;		
	},

	saveCollection: function (collectionObject, callback) {
		var createCollectionDeferred = collectionObject.save();
		createCollectionDeferred.addBoth(this, callback);
		return createCollectionDeferred;
	},
	
	createRequirement: function(
		projectURI, folderURI, objectTypeURI,
		richText, title, description,
		enableEditInGrid, enableFocusInGrid,
		resourceUriScope, moduleContextURI, isHeading,
		where, relativeSiblingURI, forceDelayCallback)
	{
		//create a resource
		var resourceObject = com.ibm.rdm.web.model.resources.ResourceManager.create({
			entityType: MimeTypes.getArtifactFormat(MimeTypes.REQUIREMENT_NEW_NAME, projectURI),
			projectURI: projectURI,
			resourceOriginURI: moduleContextURI
		});
		
		resourceObject.setProjectURI(projectURI);
		resourceObject.setTitle(title);
		resourceObject.setDescription(description);
		resourceObject.setIsHeading(isHeading);
		if (richText) {
			resourceObject.setPrimaryText(richText);
		}
		
		// Create the requirement
		var assetFolderURIOverride,
			createInFolderURI,
			requirementSaved;
		
		if (folderURI) {
			if (moduleContextURI) {
				// creating in the context of a module and we have been given an alternative folder to use for base artifact.  
				assetFolderURIOverride = folderURI;
			} else {
				// creating outside of a module, and we have been given a folder to create in.
				createInFolderURI = folderURI;
			}
		}
		
		// Give the resource a type and create the new requirement
		var addTypeAndCreateRequirementPromise = resourceObject.addType(objectTypeURI).then(
			dojo.hitch(this, function(typeResponse) {
				var createRequirementDeferred = resourceObject.save({
					refreshAfterSave: moduleContextURI?false:true,
					softRefresh: true,
					forceDelayCallback: forceDelayCallback,
					overridenAssetFolder: assetFolderURIOverride,
					createInFolderURI: createInFolderURI,
					relativeSiblingURI: relativeSiblingURI,
					where: where
				}); //TODO Module hack - data must be indexed after POST
				return createRequirementDeferred; // returning deferred, so overallDeferred will wait for createRequirementDeferred.
			})
		).then(
			dojo.hitch(this, function(createResponse) {
				this.publishResourceCreatedEvent(
					enableEditInGrid, enableFocusInGrid,
					resourceObject.resourceURI, projectURI,
					resourceObject, resourceUriScope,
					where, relativeSiblingURI
				);
				return createResponse;
			})
		);
		
		return addTypeAndCreateRequirementPromise;
	},

	createRequirementStub: function(args) {
		// summary:
		//		Creates a 'stub' requirement - essentially instantiates a requirement and does everything except the POST
		//		This is used for immediate insertion into the grid, and later is saved/posted via a user action once edit
		//		has been completed on the requirement
		var projectURI = args.projectURI,
			folderURI = args.folderURI,
			objectTypeURI = args.typeURI, 
			richText = args.primaryText, 
			title = args.title, 
			description = args.description, 
			enableEditInGrid = args.enableEditInGrid, 
			resourceUriScope = args.resourceUriScope, 
			moduleContextURI = args.moduleContextURI, 
			isHeading = args.isHeading, 
			relativeSiblingURI = args.relativeSiblingURI,
			where = args.where;
		
		//create a resource
		var resourceObject = com.ibm.rdm.web.model.resources.ResourceManager.create({
			entityType: MimeTypes.getArtifactFormat(MimeTypes.REQUIREMENT_NEW_NAME, projectURI),
			projectURI: projectURI,
			resourceOriginURI: moduleContextURI
		});
		
		resourceObject.setProjectURI(projectURI);
		resourceObject.setTitle(title);
		resourceObject.setDescription(description);
		resourceObject.setIsHeading(isHeading);

		if (richText) {
			resourceObject.setPrimaryText(richText);
		}
		
		// Create the requirement
		var assetFolderURIOverride,
			createInFolderURI,
			requirementSaved;
		
		if (folderURI) {
			if (moduleContextURI) {
				// creating in the context of a module and we have been given an alternative folder to use for base artifact.  
				assetFolderURIOverride = folderURI;
			} else {
				// creating outside of a module, and we have been given a folder to create in.
				createInFolderURI = folderURI;
			}
		}

		resourceObject.setInitialSaveParams({
			refreshAfterSave: moduleContextURI?false:true,
			softRefresh: true,
			forceDelayCallback: false,
			overridenAssetFolder: assetFolderURIOverride,
			createInFolderURI: createInFolderURI,
			where: where,
			relativeSiblingURI: relativeSiblingURI
			
		});
		
		// Give the resource a type
		var addTypeAndCreateRequirementPromise = resourceObject.addType(objectTypeURI).then(
			dojo.hitch(this, function(typeResponse) {
				var stubId = com.ibm.rdm.web.model.resources.ResourceManager.getNextStubId();
				resourceObject.resourceURI = stubId;
				resourceObject.stubId = stubId;
				return resourceObject;
			})
		);
		
		return addTypeAndCreateRequirementPromise;
	},
	
	createCollection: function(projectURI, folderURI, objectTypeURI, title, description, resourcesArray, associatedSnapshot, enableEditInGrid, resourceUriScope) {
		//create a collection resource
		var resourceObject = com.ibm.rdm.web.model.resources.ResourceManager.create(
			{entityType: MimeTypes.getArtifactFormat(MimeTypes.COLLECTION_NAME, projectURI),
			projectURI: projectURI}
		);
		
		// set additional properties
		resourceObject.setProjectURI(projectURI);
		resourceObject.setTitle(title);
		resourceObject.setDescription(description);
		if (resourcesArray && resourcesArray.length > 0){
			resourceObject.setArtifactURIs(resourcesArray);
		}
		
		// add the type to the resource object
		var addTypeAndCreateCollectionPromise = resourceObject.addType(objectTypeURI).then(
			// chain the type response with the resource object save
			dojo.hitch(this, function(typeResponse) {
				var createCollectionDeferred = resourceObject.save({
					refreshAfterSave: true,
					softRefresh: true,
					createInFolderURI: folderURI
				});
				return createCollectionDeferred;
			})
		).then(
			// resource object save callbacks
			dojo.hitch(this, function(createResponse) {
				this.publishResourceCreatedEvent(
					enableEditInGrid, false,
					resourceObject.resourceURI, projectURI,
					resourceObject, resourceUriScope);
				return createResponse;
			})
		);
		
		return addTypeAndCreateCollectionPromise;
	},
	
	createGlossary: function(
		projectURI, folderURI, objectTypeURI, title, description,
		enableEditInGrid, enableFocusInGrid,
		resourceUriScope)
	{
		var deferred = new dojo.Deferred();
		var resourceObject = com.ibm.rdm.web.model.resources.ResourceManager.create(
			{entityType: MimeTypes.getArtifactFormat(MimeTypes.GLOSSARY_NAME, projectURI),
			projectURI: projectURI}
		);
		
		resourceObject.setProjectURI(projectURI);
		resourceObject.setTitle(title);
		resourceObject.addType(objectTypeURI);
		resourceObject.setDescription(description);
		
		// add the type to the resource object
		var addTypeAndCreateGlossaryPromise = resourceObject.addType(objectTypeURI).then(
			// chain the type response with the resource object save
			dojo.hitch(this, function(typeResponse) {
				var createGlossaryDeferred = resourceObject.save({
					refreshAfterSave: true,
					softRefresh: true,
					createInFolderURI: folderURI
				});
				return createGlossaryDeferred;
			})
		).then(
			// resource object save callbacks
			dojo.hitch(this, function(createResponse) {
				this.publishResourceCreatedEvent(
					enableEditInGrid, enableFocusInGrid,
					resourceObject.resourceURI, projectURI,
					resourceObject, resourceUriScope);
				return createResponse;
			})
		);
		
		return addTypeAndCreateGlossaryPromise;
	},
	
	createTerm: function(
		projectURI, folderURI, objectTypeURI, title, description,
		enableEditInGrid, enableFocusInGrid,
		resourceUriScope)
	{
		var deferred = new dojo.Deferred();
		var resourceObject = com.ibm.rdm.web.model.resources.ResourceManager.create(
			{entityType: MimeTypes.getArtifactFormat(MimeTypes.TERM_NAME, projectURI),
			projectURI: projectURI}
		);
		
		resourceObject.setProjectURI(projectURI);
		resourceObject.setTitle(title);
		resourceObject.setDescription(description);
		
		var objectType = null;
		if (typeof(objectTypeURI) == 'object') {
			for (var typeUri in objectTypeURI) {
				resourceObject.addType(typeUri);
				objectType = typeUri;
			}
		} else {
			objectType = objectTypeURI;
			resourceObject.addType(objectTypeURI);
		}
		
		// add the type to the resource object
		var addTypeAndCreateTermPromise = resourceObject.addType(objectTypeURI).then(
			// chain the type response with the resource object save
			dojo.hitch(this, function(typeResponse) {
				var createTermDeferred = resourceObject.save({
					refreshAfterSave: true,
					softRefresh: true,
					createInFolderURI: folderURI
				});
				return createTermDeferred;
			})
		).then(
			// resource object save callbacks
			dojo.hitch(this, function(createResponse) {
				this.publishResourceCreatedEvent(
					enableEditInGrid, enableFocusInGrid,
					resourceObject.resourceURI, projectURI,
					resourceObject, resourceUriScope);
				return createResponse;
			})
		);
		
		return addTypeAndCreateTermPromise;
	},
	
	createModule: function(projectURI, folderURI, objectTypeURI, title, description, enableEditInGrid, resourceUriScope) {
		var deferred = new dojo.Deferred();
		var resourceObject = com.ibm.rdm.web.model.resources.ResourceManager.create({
			entityType: MimeTypes.getArtifactFormat(MimeTypes.MODULE_NAME, projectURI),
			projectURI: projectURI,
			resourceOriginURI: folderURI
		});
		
		resourceObject.setProjectURI(projectURI);
		resourceObject.setTitle(title);
		resourceObject.setDescription(description);
		
		//52261: Server will create a default asset folder for a new Module if it is not specified by the client.
		//resourceObject.setAssetFolderURI(folderURI);
		
		var addTypeDeferred = resourceObject.addType(objectTypeURI);
		addTypeDeferred.addCallback(this,function(typeResponse){
			var createModuleDeferred = resourceObject.save({refreshAfterSave: true, softRefresh: true, forceDelayCallback: true});
			createModuleDeferred.addCallback(this, function(createResponse){
				deferred.callback(resourceObject);
				this.publishResourceCreatedEvent(
					enableEditInGrid, false,
					resourceObject.resourceURI, projectURI,
					resourceObject, resourceUriScope);
				
				//52261: Server will create a default asset folder for a new Module if it is not specified by the client.
				ChangeNotifier.notifyListeners({		
					eventTopic: com.ibm.rdm.web.notifications.EventTopics.FOLDER_CREATED,    				
					resourceURI: resourceObject.getAssetFolderURI(),
					projectURI : projectURI,
					newName: dojo.string.substitute(this._messages.moduleContentsFolder, [title]),
					newDescription: "",
					newParent: com.ibm.rdm.web.util.queryutils.getLastPathSegment(folderURI),
					isAssetFolder: true
				});	
			});
			createModuleDeferred.addErrback(this,function(createErrback){
				deferred.callback(createErrback);
				return createErrback;
			});
			return typeResponse;
		});
		addTypeDeferred.addErrback(this,function(errback){
			deferred.callback(errback);
			return errback;
		});
		return deferred;
	},

	mergePartialReqWithFullResource: function(partialReq, fullResource, args){
		var projectURI = args.projectURI,
			fullAttributes = fullResource.getAttributes(),
			removedAttributes = fullResource.getRemovedAttributes(),
			currAttributeURI,
			currentAttributeObject,
			attributeURIToUse,
			currentAttributeValue,
			primaryTextSameAsURI = RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.PrimaryText;

		// Copy all attribute values from the full resource to the partial  All attributes are copied, not just those
		// that are on the Partial Requirement because we can't tell if there is an attribute that was previously set
		// to blank, and has now been set to some value that the grid needs to display.
		// In the full resource, all attributes are stored against their project specific URIs.  In the partial resource
		// well-known attributes are stored against their sameAs URIs, except for primary text, which for some reason
		// is stored against its project-specific URI.  So when copying we have to map the URIs to the sameAs URI where
		// applicable, EXCEPT for primary text which we have to special-case!

		for (currAttributeURI in fullAttributes){
			currentAttributeObject = RRMTypes.getWellKnownAttributeObjectFromURI(currAttributeURI, projectURI);
			if (currentAttributeObject && currentAttributeObject.sameAs && currentAttributeObject.sameAs !== primaryTextSameAsURI){
				attributeURIToUse = currentAttributeObject.sameAs;
			} else {
				attributeURIToUse = currAttributeURI;
			}

			if (attributeURIToUse){
				//Clone the attribute so that any munging by the grid while being displayed does not come back to the resource itself
				currentAttributeValue = dojo.clone(fullResource.getAttributeValue(currAttributeURI));
				if (currentAttributeValue){
					partialReq.setAttribute(attributeURIToUse, currentAttributeValue);
				}
			}
		}

		// Remove any attributes that have been removed from the full resource.
		for (var nextAttributeURI in removedAttributes) {
			currentAttributeObject = RRMTypes.getWellKnownAttributeObjectFromURI(nextAttributeURI, projectURI);
			if (currentAttributeObject && currentAttributeObject.sameAs && currentAttributeObject.sameAs !== primaryTextSameAsURI) {
				attributeURIToUse = currentAttributeObject.sameAs;
			} else {
				attributeURIToUse = nextAttributeURI;
			}
			if (attributeURIToUse) {
				partialReq.removeAttribute(attributeURIToUse);
			}
		}

		if (fullResource.getBoundArtifact()){
			partialReq.setAttribute(RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.BoundArtifact, fullResource.getBoundArtifact());
		}
		//console.debug("Full resource type is ", fullResource.getTypes());
		//partialReq.setAttribute(RRMTypes.WELL_KNOWN_OBJECT_TYPE, fullResource.getType());
		//partialReq.addType(fullResource.getType());
		//partialReq._type = {};
		//partialReq._type[fullResource.getType()] = fullResource.getType()
		if (dojo.isFunction(partialReq.setType)) {
			partialReq.setType(fullResource.getType());
		}
		partialReq.setAttribute(RRMTypes.WELL_KNOWN_HAS_GENERATED_TITLE, fullResource.hasServerGeneratedTitle() ? "true" : "false");
		partialReq.setAttribute(RRMTypes.WELL_KNOWN_IS_HEADING, fullResource.isHeading() ? "true" : "false");

		if (fullResource.revision !== null){
			partialReq.revision = fullResource.revision;
		}
		
		return partialReq;
	},
	
	publishResourceCreatedEvent: function(
		enableEditInGrid, enableFocusInGrid,
		resourceURI, projectURI, resourceObject, resourceUriScope,
		/*String?*/where, /*String?*/relativeSiblingURI)
	{
		ChangeNotifier.notifyListeners({
			eventTopic: com.ibm.rdm.web.notifications.EventTopics.RESOURCE_CREATED,
			resourceURI: resourceURI,
			projectURI : projectURI,
			resourceUriScope: resourceUriScope,
			resourceObject: resourceObject,
			enableEditInGrid: enableEditInGrid,
			enableFocusInGrid: enableFocusInGrid,
			where: where,
			relativeSiblingURI: relativeSiblingURI
		});
	},
	
	createResource: function(args) {
		var artifactFormat = args.format;
		var projectURI = args.projectURI;
		var folderURI = args.folderURI;
		var typeURI = args.typeURI;
		var artifactTemplateURI = args.artifactTemplateURI;
		var relativeSiblingURI = args.relativeSiblingURI;
		var title = args.title;
		var description = args.description;
		var enableEditInGrid = args.enableEditInGrid;
		var enableFocusInGrid = args.enableFocusInGrid;
		var resourceUriScope = args.resourceUriScope;
		var moduleContextURI = args.moduleContextURI;
		var where = args.where;
		var isHeading = args.isHeading;
		var isModule = this.isModule(artifactFormat);
		var forceDelayCallback = args.forceDelayCallback;

		var deferred = null;
		if (artifactTemplateURI) {
			deferred = this._createArtifactSeededByArtifactTemplate(
				title, description, projectURI, artifactTemplateURI, folderURI,
				enableEditInGrid, enableFocusInGrid,
				resourceUriScope, isModule, moduleContextURI,
				where, relativeSiblingURI
			);
		} else {
			switch (artifactFormat) {
				case MimeTypes.getArtifactFormat(MimeTypes.REQUIREMENT_NEW_NAME, projectURI):
					this.setLastSelectedObjectType(typeURI, projectURI);
						if (args.createStub){
							deferred = this.createRequirementStub(args);
						}
						else{
							deferred = this.createRequirement(
								projectURI, folderURI, typeURI, args.primaryText,
								title, description,
								enableEditInGrid, enableFocusInGrid,
								resourceUriScope, moduleContextURI,
								isHeading, where, relativeSiblingURI,
								forceDelayCallback);
						}
					break;
				case MimeTypes.getArtifactFormat(MimeTypes.COLLECTION_NAME, projectURI):
					deferred = this.createCollection(
						projectURI, folderURI, typeURI,
						title, description, args.collectionResourcesArray,
						args.associatedSnapshot, enableEditInGrid, resourceUriScope);
					break;
				case MimeTypes.getArtifactFormat(MimeTypes.GLOSSARY_NAME, projectURI):
					deferred = this.createGlossary(
						projectURI, folderURI, typeURI, title, description,
						enableEditInGrid, enableFocusInGrid, resourceUriScope);
					break;
				case MimeTypes.getArtifactFormat(MimeTypes.TERM_NAME, projectURI):
					deferred = this.createTerm(
						projectURI, folderURI, typeURI, title, description,
						enableEditInGrid, enableFocusInGrid,
						resourceUriScope);
					break;
				case MimeTypes.getArtifactFormat(MimeTypes.MODULE_NAME, projectURI):
					deferred = this.createModule(
						projectURI, folderURI, typeURI, title,
						description, enableEditInGrid, resourceUriScope);
					break;
				default:
					this.setLastSelectedObjectType(typeURI, projectURI);
					var artifactHandler =
						ArtifactExtension.getArtifactHandler(artifactFormat, projectURI);
					if (artifactHandler !== null) {
						deferred = artifactHandler.create(
							projectURI, folderURI, typeURI, title, description,
							enableEditInGrid, enableFocusInGrid,
							args.rrcxCreateOptions, resourceUriScope, moduleContextURI, 
							isHeading, where, relativeSiblingURI);
					}
					else {
						deferred = this.createRequirement(
							projectURI, folderURI, typeURI, args.primaryText,
							title, description,
							enableEditInGrid, enableFocusInGrid,
							resourceUriScope, moduleContextURI,
							isHeading, where, relativeSiblingURI,
							forceDelayCallback
						);
					}
				break;
			}
		}
		return deferred;
	},
	
	setLastSelectedObjectType: function(objectTypeURI, projectURI) {
		if (objectTypeURI) {
			com.ibm.rdm.web.LAST_SELECTED_OBJECT_TYPES[projectURI] = objectTypeURI;
		}
	},

	getLastSelectedObjectType: function(projectURI) {
		var lastSelectedObjectType = com.ibm.rdm.web.LAST_SELECTED_OBJECT_TYPES[projectURI];
		return lastSelectedObjectType;
	},
	
	setLastSelectedArtifactTemplateForObjectTypeAndArtifactFormat: function(
			objectTypeURI, artifactFormatURI, projectURI, artifactTemplateURI) {
		// summary:
		//		Store away in a cookie the fact that when the user selects a
		//		particular artifact type and artifact format, a particular
		//		artifact template was ALSO selected.
		this._initProjectArtifactTemplatesIfNeeded(projectURI);
		var compositeTypeKey = this._getCompositeTypeKeyToUse(objectTypeURI, artifactFormatURI);
		var mapCompositeTypeKeyToToArtifactTemplateUri =
			MAP_PROJECT_URI_TO_MAPS_OF_COMPOSITE_TYPE_KEY_TO_ARTIFACT_TEMPLATE_URI[projectURI];
		if (!mapCompositeTypeKeyToToArtifactTemplateUri) {
			mapCompositeTypeKeyToToArtifactTemplateUri =
				MAP_PROJECT_URI_TO_MAPS_OF_COMPOSITE_TYPE_KEY_TO_ARTIFACT_TEMPLATE_URI[projectURI] = {};
		}
		mapCompositeTypeKeyToToArtifactTemplateUri[compositeTypeKey] = artifactTemplateURI;
		this._saveCookieProjectArtifactTypeMappings(projectURI);
	},

	getLastSelectedArtifactTemplateForObjectTypeAndArtifactFormat: function(
			objectTypeURI, artifactFormatURI, projectURI) {
		// summary:
		//		Return the last specified artifact template value used for
		//		a particular artifact type / artifact format pair.
		this._initProjectArtifactTemplatesIfNeeded(projectURI);
		var compositeTypeKey = this._getCompositeTypeKeyToUse(objectTypeURI, artifactFormatURI);
		var mapCompositeTypeKeyToToArtifactTemplateUri =
			MAP_PROJECT_URI_TO_MAPS_OF_COMPOSITE_TYPE_KEY_TO_ARTIFACT_TEMPLATE_URI[projectURI];
		return (
			mapCompositeTypeKeyToToArtifactTemplateUri ?
			mapCompositeTypeKeyToToArtifactTemplateUri[compositeTypeKey] :
			null
		);
	},

	getObjectTypeAndArtifactTypePairsContainingArtifactTemplateMappings: function(projectURI) {
		// summary:
		//		Return information about which artifact type URI / artifact
		//		format URI pairs contain a mapping to an artifact template URI.
		// returns:
		//		An array of JSON objects, each of the following format:
		//		    {
		//		        objectTypeURI: <objectTypeURI>,
		//		        artifactFormatURI: <artifactFormatURI>
		//		    }
		//		There will be an entry in this array for every previously applied
		//		artifactTypeURI/artifactFormatURI to artifactTemplateURI mapping.
		var typePairsContainingMappings = [];
		var mapCompositeTypeKeyToToArtifactTemplateUri =
			MAP_PROJECT_URI_TO_MAPS_OF_COMPOSITE_TYPE_KEY_TO_ARTIFACT_TEMPLATE_URI[projectURI];
		for (var compositeTypeKey in mapCompositeTypeKeyToToArtifactTemplateUri) {
			// Note: The mapped value may be 'null' so we need the following 'if' ...
			if (mapCompositeTypeKeyToToArtifactTemplateUri[compositeTypeKey]) {
				var typeAndFormatPair =
					this._getTypeAndFormatFromCompositeTypeKey(compositeTypeKey);
				typePairsContainingMappings.push(typeAndFormatPair);
			}
		}
		return typePairsContainingMappings;
	},
	
	clearLastSelectedArtifactTypeAndArtifactFormatToArtifactTemplateMappingIfNeeded: function(
			artifactTypeURI, artifactFormatURI, artifactTemplateURI, projectURI) {
		// summary:
		//		If there presently exists a "last selected" mapping between
		//		a particular artifact type / artifact format pair and a
		//		particular artifact template, remove this mapping.

		var currentMappingForArtifactTemplateURI =
			this.getLastSelectedArtifactTemplateForObjectTypeAndArtifactFormat(
				artifactTypeURI, artifactFormatURI, projectURI
			);
		if (currentMappingForArtifactTemplateURI === artifactTemplateURI) {
			this.setLastSelectedArtifactTemplateForObjectTypeAndArtifactFormat(
				artifactTypeURI, artifactFormatURI, projectURI, null
			);
		}
	},

	getArtifactCollectionMembership: function(artifactURI, /* Optional */ skipCache) {
		// Summary:
		//		Get the collections of which this artifact is a member
		// artifactURI:
		//		The URI of the artifact for which we want collection membership.
		// skipCache:
		//		Whether we should skip the internal cache. Defaults to true.
		
		skipCache = (skipCache == "undefined") || skipCache;
		
		var queryString = "query=rrmCollection:artifact=<" + artifactURI 
			+ ">&namespaces=rrmCollection=http://jazz.net/xmlns/alm/rm/Collection/v1.0/";
		
		var collectionMembershipDeferred = RDMClient.doPost({
			uri: com.ibm.rdm.web.FRONTING_SERVICE_QUERY,
			postData: queryString,
			delayCallbackUntilDataIndexed: false,
			parser: function(response){
				
				var entryParser = new com.ibm.rdm.web.parsers.xml2json(response.doc);
				var parsedEntries = entryParser.parse().feed;
				
				var collectionURIs = [];
				
				for(var node in parsedEntries) {
					if(node.indexOf("entry") > -1){
						collectionURIs.push(parsedEntries[node].content.result.binding.uri.value);
					}
				}
				
				return collectionURIs;
			}
		});
		
		return collectionMembershipDeferred;
	},

	_createArtifactSeededByArtifactTemplate: function(
		title, description,
		projectURI, artifactTemplateURI, folderURI,
		enableEditInGrid, enableFocusInGrid,
		resourceUriScope, isModule, moduleContextURI,
		where, /*String?*/relativeSiblingURI)
	{
		var deferred = new dojo.Deferred();
		var self = this;
		var postData = (
			"title=" + encodeURIComponent(title)
			+ "&description=" + encodeURIComponent(description)
		);
		
		if (isModule) {
			 postData += "&targetFolder="+encodeURIComponent(folderURI);
		} else if (folderURI) {
			postData += "&assetFolder=" + encodeURIComponent(folderURI);
		}

		var getRelativeTargetPosition = dojo.hitch(this, function(where) {
			switch(where) {
			case "below":
				return RRMTypes.MODULE_RELATIVE_TARGET_POSITION.BELOW;
			case "before":
				return RRMTypes.MODULE_RELATIVE_TARGET_POSITION.BEFORE;
			case "after":
				return RRMTypes.MODULE_RELATIVE_TARGET_POSITION.AFTER;
			// Note by SPC: This code was originally written to return the value
			// for below when 'where' was set to either "last" or undefined, so
			// we're making sure to do that here ...
			case "last":
			default:
				return RRMTypes.MODULE_RELATIVE_TARGET_POSITION.BELOW;
			}
		});
		
		//TODO: artifacts created in modules using one-click or dialog are always added 
		// to the bottom (below root node) as these actions are not contextual
		// If the behavior ever changes, this part of the code must adopt it as well 
		//50596: As a user, I will be able to create requirements in module using artifact templates
		if (moduleContextURI) {
			postData += (
				"&destinationBinding=" + encodeURIComponent(relativeSiblingURI || moduleContextURI) +
				"&relativeTargetPosition=" + encodeURIComponent(getRelativeTargetPosition(where))//after, before, below. Default value is below.
			);
		}
		
		var postDeferred = RDMClient.doPost({
			uri: artifactTemplateURI + "?apply",
			postData: postData,
			projectURI: projectURI,
			xhrArgs: {
				headers: {
					"Content-Type":	"application/x-www-form-urlencoded"
				}
			},
			delayCallbackUntilDataIndexed: true
		});
		postDeferred.addCallback(this, function(response) {
			var taskTrackerURI = XhrUtils.getHeader(response, "Location");
			var onTaskResultComplete = dojo.hitch(this, function(taskResultURI){
				// Note: We pass 'true' for the last param to _performMoveOperation
				// as we need to wait for the new resource to be indexed BEFORE
				// we subsequently make a ResourceManager.get call.
				// Originally this code was doing the ResourceManager.get call before
				// the move and this resulted in the ResourceManager.get finding that
				// the resource hasn't been indexed yet.
				var resourceDeferred =
					com.ibm.rdm.web.model.resources.ResourceManager.get({
						resourceURI: taskResultURI,
						projectURI: projectURI,
						skipCache: true
					});
				resourceDeferred.addCallback(this, function(resourceObject) {
					deferred.callback(resourceObject);
					this.publishResourceCreatedEvent(
						enableEditInGrid, enableFocusInGrid,
						resourceObject.resourceURI, projectURI,
						resourceObject, resourceUriScope,
						where, relativeSiblingURI
					);
				});
				resourceDeferred.addErrback(function(response) {
					deferred.errback(response);
					return response;
				});
			});
			var onTaskResultError = dojo.hitch(this, function(){
				deferred.errback();
			});
			this._getTaskTrackerTaskResult(
				taskTrackerURI, onTaskResultComplete, onTaskResultError
			);
			return response;
		});
		postDeferred.addErrback(this, function(response) {
			deferred.errback(response);
			return response;
		});
		return deferred;
	},

	_getTaskTrackerTaskResult: function(
		taskTrackerURI, onTaskResultComplete, onTaskResultError)
	{
		// TODO Do we need to inform the "Create an artifact with attributes"
		// of intermediate results?
		var taskProgressNotifier = new TaskProgressNotifier(),
			requestTaskProgressNotification;
		var onComplete = function(taskResultURI) {
			onTaskResultComplete(taskResultURI);
		};
		var onIncomplete = function() {
			// Re-issue a progress request (as the task hasn't completed yet) ...
			window.setTimeout(requestTaskProgressNotification, 250);
		};
		requestTaskProgressNotification = function() {
			var deferred = taskProgressNotifier.notifyProgressWithCompletion(taskTrackerURI, true);
			deferred.addCallback(this, function(response) {
				var taskProgressMessage = response.taskProgressMessage;
				var taskResultURI = response.taskResultURI;
				if (taskResultURI) {
					onComplete(taskResultURI);
				}
				else if (taskProgressMessage
						 && taskProgressMessage.toLowerCase().indexOf("error"))
				{
					// Note: According to WI 59693 Comment 5 looking for
					// "error" (any case) in the "taskProgress" XML element
					// (captured here by taskProgressMessage) is the proper
					// way to account for such things as NPE exceptions that
					// are inappropriately raised the server Java code.
					onIncomplete();
				}
				else {
					onTaskResultError();
				}
				return response;
			});
		};
		// Give the server a little initial time before asking for an update.
		window.setTimeout(requestTaskProgressNotification, 250);
	},

	_initProjectArtifactTemplatesIfNeeded: function(projectURI) {
		var mapCompositeTypeKeyToToArtifactTemplateUri =
			MAP_PROJECT_URI_TO_MAPS_OF_COMPOSITE_TYPE_KEY_TO_ARTIFACT_TEMPLATE_URI[projectURI];
		if (!mapCompositeTypeKeyToToArtifactTemplateUri) {
			mapCompositeTypeKeyToToArtifactTemplateUri = (
				this._getCookieProjectMapOfObjectTypeUriToArtifactTemplateUri(projectURI)
				|| {}
			);
			MAP_PROJECT_URI_TO_MAPS_OF_COMPOSITE_TYPE_KEY_TO_ARTIFACT_TEMPLATE_URI[projectURI] =
				mapCompositeTypeKeyToToArtifactTemplateUri;
		}
	},

	_getCompositeTypeKeyToUse: function(objectTypeURI, artifactFormatURI) {
		return (objectTypeURI + " " + artifactFormatURI);
	},

	_getTypeAndFormatFromCompositeTypeKey: function(compositeTypeKey) {
		var spaceIndex = compositeTypeKey.indexOf(" ");
		var objectTypeURI = compositeTypeKey.substring(0, spaceIndex);
		var artifactFormatURI = compositeTypeKey.substring(spaceIndex+1);
		return {
			objectTypeURI: objectTypeURI,
			artifactFormatURI: artifactFormatURI
		};
	},

	_saveCookieProjectArtifactTypeMappings: function(projectURI) {
		// summary:
		//		Save a project specific cookie representing the user's default
		//		artifact template to use for each of the project's artifact
		//		types.
		// projectURI:
		//		The project URI of the project for which we're to create/update
		//		a cookie for.
		var mapCompositeTypeKeyToToArtifactTemplateUri =
			MAP_PROJECT_URI_TO_MAPS_OF_COMPOSITE_TYPE_KEY_TO_ARTIFACT_TEMPLATE_URI[projectURI];
		var cookieName = ARTIFACT_TEMPLATES_COOKIE_NAME_PREFIX + projectURI;
		var cookieValue = "";
		var isFirst = true;
		for (var compositeTypeKey in mapCompositeTypeKeyToToArtifactTemplateUri) {
			var artifactTemplateURI =
				mapCompositeTypeKeyToToArtifactTemplateUri[compositeTypeKey];
			if (artifactTemplateURI) {
				var typeAndFormatPair =
					this._getTypeAndFormatFromCompositeTypeKey(compositeTypeKey);
				var abbreviatedObjectTypeURI =
					queryutils.getLastPathSegment(typeAndFormatPair.objectTypeURI);
				var abbreviatedArtifactFormatURI =
					queryutils.getLastPathSegment(typeAndFormatPair.artifactFormatURI);
				var abbreviatedArtifactTemplateURI =
					queryutils.getLastPathSegment(artifactTemplateURI);
				var valueToStore = (
					abbreviatedObjectTypeURI
					+ " " + abbreviatedArtifactFormatURI
					+ " " + abbreviatedArtifactTemplateURI
				);
				if (isFirst) {
					cookieValue += valueToStore;
					isFirst = false;
				} else {
					cookieValue += " " + valueToStore;
				}
			}
		}
		com.ibm.rdm.web.util.storage.set({
			"key" : cookieName,
			"data" : cookieValue,
			"cookieFallback" : true,
			"cookieData" : {expires:365}
		});
	},

	_getCookieProjectMapOfObjectTypeUriToArtifactTemplateUri: function(projectURI) {
		// summary:
		//		Get the project specific mapping of artifact type URI to
		//		artifact template URI, representing the user's last applied
		//		artifact-type/artifact-template value in the "Create artifact
		//		with attributes" dialog.
		var cookieName = ARTIFACT_TEMPLATES_COOKIE_NAME_PREFIX + projectURI;
		var cookieValue = com.ibm.rdm.web.util.storage.get({
			"key" : cookieName,
			"cookieFallback" : true
		});
		var mapCompositeTypeKeyToToArtifactTemplateUri = null;
		if (cookieValue) {
			mapCompositeTypeKeyToToArtifactTemplateUri = {};
			var tokens = cookieValue.split(" ");
			for (var i = 0; i < tokens.length;) {
				var objectTypeURI =
					com.ibm.rdm.web.FRONTING_SERVICE_TYPES + "/" + tokens[i++];
				var artifactFormatURI =
					com.ibm.rdm.web.FRONTING_SERVICE_TYPES + "/" + tokens[i++];
				var artifactTemplateURI =
					com.ibm.rdm.web.FRONTING_SERVICE_TEMPLATES + "/" + tokens[i++];
				var compositeTypeKey =
					this._getCompositeTypeKeyToUse(objectTypeURI, artifactFormatURI);
				mapCompositeTypeKeyToToArtifactTemplateUri[compositeTypeKey] = artifactTemplateURI;
			}
		}
		return mapCompositeTypeKeyToToArtifactTemplateUri;
	},
	
	getModules: function (resourceUris, callbackFn) {
		if (!com.ibm.rdm.web.MODULE_SUPPORT || !resourceUris || resourceUris.length == 0) {
			callbackFn({});
			return;
		}
		
		var deferred = com.ibm.rdm.web.query.ViewServiceClient.getViewContents({
			requestedAttributeUris: ["http://purl.org/dc/terms/title"],
			pageSize: 512,
			artifactUris: resourceUris
		});
		deferred.addCallback(this, function (response) {
			var result = com.ibm.rdm.web.query.ViewServiceClient.parseViewContentsResponse(response, null, null, false);
			var resources = result.resources;
			//convert to map
			var modulesMap = {};
			for (var i=0; i<resources.length; i++) {
				modulesMap[resources[i].getResourceURI()] = resources[i];
			}
			//Call the callback fn
			callbackFn(modulesMap);
		});
	},
	
	getModulesForProject: function (projectURI) {
		var attributeConditionInfos = [{
			attributeDefinitionURI: RRMTypes.getWellKnownAttributeURI(projectURI, RRMTypes.WELL_KNOWN_ATTRIBUTE_NAMES.ArtifactFormat),
			attributeType: RRMTypes.ATTRIBUTE_TYPES.ArtifactFormatsType,
			attributeValues: ["is", RRMTypes.getArtifactFormats(projectURI)["Module"]],
			isEnum: true
		}];
			
		var deferred = com.ibm.rdm.web.query.ViewServiceClient.getViewContents({
			requestedAttributeUris: ["http://purl.org/dc/terms/title"],
			attributeConditionInfos: attributeConditionInfos,
			projectURI: projectURI
		});
		deferred.addCallback(this, function (response) {
			var result = com.ibm.rdm.web.query.ViewServiceClient.parseViewContentsResponse(response, null, null, false);
			return result.resources;
		});
		return deferred;
	},
	
	//gets all modules where artifact(s) is reused
	//View service will only honor request for one URI at present - (ebcraig - defect 65289)
	getModulesForBoundArtifacts: function (resourceUri, callbackFn, additionalRequestParams) {
		if (!com.ibm.rdm.web.MODULE_SUPPORT || !resourceUri) {
			callbackFn({});
			return;
		}
		
		if (dojo.isArray(resourceUri) && (resourceUri.length > 0)) {
			resourceUri = resourceUri[0];
		}
		
		var requestedAttrs = {
			requestedAttributeUris: [
				com.ibm.rdm.web.model.types.RRMTypes.WELL_KNOWN_MODULE_TYPE,
				com.ibm.rdm.web.model.types.RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.BoundArtifact,
				[
					com.ibm.rdm.web.model.types.RRMTypes.WELL_KNOWN_MODULE_TYPE,
					com.ibm.rdm.web.model.types.RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.Title
				],
				[
					com.ibm.rdm.web.model.types.RRMTypes.WELL_KNOWN_MODULE_TYPE,
					com.ibm.rdm.web.model.types.RRMTypes.WELL_KNOWN_SAME_AS_ATTRIBUTE_NAMES.Identifier]
				],
			pageSize: 512,
			moduleBoundArtifact: resourceUri
		};
		
		if (additionalRequestParams) {
			if (additionalRequestParams.requestedAttributeUris) {
				requestedAttrs.requestedAttributeUris = dojo.mixin(additionalRequestParams.requestedAttributeUris, requestedAttrs.requestedAttributeUris);
			}
			requestedAttrs = dojo.mixin(additionalRequestParams, requestedAttrs);
		}
		
		//Defect 55856 Performance problem if count=true. Force no count when getting the modules for bound artifacts.
		requestedAttrs.forceNoCount= true;
		var deferred = com.ibm.rdm.web.query.ViewServiceClient.getViewContents(requestedAttrs);
		deferred.addCallback(this, function (response) {
			var result = com.ibm.rdm.web.query.ViewServiceClient.parseViewContentsResponse(response, null, null, false);
			var resources = result.resources;
			//convert to map
			var moduleArtifactsMap = {};
			for (var i=0; i<resources.length; i++) {
				
				var boundArtifactURI =  resourceUri;
				if (!moduleArtifactsMap[boundArtifactURI]) {
					moduleArtifactsMap[boundArtifactURI] = [];
				}
				moduleArtifactsMap[boundArtifactURI].push(resources[i]);
			}
			//Call the callback fn
			callbackFn(moduleArtifactsMap);
		});
	},
	
	getAssociatedModuleTemplates: function (resourceURI, projectURI) {
		// summary:
		//		Asks the server if the module specified by resourceURI has any associated
		//		templates, deleting them if there are.
		var uri = com.ibm.rdm.web.FRONTING_SERVICE_TEMPLATES;
		var content = {
			resourceContext: projectURI,
			artifact: resourceURI
		};

		return RDMClient.doGet({
			uri: uri,
			skipCache: true,
			xhrArgs: {
				content: content
			}
		});
	}
};

})();
