/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2014. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.defects.web.ui.internal.DefectClmLinksHelper");

dojo.require("com.ibm.rqm.defects.web.ui.internal.ClmLinksHelper");
dojo.require("com.ibm.rqm.defects.web.client.ChangeRequestManager");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.defects.web.helper.RemoteItemPicker");
dojo.require("com.ibm.rqm.defects.web.util.ItemPickerUtil");
dojo.require("com.ibm.rqm.defects.web.ui.view.common.ProgressDialog");
dojo.require("com.ibm.rqm.execution.web.ui.internal.view.common.NewDefectDataInitializer");
dojo.require("com.ibm.rqm.defects.web.ui.view.common.CQRecordTypeSelectionDialog");
dojo.require("com.ibm.rqm.defects.web.client.internal.CmIntegrationClient");
dojo.require("com.ibm.rqm.defects.web.ui.utils.DefectsUIUtil");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");
dojo.require("com.ibm.asq.common.web.ui.internal.util.Utility");

(function() {

var Constants                       = com.ibm.rqm.planning.web.client.internal.Constants;
var RemoteItemPicker				= com.ibm.rqm.defects.web.helper.RemoteItemPicker;
var ItemPickerUtil					= com.ibm.rqm.defects.web.util.ItemPickerUtil;
var ProgressDialog 					= com.ibm.rqm.defects.web.ui.view.common.ProgressDialog;
var NewDefectDataInitializer        = com.ibm.rqm.execution.web.ui.internal.view.common.NewDefectDataInitializer;
var CmIntegrationClient 			= com.ibm.rqm.defects.web.client.internal.CmIntegrationClient;
var DefectsUIUtil 					= com.ibm.rqm.defects.web.ui.utils.DefectsUIUtil;
var Utility							= com.ibm.asq.common.web.ui.internal.util.Utility;

/**
 * Defect CLM link specific class. This class supplies all of the defect link's type specific attributes.
 */
dojo.declare("com.ibm.rqm.defects.web.ui.internal.DefectClmLinksHelper", 
             com.ibm.rqm.defects.web.ui.internal.ClmLinksHelper, 
{
    //@Overrides from ClmLinksHelper
    getServiceProviderManager: function(){
        // TODO let OSLCServicesDocUtil determine the correct OSLC spec version
        return com.ibm.rqm.defects.web.client.ChangeRequestManager.getInstance();
    },
    
    //@Overrides ClmLinksHelper
    getResourceSelectDialog: function(provider){
        var defectSelectionDialog = provider.services.getSelectionDialog("defect", true);
        return defectSelectionDialog;
    },
    
    // @Overrides ClmLinksHelper
    parsePickerResults: function(provider, selectedItems){
        var oslcCoreVersion = provider.getOSLCCoreVersion();
        var workItems = [];
        if (oslcCoreVersion == "2.0") {
            workItems = ItemPickerUtil.parseV2(selectedItems);
        }
        else if (oslcCoreVersion == "1.0") {
            workItems = ItemPickerUtil.parse(selectedItems);
        }
        return workItems;
    },
    
    createDefectLink: function(stepModel, createNew){
        var serviceProviderManager = this.getServiceProviderManager();
        
        serviceProviderManager.getCurrentServiceProvider(dojo.hitch(this, function(currentServiceProvider){
            if (currentServiceProvider) {
            	if(createNew) { // Display New Defect picker
            		this.showNewDefectDialog(currentServiceProvider, stepModel);
            	} else { // Display Existing Defect picker
            		this.showSelectDefectDialog(currentServiceProvider, stepModel);
            	}
            }
            else {
                // No service provider is configured.
            	this._displayAlertDialog(this.getProviderNotConfiguredMessage());
            }
        }));
    },
    
    showSelectDefectDialog: function(currentServiceProvider, stepModel) {
    	var self = this;
    	var selectionDialog = this.getResourceSelectDialog(currentServiceProvider);
        if (selectionDialog) {
            var selectionDialogUri = selectionDialog.uri;
            // select an item from the selection dialog
            var params = {
                dialogTitle: Constants.getMessage("labelAddExistingDefect"),
                secondaryTitle: " ",
                pickerURL: selectionDialogUri,
                height: selectionDialog.height,
                width: selectionDialog.width,
                onOk: function(selectedItems){
                    var resources = self.parsePickerResults(currentServiceProvider, selectedItems);
                    var recordData = stepModel.executionRecordData;
                    NewDefectDataInitializer.getDefectDescription(recordData, this.executionResultGroupId, null, currentServiceProvider, dojo.hitch(this, function(desc) {
        				desc = self._createDefectDescription(desc, stepModel);
        				desc = self._replaceDelimiter(desc, currentServiceProvider.isRTC());
        				self.handleInsertDefectLink(stepModel, desc, resources, currentServiceProvider);
                    }));
                },
                onClose: function(){
                }
            };
            params.OSLCCoreVersion = currentServiceProvider.getOSLCCoreVersion();
            new RemoteItemPicker(params);
        }
        else {
            // service provider does not provide a picker
        	this._displayAlertDialog(DefectsUIUtil.getMessage("msgProvidersDoesNotProvideDialog"));
        }
    },
    
    showNewDefectDialog: function(currentProvider, stepModel){

		//For CQ, need to let user select record type first.
		if(dojo.isFunction(currentProvider.isCQ) && currentProvider.isCQ()){
			this._selectCQRecordType(currentProvider, stepModel);
		}else{
			this._createNewDefectDialog(null, currentProvider, null, stepModel, -1);
		}

	},
	
	_createNewDefectDialog: function(cqMonitor, currentProvider, creationDialog, stepModel, headlineMaxSize){
		if(cqMonitor){
			cqMonitor.done();
		}
		// Including script name and Id to make it available for Defect description.
		var recordData = stepModel.executionRecordData;
		NewDefectDataInitializer.scriptName = this.script.name;
		NewDefectDataInitializer.scriptId = this.script.id;
		NewDefectDataInitializer.getDefectDescription(recordData, this.executionResultGroupId, null, currentProvider, dojo.hitch(this, function(desc) {
			var testCaseName = recordData.testCase.name;
			var recordName = recordData.name;
			var summary = recordName != testCaseName ? 
					Constants.getMessage("labelFailingTestCaseWithEWI", testCaseName, recordName) : 
					Constants.getMessage("labelFailingTestCaseWithoutEWI", testCaseName) ;
            
           	if(headlineMaxSize && headlineMaxSize > 0){
            	//In case the summary is longer than the max size, will change the message to labelFailingTestCaseWithoutEWI and have a try
            	if(summary.length > headlineMaxSize){
            		summary = Constants.getMessage("labelFailingTestCaseWithoutEWI", testCaseName);
            		var plainText = Constants.getMessage("labelFailingTestCaseWithoutEWI");
            		
            		var summarylength = summary.length;
            		//for the case we can not even render the labelFailingTestCaseWithoutEWI string, will give up the summary
            		if((plainText.length + 2) > headlineMaxSize && summarylength > headlineMaxSize){
            			summary = "";
            		}else if(summarylength > headlineMaxSize){
            			summary = Constants.getMessage("labelFailingTestCaseWithoutEWI", testCaseName.substring(0, headlineMaxSize - plainText.length - 1) + "...");
            		} 
            	}
            }
            
            desc = this._createDefectDescription(desc, stepModel);
			desc = this._replaceDelimiter(desc, currentProvider.isRTC());
			
			if(!creationDialog){
				creationDialog = currentProvider.services.getCreationDialog("defect", true);
			}
      		var creationUrl = creationDialog.uri;
            if (creationUrl.indexOf("?") > -1) {
               	creationUrl = creationUrl + "&" + ItemPickerUtil.getWICreationDialogParamString(summary, desc);
            }
			this._retrievePrefillingUriAndOpenCreationDialog(currentProvider, creationDialog, summary, desc, creationUrl, stepModel);
		}));
	},
	
	_retrievePrefillingUriAndOpenCreationDialog: function(currentProvider, creationDialog, summary, description, defaultCreationUrl, stepModel){
		var postData = {};
		var oslcCoreVersion = currentProvider.getOSLCCoreVersion();
		
		if(oslcCoreVersion == "2.0"){
			postData = {
				"dcterms:title": summary,
				"dcterms:description": description
			};
			
			var creationURI = currentProvider.providerUris.defaultCreationDialogURI;
			if(dojo.isFunction(currentProvider.isCQ) && currentProvider.isCQ()){
				creationURI = creationDialog.uri;
			}
			
			currentProvider.getPreFilledChangeRequestUri(creationURI, postData, 
				dojo.hitch(this, function(url){
					//need to let user manually shorten the url under this situation
					// 1. Using IE
					// 2. Using CQ as CCM provider &&
					// 3. The pre-filling url is too long, and CQ return the url without any parameters
					if(dojo.isIE && currentProvider.isCQ() && url && url.indexOf("?") == -1){
						this.openWarningTooLongUrlDialog(currentProvider, creationDialog, summary, description, url, defaultCreationUrl, stepModel);
					}else{
						this.openCreationDialog(currentProvider, creationDialog, url, stepModel, description);
					}
				}), dojo.hitch(this, function(error){
					this.openCreationDialog(currentProvider, creationDialog, defaultCreationUrl, stepModel, description);
				})
			);
		}else if(oslcCoreVersion == "1.0"){
			//RTC doesn't implement the Post Pre-filling Creation Dialog in OSLC CM 1.0, so use the url parameter to transport the summary and description string.
			this.openCreationDialog(currentProvider, creationDialog, defaultCreationUrl, stepModel, description);
		}
	},
	
	openWarningTooLongUrlDialog: function(currentProvider, creationDialog, summary, description, cqCreationUrl, defaultCreationUrl, stepModel){
		//As CQ is using different encoding strategy as RQM, so the encoded url might be different with RQM's,
		//Based on this, need to check whether the RQM encoded url does not exceed the max size.
		//If not exceed, just use this url to create the defect.
		if(defaultCreationUrl && defaultCreationUrl.length <= IE_MAX_URL_SIZE){
			this.openCreationDialog(currentProvider, creationDialog, defaultCreationUrl, stepModel, description);
			return;
		}
		
		var warningContentDiv = dojo.create("div", {"class":"dialog-content-div"});
		warningContentDiv.appendChild(document.createTextNode(Constants.getMessage("msgCQUrlIsTooLong")));
		
		var args = {
            zIndex: 998,
            forceZIndex: true,
            okLabel: Constants.getMessage("labelYes"),
            cancelLabel: Constants.getMessage("labelNo"),        
            
            //if user decide to shorten the url, then pop up a dialog for user to edit the summary and description
            _yesAction: dojo.hitch(this, function(){
            	this.floatingPane.close();  
            	this.openShortenUrlDialog(currentProvider, creationDialog, summary, description, stepModel);
            }),
            
            //if user decide not to shorten the url, just use the URL that returned from CQ to generate the defect
            _noAction: dojo.hitch(this, function(){
            	this.floatingPane.close();
            	this.openCreationDialog(currentProvider, creationDialog, cqCreationUrl, stepModel, description);
            })
        };
        
        this.floatingPane = new com.ibm.asq.common.web.ui.internal.widgets.SelectionDialog(args);
        this.floatingPane.contentContainer.appendChild(warningContentDiv);
        this.floatingPane.open(false);
		this.floatingPane._yesButton.disabled = false;
        var span = dojo.create("span");
        span.appendChild(document.createTextNode(Constants.getMessage("titleWarning")));
        this.floatingPane._headerText.appendChild(span);
        this.floatingPane.domNode.style.width = "40em";
	},
	
	openShortenUrlDialog: function(currentProvider, creationDialog, summary, description, stepModel){
		if(!summary){
			summary = "";
		}
		
		if(!description){
			description = "";
		}
		
		var contentDiv = dojo.create("div", {"class":"dialog-content"});
		var contentTable = dojo.create("table", {"class":"dialog-content"}, contentDiv);
		
		var tableRow = this._createDefectInformationTableRow(contentTable);
		tableRow.firstColumn.appendChild(document.createTextNode(Constants.getMessage("labelTotalSize")));
		var sizeFont = dojo.create("font", {"color":"red"});
		sizeFont.appendChild(document.createTextNode(summary.length + description.length));
		tableRow.secondColumn.appendChild(sizeFont);
		
		var tableRow = this._createDefectInformationTableRow(contentTable);
		var headlineInput = dojo.create("input", {"value": summary, "class":"headline-input"});
		tableRow.firstColumn.appendChild(document.createTextNode(Constants.getMessage("labelSummary")));
		tableRow.secondColumn.appendChild(headlineInput);
		
		var tableRow = this._createDefectInformationTableRow(contentTable);
		var descriptionInput = dojo.create("textarea", {"rows":"10","value": description, "cols":"60"});
		tableRow.firstColumn.appendChild(document.createTextNode(Constants.getMessage("labelDescription")));
		tableRow.secondColumn.appendChild(descriptionInput);
		
		var checkUrlLength = dojo.hitch(this, function(){
			
			var summaryValue = headlineInput.value;
			var descriptionValue = descriptionInput.value;
			if(!summaryValue){
				summaryValue = "";
			}
			if(!descriptionValue){
				descriptionValue = "";
			}
			var url = this.generatePrefillingUrl4CQ(currentProvider, creationDialog, summaryValue, descriptionValue);
			if(url && url.length <= IE_MAX_URL_SIZE){
				this.floatingPane._yesButton.disabled = false;
				sizeFont.color = "green";
			}else{
				this.floatingPane._yesButton.disabled = true;
				sizeFont.color = "red";
			}
			dojo.empty(sizeFont);
			sizeFont.appendChild(document.createTextNode(summaryValue.length + descriptionValue.length));
		});
		
		dojo.connect(headlineInput, "onkeyup", null, checkUrlLength);
		dojo.connect(descriptionInput, "onkeyup", null, checkUrlLength);

		var args = {
            zIndex: 998,
            forceZIndex: true,  
            
            _yesAction: dojo.hitch(this, function(){
            	this.floatingPane.close(); 
            	var summaryValue = headlineInput.value;
            	var descriptionValue = descriptionInput.value;
            	if(!summaryValue){
            		summaryValue = "";
            	}
            	if(!descriptionValue){
            		descriptionValue = "";
            	}
            	this.openCreationDialog(currentProvider, creationDialog, 
            			this.generatePrefillingUrl4CQ(currentProvider, creationDialog, summaryValue, descriptionValue), stepModel, descriptionValue);
            })
        };
        
        this.floatingPane = new com.ibm.asq.common.web.ui.internal.widgets.SelectionDialog(args);
        this.floatingPane.contentContainer.appendChild(contentDiv);
        this.floatingPane.open(false);
        var span = dojo.create("span");
        span.appendChild(document.createTextNode(Constants.getMessage("titleShortenCQDefectDescription")));
        this.floatingPane._headerText.appendChild(span);
        this.floatingPane.domNode.style.width = "40em";
	},
	
	openCreationDialog: function(currentProvider, creationDialog, creationUrl, stepModel, desc){
		if(!creationDialog){
			creationDialog = currentProvider.services.getCreationDialog("defect", true);
		}
		var self = this;
		var params= {
	    	dialogTitle: Constants.getMessage("labelAddNewDefect"),
	        secondaryTitle: " ",
			pickerURL: creationUrl,
			height: creationDialog.height,
			width: creationDialog.width,
			onOk: dojo.hitch(this, function(result) {
				var workItems = self.parsePickerResults(currentProvider, result);
				self.handleInsertDefectLink(stepModel, desc, workItems, currentProvider);
			}),
			onClose: function() {
				console.debug("close!");
			}
	
		};	  
		params.OSLCCoreVersion = currentProvider.getOSLCCoreVersion();          
	    new RemoteItemPicker(params);
	},
	
	_selectCQRecordType: function(currentProvider, stepModel){
		
		var okCallback = dojo.hitch(this, function(currentProvider, creationDialog){
			ProgressDialog.doRunWithProgress(Constants.getMessage("msgRetrieveCQresource"), dojo.hitch(this, function(monitor){
				monitor.beginTask(Constants.getMessage("msgRetrieveCQresource"));
				var resourceShapes = currentProvider.services.resourceShapes ? currentProvider.services.resourceShapes : [];
				var queryRecordKey = creationDialog.uri.match("/[0-9]+/");
				if(queryRecordKey && resourceShapes.length > 0){
					queryRecordKey = queryRecordKey[0].match("[0-9]+");
					queryRecordKey = queryRecordKey[0];				
					for(var step = 0; step < resourceShapes.length; step ++){
						var resourceShape = resourceShapes[step];
						if(resourceShape && resourceShape.indexOf(queryRecordKey) > -1 && dojo.isFunction(currentProvider.getHeadlineMaxSize4CQ)){
							currentProvider.getHeadlineMaxSize4CQ(resourceShape, currentProvider.getOSLCCoreVersion(), dojo.hitch(this, "_createNewDefectDialog", monitor, currentProvider, creationDialog, stepModel));
							return;
						}
					}
				}
				this._createNewDefectDialog(monitor, currentProvider, creationDialog, stepModel, -1);
			}));
		});
	
		new com.ibm.rqm.defects.web.ui.view.common.CQRecordTypeSelectionDialog({
			serviceProvider: currentProvider,
			isPrefilling: true,
			onOkCallback: okCallback
		});
	
	},
    
    _createDefectDescription: function(desc, stepModel){
		// add build info if any
		if(this._buildRecord) {
        	var buildRec = this._buildRecord;
        	var buildInfo = (buildRec.parentBuildDefName) ? (buildRec.parentBuildDefName +  " : " + buildRec.name) : buildRec.name;
        
        	desc += "<br/>\r\n";
	        desc += Constants.getMessage("labelBuildRecord") + ":";
	        desc += "<br/>\r\n";
	    	desc += buildInfo;
	        desc += "<br/>\r\n";
    	}
		desc += "<br/>\r\n";
        desc += Constants.getMessage("labelToProduce");
        desc += "<br/><br/>\r";
		var currentStep = 0;
		var stepNum = stepModel.stepNum;
		var steps = this.executionSteps.elementResults;
		var isKeywordStep = this.isKeyWordStep(stepModel.step);
		if(isKeywordStep) {
			// Calculate the actual current step inside a keyword
			stepNum = stepNum + stepModel.step.kwIndex;
		}
        for (var i = 0; i <= stepNum; i++) {
        	var step = steps[i];
			// include keyword steps in description
			if (step.keywordStepType == 0 || this.isKeyWordStep(step)) {
				var stepText = dojo.trim(unescape(Utility.decodeHTMLEntities(step.description)));
				//attach the text to one html node, it could render the text correctly encoded,
				//then get the correctly encoded text as stepText.
				var div = document.createElement('div');
				div.innerHTML = stepText;
				stepText = div.innerHTML;
				//remove all html tags
				stepText = stepText.replace(/<[^>]*?>/g, "");
				desc += (currentStep + 1) + ") " + stepText + "<br/>\r\n";
				currentStep++;
			}
        }
        
        desc += "<br/>" + Constants.getMessage("labelObserve");
		
		return desc;
	},
	
	_replaceDelimiter : function(desc, isRTC){
		//some delimiter are appended in NewDefectDataInitializer, 
		//and some are appended in _createDefectDescription function,
		//do some replacement here.
		if(isRTC){
			desc = desc.replace(/\r\n/g, "");
		}else{
			desc = desc.replace(/\<br\/\>/g, "");
		}
		return desc;
	},
    
    handleInsertDefectLink: function(stepModel, desc, defectLinks, currentProvider){
    	if (defectLinks.length > 0) {
			var workItem = defectLinks[0];
			if(!stepModel.step.properties) {
				stepModel.step.properties = [];
			}
			var defects = stepModel.step.properties;
			var recordData = stepModel.executionRecordData;
			var	linkComment = (recordData.name && recordData.name != recordData.testCase.name) ? 
					Constants.getMessage("labelFailingTestCaseWithEWI", recordData.testCase.name, recordData.name) : 
					Constants.getMessage("labelFailingTestCaseWithoutEWI", recordData.testCase.name);
			
			ProgressDialog.doRunWithProgress(Constants.getMessage("labelAssociateDefect"), dojo.hitch(this, function(monitor){
				
				monitor.beginTask(Constants.getMessage("labelAssociateDefect"));
				
				var items = null;												
				if(this.enableValidateRequirement){	
					items = [];												
					var step = stepModel.step;
					if(step.linkTypeDTOs){
						var linkDTOs = step.linkTypeDTOs[0].linkDTOs;
						dojo.forEach(linkDTOs, function(req){
							items.push({label: req.comment,
										'summary': req.url,
										'uri': req.url
								   	   });
						});
					}
				}
				
				currentProvider.createBackLinks4AssociatingDefectsWithTestResult(
					this.resultId, 
					linkComment, 
					desc, 
					recordData.testCase.versionableItemId, 
					recordData.testCase.name, 
					(recordData.testPlan ? recordData.testPlan.versionableItemId : null), 
					(recordData.testPlan ? recordData.testPlan.name : null), 								
					workItem.url, 
					workItem.summary, 
					items, 
					dojo.hitch(this, function(status){
						if (status && !status.isOK() && status.getSeverity() != Status.CANCEL) {
							status.log();
							this._displayAlertDialog(Constants.getMessage("labelUnableCreateWorkItem", status.getMessage()));
							monitor.done();
							return;
						}

						console.debug("defect back links created");

						var createBlocksTERLink = dojo.hitch(this, function() {
							currentProvider.createBlocksTestExecutionRecordLink(recordData.itemId, recordData.name, workItem.url, dojo.hitch(this, function(status, response){
				                console.debug("blocking ter back link created");
								monitor.done();
					        }), dojo.hitch(this, function(status, response){
								console.debug("failed to create block test execution record back link");
								this._displayAlertDialog(status.getMessage() || response);
								monitor.done();
							}));
						});
							
						var callback = dojo.hitch(this, function(){
							if(this.autoBlockTER){
								this._postLink(recordData.itemId, workItem, createBlocksTERLink, CmIntegrationClient.EWI_BLOCKING_DEFECT);
							} else {
								monitor.done();
							}
						});
						
						var callbackStepER = dojo.hitch(this, function(){
							console.debug("Step result defect links created");
						});
						
						if(currentProvider.hasWorkItemGadget())
							workItem.providerProjectKey = currentProvider.providerProjectKey;
						
						this._postLink(this.resultId, workItem, callback, CmIntegrationClient.RESULT_RELATEDTO_DEFECT);
						var stepResultItemId = stepModel.step.internalId;
						this._postLink(stepResultItemId, workItem, callbackStepER, CmIntegrationClient.LINK_TYPE_DEFECT_RELATEDTO_EXECUTION_ELEMENT_RESULT);
						defects.push(this._convertDefectToPropertyFormat(workItem));
						this.onLinkCreated(stepModel);
					}),	
					dojo.hitch(this, function(status, response){
						console.error("failed to create defect back links");
						this._displayAlertDialog(status.getMessage() || response);
						monitor.done();
					})
				);
			}));	
		}
    },
    
    generatePrefillingUrl4CQ: function(currentProvider, creationDialog, summary, description){
		if(!creationDialog){
			creationDialog = currentProvider.services.getCreationDialog("defect", true);
		}
		
		var creationUrl = null;
        if (creationDialog.uri.indexOf("?") > -1) {
            creationUrl = creationDialog.uri + "&" + ItemPickerUtil.getWICreationDialogParamString(summary, description);
        }else {
			creationUrl = creationDialog.uri + "?" + ItemPickerUtil.getWICreationDialogParamString(summary, description);	
		}
		return creationUrl;	
	},
	
	_createDefectInformationTableRow: function(contentTable){
		var tr, td1, td2;
		
		if(dojo.isIE){
			tr = contentTable.insertRow();
			td1 = tr.insertCell();
			td2 = tr.insertCell();
			dojo.addClass(td1, "table-row-head");
			dojo.addClass(td2, "table-row-content");
		}else{
			tr = dojo.create("tr", {}, contentTable);
			td1 = dojo.create("td", {"class":"table-row-head"}, tr);
			td2 = dojo.create("td", {"class":"table-row-content"}, tr);
		}
		
		var tableRow = {
			row: tr,
			firstColumn: td1,
			secondColumn: td2
		};
		
		return tableRow;
	},
    
    _displayAlertDialog: function(message){
        var args = {
			message: message
		};
		var alertDialog = new com.ibm.asq.common.web.ui.internal.widgets.AlertDialog(args);
        alertDialog.open();
    },
    
    _postLink: function(itemId, workItem, callback, linkType){
		var linkParams = {
			itemId: itemId,
			cmUri: workItem.url,
			summary: workItem.summary,
			linkType: linkType,
			createRelatedLinks: true
		};
		
		CmIntegrationClient.postNewLink(linkParams, dojo.hitch(this, function(status){
			callback();
        }));
    },
    
    _convertDefectToPropertyFormat: function(workItem) {
		var property = {
			propertyType: "rtcdefectView",
			propertyName: workItem.summary,
			propertyValue: workItem.url
		};
		return property; 
	},
	
	onLinkCreated: function(event) {
		// To be overrode by subclasses
	},
    
    //@Overrides ClmLinksHelp
    getProviderNotConfiguredMessage: function(){
    	return DefectsUIUtil.getMessage("msgNoneCMProvidersConfigured");
    }
    
});
	
})();