<?xml version="1.0" encoding="utf-8"?>
<!-- 
/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011, 2014. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
 -->
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:xalan="http://xml.apache.org/xalan"
    xmlns:rqm="http://jazz.net/xmlns/alm/qm/v0.1/"
    xmlns:dc="http://purl.org/dc/elements/1.1/"
    xmlns:alm="http://jazz.net/xmlns/alm/v0.1/"
    xmlns:result="http://jazz.net/xmlns/alm/qm/v0.1/executionresult/v0.1"
    xmlns:exslt="http://exslt.org/common"
    xmlns:HttpClientExtension="com.ibm.rqm.print.render.fop.xml.xslt.extension.http.HttpClientExtension"
    xmlns:proxy="com.ibm.rqm.print.render.fop.xml.xslt.extension.proxy.ExecutionResultProxy"
    xmlns:log="xalan://com.ibm.rqm.print.render.fop.xml.xslt.extension.LoggingExtension"
    xmlns:utils="xalan://com.ibm.rqm.print.render.fop.xml.xslt.extension.UtilsExtension"
    exclude-result-prefixes="xsl xalan rqm dc alm result exslt HttpClientExtension proxy log">
    
    <!-- included stylesheets -->
    <xsl:import  href="cache://stylesheets/prep/abstract-prep-handler.xsl"/>
    <xsl:include href="cache://stylesheets/_util/replaceAll.xsl"/>
    
    <!-- params -->
    <xsl:param name="artifact"/>
    <xsl:param name="repoService"/>
    <xsl:param name="exeResultService"/>
    <xsl:param name="catTypeRestService"/>
    
    <!-- proxy definition -->
    <xalan:component prefix="proxy">
        <xalan:script lang="javaclass" src="xalan://com.ibm.rqm.print.render.fop.xml.xslt.extension.proxy.ExecutionResultProxy"/>
    </xalan:component>
    
    <xsl:variable name="item" select="proxy:new($artifact, $repoService, $exeResultService, $catTypeRestService)"/>
    
    <!-- templates -->
    <xsl:template match="rqm:executionresult">
        <xsl:copy>
            <xsl:apply-templates select="dc:identifier"/>
            <xsl:apply-templates select="rqm:webId"/>
            <xsl:apply-templates select="dc:title"/>
            <xsl:apply-templates select="result:approvalstate"/>
            <xsl:apply-templates select="alm:state"/>
            <xsl:apply-templates select="result:machine"/>
            <xsl:apply-templates select="alm:owner"/>
            <xsl:apply-templates select="dc:creator"/>
            <xsl:apply-templates select="result:testedby"/>
            <xsl:apply-templates mode="expand" select="rqm:testplan"/>
            <xsl:apply-templates mode="expand" select="rqm:testphase"/>
            <xsl:apply-templates mode="expand" select="rqm:testcase"/>
            <xsl:apply-templates mode="expand" select="rqm:executionscript"/>
            <xsl:apply-templates mode="expand" select="rqm:datapool"/>
            <xsl:apply-templates mode="expand" select="rqm:buildrecord"/>
            <xsl:apply-templates select="result:weight"/>
            <xsl:apply-templates select="result:starttime"/>
            <xsl:apply-templates select="result:endtime"/>
            <xsl:apply-templates select="rqm:testdatarows"/>
            
            <!-- sections -->
            <xsl:variable name="sections" select="proxy:getSections($item)"/>
            
            <xsl:if test="log:isDebugEnabled()">
                <xsl:value-of select="log:debug_xml($sections)"/></xsl:if>
            
            <xsl:apply-templates select="$sections/rqm:section">
                <xsl:with-param name="artifact" select="current()"/>
            </xsl:apply-templates>
        </xsl:copy>
    </xsl:template>
    
    <xsl:template match="alm:state">
        <!-- The REST API does not let us know if the state has been customized so we will need to check via 
             a proxy method and add the customized value if it has been. -->
        <xsl:variable name="customizedStateName" select="proxy:getCustomResultName($item)"/>
        
        <xsl:copy>
            <xsl:if test="$customizedStateName">
                <xsl:attribute name="value"><xsl:value-of select="$customizedStateName"/></xsl:attribute>
            </xsl:if>
            <xsl:apply-templates select="@*|node()"/>            
        </xsl:copy>
    </xsl:template>
    
    <xsl:template mode="expand" match="rqm:testplan[@href]">
        <xsl:copy-of select="HttpClientExtension:get($restClient, @href, 'fields=feed/entry/content/testplan/(identifier|title|webId)')"/>
    </xsl:template>
    
    <xsl:template mode="expand" match="rqm:testphase">
        <xsl:copy-of select="HttpClientExtension:get($restClient, @href, 'fields=feed/entry/content/testphase/(identifier|webId|title)')"/>
    </xsl:template>
    
    <xsl:template mode="expand" match="rqm:testcase[@href]">
        <xsl:copy-of select="HttpClientExtension:get($restClient, @href, 'fields=feed/entry/content/testcase/(identifier|title|webId)')"/>
    </xsl:template>
    
    <xsl:template mode="expand" match="rqm:executionscript[@href]">
        <xsl:copy-of select="HttpClientExtension:get($restClient, @href, 'fields=feed/entry/content/executionscript/(identifier|title|webId|type)')"/>
    </xsl:template>
    
    <xsl:template mode="expand" match="rqm:datapool[@href]">
        <xsl:copy-of select="HttpClientExtension:get($restClient, @href, 'fields=feed/entry/content/datapool/(identifier|title)')"/>
    </xsl:template>
    
    <xsl:template mode="expand" match="rqm:buildrecord[@href]">
        
        <xsl:copy-of select="HttpClientExtension:get($restClient, @href, 'fields=feed/entry/content/buildrecord/(identifier|title|webId)')"/>
    
    	<!-- Resolve the build definition associated with the build record: -->
		<xsl:variable name="buildDefinitionFeed">
            
            <xsl:variable name="buildRecordFilter">buildrecord[@href="<xsl:value-of select="utils:encodeUri( @href)"/>"]</xsl:variable>
            
            <xsl:call-template name="restGET">
                <xsl:with-param name="path"  select="'../builddefinition'"/>
                <xsl:with-param name="query" select="concat('fields=/feed/entry/content/builddefinition/(', $buildRecordFilter, '|identifier|title)')"/>
            </xsl:call-template>    
        </xsl:variable>
        
        <xsl:copy-of select="exslt:node-set($buildDefinitionFeed)//rqm:builddefinition"/>
    </xsl:template>
    
    <!-- Weight Distribution -->
    <xsl:template match="rqm:section[@name='Weight Distribution']">
        <xsl:param name="artifact"/>
        
        <xsl:copy>
            <xsl:apply-templates select="@*"/>
            <xsl:apply-templates select="$artifact/result:pointspassed"/>
            <xsl:apply-templates select="$artifact/result:pointsfailed"/>
            <xsl:apply-templates select="$artifact/result:pointsattempted"/>
            <xsl:apply-templates select="$artifact/result:pointsblocked"/>
            <xsl:apply-templates select="$artifact/result:pointsinconclusive"/>
            <xsl:apply-templates select="$artifact/rqm:pointspermfailed"/>
            <xsl:apply-templates select="$artifact/rqm:pointsdeferred"/>
        </xsl:copy>
    </xsl:template>
    
    <!-- Test Environment -->
    <xsl:template match="rqm:section[@name='Test Environment']">
        <xsl:param name="artifact"/>
        
        <xsl:variable name="ewi" select="HttpClientExtension:get($restClient, $artifact/rqm:executionworkitem/@href, 'fields=feed/entry/content/executionworkitem/(configuration)')"/>
        
        <xsl:copy>
            <xsl:apply-templates select="@*"/>
            <xsl:apply-templates mode="expand" select="$ewi/rqm:configuration"/>
            <xsl:apply-templates select="$artifact/rqm:channel"/>
        </xsl:copy>
    </xsl:template>
    
    <xsl:template mode="expand" match="rqm:configuration[@href]">
        <xsl:apply-templates select="HttpClientExtension:get($restClient, @href, 'fields=feed/entry/content/configuration/(identifier|title|configuration)')"/>
    </xsl:template>
    
    <xsl:template match="rqm:channel[@href]">
        <xsl:apply-templates select="HttpClientExtension:get($restClient, @href)"/>
    </xsl:template>
    
    <!-- Defects -->
    <xsl:template match="rqm:section[@name='Defects']">
        <xsl:param name="artifact"/>
        
        <xsl:copy>
            <xsl:apply-templates select="@*"/>
            <xsl:apply-templates select="$artifact/rqm:defect"/>
        </xsl:copy>
    </xsl:template>
    
    <!-- Notes -->
    <xsl:template match="rqm:section[@name='Notes']">
        <xsl:param name="artifact"/>
        
        <!-- Note: The Notes section only contains result:details when a test case is executed. -->
    	<xsl:if test="$artifact/result:stepResults">
	        <xsl:copy>
	            <xsl:apply-templates select="@*"/>
	            <xsl:apply-templates select="$artifact/result:details/*"/>
	        </xsl:copy>
        </xsl:if>
    </xsl:template>
    
    <!-- Categories -->
    <xsl:template match="rqm:section[@name='Categories']">
        <xsl:param name="artifact"/>
        <xsl:variable name="categories" select="proxy:getCategories($item)/rqm:category"/>
        
        <xsl:copy>
            <xsl:apply-templates select="@*"/>
            <xsl:apply-templates select="$categories"/>
            <xsl:apply-templates select="$artifact/rqm:customAttributes"/>
        </xsl:copy>
    </xsl:template>
    
    <!-- Execution Variables -->
    <xsl:template match="rqm:section[@id='com.ibm.rqm.planning.editor.section.suiteExecVars' or @id='com.ibm.rqm.planning.editor.section.testcaseExecVars']">
        <xsl:param name="artifact"/>
        
        <xsl:copy>
            <xsl:apply-templates select="@*"/>
            <xsl:apply-templates select="$artifact/rqm:variables"/>
        </xsl:copy>
    </xsl:template>
    
    <!-- Previous Results -->
    <xsl:template match="rqm:section[@name='Previous Results']">
        <xsl:param name="artifact"/>
        
        <xsl:copy>
            <xsl:apply-templates select="@*"/>
            <table-list key='RQM-EXE-HISTORY-TABLEVIEWER'>
                <xsl:variable name="executionresults">
                    <xsl:variable name="query">
                        <xsl:text>fields=feed/entry/content/executionresult[executionworkitem/@href='</xsl:text>
                        <xsl:value-of select="utils:encodeUri(proxy:urlDecode($item, $artifact/rqm:executionworkitem/@href))"/>
                        <xsl:text>']/identifier</xsl:text>
                    </xsl:variable>
                    
                    <xsl:call-template name="restGET">
                        <xsl:with-param name="path" select="'../executionresult'"/>
                        <xsl:with-param name="query" select="$query"/>
                    </xsl:call-template>    
                </xsl:variable>
                
                <xsl:apply-templates mode="collapse" select="exslt:node-set($executionresults)//rqm:executionresult"/>
            </table-list>
        </xsl:copy>
    </xsl:template>
    
    <xsl:template mode="collapse" match="rqm:executionresult">
        <xsl:copy>
            <xsl:attribute name="href"><xsl:value-of select="dc:identifier"/></xsl:attribute>
        </xsl:copy>
    </xsl:template>
    
    <xsl:template match="rqm:section[@name='Result Details']">
        <xsl:param name="artifact"/>
        
        <!-- Note: The Result Details section contains result:stepResults when a test script is executed and result:details when a test case is executed. -->
         <xsl:choose>
    		<xsl:when test="$artifact/result:stepResults">
        		<xsl:copy>
        			<xsl:apply-templates select="@*"/>
        			<xsl:apply-templates select="$artifact/result:stepResults"/>
        		</xsl:copy>
    		</xsl:when>
    		<xsl:otherwise>
				<xsl:copy>
            		<xsl:apply-templates select="@*"/>
            		<xsl:apply-templates select="$artifact/result:details/*"/>
        		</xsl:copy>
 			</xsl:otherwise>
		</xsl:choose>
    </xsl:template>
    
    <xsl:template match="rqm:section[@name='Formal Review']">
        <xsl:param name="artifact"/>
        
        <xsl:copy>
            <xsl:apply-templates select="@*"/>
            <xsl:apply-templates select="$artifact/rqm:approvals"/>
        </xsl:copy>
    </xsl:template>
    
       
	<!-- Audit -->
    <xsl:template match="rqm:section[@name='Audit']">
        <xsl:param name="artifact"/>
        <xsl:copy>
            <xsl:apply-templates select="@*"/>
            <xsl:apply-templates select="$artifact/result:starttime"/>
            <xsl:apply-templates select="$artifact/result:testcasestateatexecution"/>
            <xsl:apply-templates select="$artifact/result:testplanstateatexecution"/>
            <xsl:apply-templates select="$artifact/result:testscriptstateatexecution"/>
        </xsl:copy>
    </xsl:template>
   
    <!-- eSignatures -->
    <xsl:template match="rqm:section[@id='com.ibm.rqm.planning.editor.section.esign.audit']">
        <xsl:param name="artifact"/>
        <xsl:copy>
            <xsl:apply-templates select="@*"/>
            <xsl:apply-templates select="$artifact/rqm:esignatures"/>
        </xsl:copy>
    </xsl:template>
    
</xsl:stylesheet>