<?xml version="1.0" encoding="utf-8"?>
<!-- 
/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012, 2014. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
 -->
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:xalan="http://xml.apache.org/xalan"
    xmlns:rqm="http://jazz.net/xmlns/alm/qm/v0.1/"
    xmlns:prep="http://com.ibm.rqm.print/prep"
    xmlns:pref="http://com.ibm.rqm.export.pdf/preferences"
    xmlns:dc="http://purl.org/dc/elements/1.1/"
    xmlns:dcterms="http://purl.org/dc/terms/"
    xmlns:alm="http://jazz.net/xmlns/alm/v0.1/"
    xmlns:exslt="http://exslt.org/common"
    xmlns:HttpClientExtension="com.ibm.rqm.print.render.fop.xml.xslt.extension.http.HttpClientExtension"
    xmlns:proxy="com.ibm.rqm.print.render.fop.xml.xslt.extension.proxy.ExecutionScriptProxy"
    xmlns:log="xalan://com.ibm.rqm.print.render.fop.xml.xslt.extension.LoggingExtension"
    xmlns:utils="xalan://com.ibm.rqm.print.render.fop.xml.xslt.extension.UtilsExtension"
    exclude-result-prefixes="xalan alm dc dcterms prep exslt HttpClientExtension proxy log">
    
    <xsl:import href="cache://stylesheets/prep/abstract-prep-handler.xsl"/>
    <xsl:include href="cache://stylesheets/_util/replaceAll.xsl"/>
    
    <xsl:param name="artifact"/>
    <xsl:param name="repoService"/>
    <xsl:param name="catTypeRestService"/>
    
    <xalan:component prefix="proxy">
        <xalan:script lang="javaclass" src="xalan://com.ibm.rqm.print.render.fop.xml.xslt.extension.proxy.ExecutionScriptProxy"/>
    </xalan:component>
    
    <xsl:variable name="item" select="proxy:new($artifact, $repoService, $catTypeRestService)"/>
    
    <xsl:template match="rqm:executionscript">
        <xsl:value-of select="log:trace(concat('prepping: ', dc:identifier))"/>
        
        <xsl:copy>
            <xsl:apply-templates select="dc:identifier"/>
            <xsl:apply-templates select="rqm:webId"/>
            <xsl:apply-templates select="dc:title"/>
            <xsl:apply-templates select="alm:state"/>
            <xsl:apply-templates select="dc:creator"/>
            <xsl:apply-templates select="alm:owner"/>
            <xsl:apply-templates select="dcterms:type"/>
            <xsl:apply-templates mode="expand" select="rqm:datapool"/>
            <xsl:apply-templates select="dc:description"/>
            <xsl:apply-templates select="rqm:copiedFrom"/>
            
            <xsl:call-template name="_insert-copiedTo"/>
            <xsl:call-template name="summarySection"/>
            <xsl:call-template name="formalReviewSection"/>
            <xsl:call-template name="manualStepsSection"/>
            <!-- Note: Do NOT render the Execution Variables section since it is not included in the test script editor (see task-development 81537).           
            <xsl:call-template name="executionVariablesSection"/>
            -->
            <xsl:call-template name="associatedESignaturesSection"/>
            <xsl:apply-templates select="rqm:snapshot"/>
        </xsl:copy>
    </xsl:template>
    
    <xsl:template mode="expand" match="rqm:datapool">
        <xsl:copy-of select="HttpClientExtension:get($restClient, @href, 'fields=feed/entry/content/datapool/(identifier|title)')"/>
    </xsl:template>
    
    <!-- copiedFrom -->
    <xsl:template match="rqm:copiedFrom">
        <xsl:apply-templates mode="copiedFrom" select="HttpClientExtension:get($restClient, @href, 'fields=/feed/entry/content/executionscript/(identifier|title)')"/>
    </xsl:template>
    
    <xsl:template mode="copiedFrom" match="rqm:executionscript">
        <prep:copiedFrom>
            <xsl:apply-templates select="dc:title"/>
            <xsl:apply-templates select="dc:identifier"/>
        </prep:copiedFrom>
    </xsl:template>
    
    <!-- copiedTo -->
    <xsl:template name="_insert-copiedTo">
        <xsl:variable name="copiedToFeed">
            <xsl:variable name="tsFilter">copiedFrom/@href="<xsl:value-of select="utils:encodeUri(dc:identifier)"/>"</xsl:variable>

            <xsl:call-template name="restGET">
                <xsl:with-param name="path"  select="'../executionscript'"/>
                <xsl:with-param name="query" select="concat('fields=/feed/entry/content/executionscript[', $tsFilter, ']/(identifier|title)')"/>
            </xsl:call-template>
        </xsl:variable>
        
        <xsl:apply-templates mode="copiedTo" select="exslt:node-set($copiedToFeed)//rqm:executionscript"/>
    </xsl:template>
    
    <xsl:template mode="copiedTo" match="rqm:executionscript">
        <prep:copiedTo>
            <xsl:apply-templates select="dc:title"/>
            <xsl:apply-templates select="dc:identifier"/>
        </prep:copiedTo>
    </xsl:template>
    
    <!-- Summary -->
    <xsl:template name="summarySection">
        <xsl:value-of select="log:trace('prepping section: Summary...')"/>
        
        <xsl:variable name="categories" select="proxy:getCategories($item)/rqm:category"/>
        
        <xsl:if test="$preferences/pref:renderEmptySections='true' or rqm:categories or rqm:customAttributes">
            <rqm:section id="com.ibm.rqm.print.executionscript.section.summary">
                <xsl:apply-templates select="$categories"/>
                <xsl:apply-templates select="rqm:customAttributes"/>
            </rqm:section>
        </xsl:if>
    </xsl:template>
    
    <!-- Formal Review -->
    <xsl:template name="formalReviewSection">
        <xsl:value-of select="log:trace('prepping section: Formal Review...')"/>
        
        <xsl:if test="$preferences/pref:renderEmptySections='true' or rqm:approvals">
            <rqm:section id="com.ibm.rqm.print.executionscript.section.formalReview">
                <xsl:apply-templates select="rqm:approvals"/>
            </rqm:section>
        </xsl:if>
    </xsl:template>
    
    <!-- Manual Steps -->
    <xsl:template name="manualStepsSection">
        <xsl:value-of select="log:trace('prepping section: Manual Steps...')"/>
        
        <xsl:if test="$preferences/pref:renderEmptySections='true' or rqm:steps">
            <rqm:section id="com.ibm.rqm.print.executionscript.section.manualSteps">
                <xsl:apply-templates select="rqm:steps"/>
            </rqm:section>
        </xsl:if>
    </xsl:template>
    
    <xsl:template match="rqm:step[@dcterms:type='keyword']/rqm:link">
        <xsl:value-of select="log:trace('keyword step found, fetching step details...')"/>
        <xsl:apply-templates select="HttpClientExtension:get($restClient, @href, 'fields=feed/entry/content/keyword/(title|testscript)')"/>
    </xsl:template>
    
    <xsl:template match="rqm:testscript">
        <!-- We want to use the executionscript API, not testscript.  We can translate between the two URIs if 
             we are careful -->
        <xsl:variable name="exeScriptURI">
            <xsl:variable name="temp">
                <xsl:call-template name="replaceAll">
                    <xsl:with-param name="original" select="@href"/>
                    <xsl:with-param name="find-string" select="'/testscript/'"/>
                    <xsl:with-param name="replace-string" select="'/executionscript/'"/>
                </xsl:call-template>
            </xsl:variable>
            
            <!--<xsl:variable name="temp2">-->
                <xsl:call-template name="replaceAll">
                    <xsl:with-param name="original" select="$temp"/>
                    <xsl:with-param name="find-string" select="'/urn:com.ibm.rqm:testscript:'"/>
                    <xsl:with-param name="replace-string" select="'/urn:com.ibm.rqm:executionscript:'"/>
                </xsl:call-template>
            <!--</xsl:variable>-->
            
            <!--<xsl:value-of select="proxy:urlDecode($item, $temp2)"/>-->
        </xsl:variable>
        
        <!-- expand script to get steps -->
        <xsl:value-of select="log:trace('expanding test script steps for keyword...')"/>
        <xsl:copy-of select="HttpClientExtension:get($restClient, $exeScriptURI, 'fields=feed/entry/content/executionscript/(identifier|steps)')"/>
    </xsl:template>
    
    <!-- Execution Variables -->
    <!-- Note: Do NOT render the Execution Variables section since it is not included in the test script editor (see task-development 81537). 
    <xsl:template name="executionVariablesSection">
        <xsl:if test="$preferences/pref:renderEmptySections='true' or rqm:variables/rqm:variable">
            <rqm:section id="com.ibm.rqm.print.executionscript.section.executionVariables">
                <xsl:apply-templates select="rqm:variables/rqm:variable"/>
            </rqm:section>    
        </xsl:if>
    </xsl:template>
	-->
	    
    <!-- eSignatures -->
    <xsl:template name="associatedESignaturesSection">
        <xsl:value-of select="log:trace('prepping section: Associated E-Signatures...')"/>
        
        <xsl:if test="$preferences/pref:renderEmptySections='true' or rqm:esignatures">
            <rqm:section id="com.ibm.rqm.planning.editor.section.esign.audit">
                <xsl:apply-templates select="rqm:esignatures"/>
            </rqm:section>
        </xsl:if>
    </xsl:template>
</xsl:stylesheet>