<?xml version="1.0" encoding="utf-8"?>
<!-- 
/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2014. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
 -->
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:fo="http://www.w3.org/1999/XSL/Format" 
    xmlns:rqm="http://jazz.net/xmlns/alm/qm/v0.1/"
    xmlns:dc="http://purl.org/dc/elements/1.1/"
    xmlns:tsl="http://jazz.net/xmlns/alm/qm/v0.1/tsl/v0.1/"
    xmlns:alm="http://jazz.net/xmlns/alm/v0.1/"
    xmlns:xalan="http://xml.apache.org/xalan"
    xmlns:pref="http://com.ibm.rqm.export.pdf/preferences"
    xmlns:proxy="com.ibm.rqm.print.render.fop.xml.xslt.extension.proxy.TestsuiteLogProxy"
    xmlns:time="xalan://com.ibm.rqm.print.render.fop.xml.xslt.extension.TimeUtilsExtension"
    xmlns:ewir="http://jazz.net/xmlns/alm/qm/v0.1/executionresult/v0.1"
    xmlns:utils="xalan://com.ibm.rqm.print.render.fop.xml.xslt.extension.UtilsExtension"
    xmlns:prep="http://com.ibm.rqm.print/prep"
    exclude-result-prefixes="xsl xalan pref proxy rqm dc alm tsl time prep">
    
    <!-- included stylesheets -->
    <xsl:import  href="cache://stylesheets/render/abstract-document.xsl"/>
    <xsl:include href="cache://stylesheets/render/_util/icons/result-handler.xsl"/>
    <xsl:include href="cache://stylesheets/render/_util/lab-resource.xsl"/>
    <xsl:include href="cache://stylesheets/render/_util/summary-section.xsl"/>
    <xsl:include href="cache://stylesheets/render/_util/formal-review.xsl"/>
    <xsl:include href="cache://stylesheets/render/_util/utils.xsl"/>
    <xsl:include href="cache://stylesheets/render/_util/execution-variables.xsl"/>
    <xsl:include href="cache://stylesheets/render/_util/eSignatures.xsl"/>
    
    <!-- params -->
    <xsl:param name="artifact"/>
    <xsl:param name="repoService"/>
    <xsl:param name="catTypeRestService"/>
    <xsl:param name="exeResultService"/>
    <xsl:param name="aliasService"/>
    <xsl:param name="integrationService"/>
    <xsl:param name="erStateService"/>
       
    <xsl:variable name="header-icon" select="'basecache://com.ibm.asq.common.web/resources/ui/internal/images/icons/view16/qm_execresult48.gif'"/>
       
    <!-- proxy definition -->
    <xalan:component prefix="proxy">
        <xalan:script lang="javaclass" src="xalan://com.ibm.rqm.print.render.fop.xml.xslt.extension.proxy.TestsuiteLogProxy"/>
    </xalan:component>
    
    <xsl:variable name="item" select="proxy:new($artifact, $repoService, $catTypeRestService, $erStateService)"/>
    
    <!-- templates -->
    <xsl:template match="rqm:testsuitelog">
        <!-- Page Title -->
        <xsl:call-template name="buildTitleSection">
            <xsl:with-param name="name" select="dc:title"/>
			<xsl:with-param name="id" select="rqm:webId"/>
			<xsl:with-param name="href" select="dc:identifier"/>
			<xsl:with-param name="header-icon" select="$header-icon"/>
        </xsl:call-template>
        
        <!-- Details -->
        <xsl:call-template name="_buildDetailsSection"/>
        
        <!-- Environment -->
		<xsl:call-template name="_buildEnvironmentSection"/>
		        
        <!-- Categories -->
        <xsl:variable name="category-list" select="proxy:getCategories($item)/rqm:category"/>
        
        <xsl:if test="$preferences/pref:renderEmptySections='true' or $category-list">
            <xsl:call-template name="buildStandardSummarySection">
                <xsl:with-param name="header" select="$strings/Categories"/>
                <xsl:with-param name="category-list" select="$category-list"/>
                <xsl:with-param name="attributes" select="rqm:customAttributes"/>
            </xsl:call-template>
        </xsl:if>
        
        <!-- Weight Distribution -->
        <xsl:call-template name="_buildWeightDistribution"/>
             	
        <!-- Result Details -->
        <xsl:if test="$preferences/pref:renderEmptySections='true' or rqm:executionresult">
            <fo:block xsl:use-attribute-sets="section-heading"><xsl:value-of select="$strings/Result_Details"/></fo:block>
            <xsl:if test="rqm:executionresult or $preferences/pref:renderEmptyTables='true'">
                <xsl:call-template name="buildSuiteResultDetailsTable">
                    <xsl:with-param name="results" select="rqm:executionresult"/>
                    <xsl:with-param name="configuration" select="document(concat('rest://',rqm:suiteexecutionrecord/@href))/rqm:suiteexecutionrecord/rqm:configuration"/>
                </xsl:call-template>
            </xsl:if>
        </xsl:if>
        
        <!-- Formal Review -->
        <xsl:apply-templates select="rqm:approvals"/>
        
        <!-- Execution Variables -->
        <xsl:call-template name="buildExecutionVariableSection">
         <xsl:with-param name="variables" select="rqm:variables/rqm:variable"/>
        </xsl:call-template>
        
        <!--  Audit Section -->>
        <xsl:call-template name="_buildAuditSection"/>
        
    </xsl:template>
    
    <!-- Execution Result Summary -->
    <xsl:template name="_buildDetailsSection">
        <fo:table>
            <fo:table-body>
                <!-- Blank row -->
                <fo:table-row height="8pt">
                    <fo:table-cell><fo:block linefeed-treatment="preserve"/></fo:table-cell>
                </fo:table-row>
                <!-- Process State -->
	            <!-- Passed Test Cases -->
                <fo:table-row>
                    <fo:table-cell>
                    	<fo:block><xsl:value-of select="$strings/State"/>:</fo:block>
                    </fo:table-cell>
                    <fo:table-cell>
                    	<xsl:variable name="stateId" select="alm:processState"/>                  
                        <xsl:variable name="stateName" select="$strings/*[name()=$stateId]"/>    
                       	<fo:block>
                        	<xsl:choose>
                        		<xsl:when test="$stateName">
                        		    <xsl:value-of select="$stateName"/>
                        		</xsl:when>
                        		<xsl:otherwise>
                        			<xsl:variable name="workflowType" select="'TESTSUITERESULT'"/>    
                        			<xsl:value-of select="utils:getStateName($projectArea, $workflowType, $stateId)"/>
                        		</xsl:otherwise>
                        	</xsl:choose>
                        </fo:block>                  
                    </fo:table-cell>
	                <fo:table-cell><fo:block><xsl:value-of select="$strings/passedTestCases"/>:</fo:block></fo:table-cell>
	                <fo:table-cell><fo:block xsl:use-attribute-sets="value"><xsl:value-of select="tsl:testcasespassed"/></fo:block></fo:table-cell>
                </fo:table-row>
                <!-- Actual Result -->
	            <!-- Failed Test Cases -->
                <fo:table-row>
                    <fo:table-cell>
                        <fo:block><xsl:value-of select="$strings/Actual_Result"/>:</fo:block>
                    </fo:table-cell>
                    <fo:table-cell>
                        <xsl:variable name="state" select="alm:state"/>
                        <xsl:variable name="customStateName" select="proxy:getCustomResultName($item)"/>
                        <fo:block>
                            <xsl:choose>
				        		<xsl:when test="$customStateName">
				        	 		<xsl:value-of select="$customStateName"/></xsl:when>
				        		<xsl:otherwise>
				        			<xsl:value-of select="$strings/*[name()=$state]"/></xsl:otherwise>
				        	</xsl:choose>
                            <xsl:text> </xsl:text>
                            <xsl:call-template name="printResultStateIcon">
                                <xsl:with-param name="size">45%</xsl:with-param>
                                <xsl:with-param name="priority_id"><xsl:value-of select="alm:state"/></xsl:with-param>
                            </xsl:call-template>
                        </fo:block>
                    </fo:table-cell>
	                <fo:table-cell><fo:block><xsl:value-of select="$strings/failedTestCases"/>:</fo:block></fo:table-cell>
	                <fo:table-cell><fo:block xsl:use-attribute-sets="value"><xsl:value-of select="tsl:testcasesfailed"/></fo:block></fo:table-cell>
                </fo:table-row>
                <!-- Start Time-->
	            <!-- Inconclusive Test Cases -->
                <fo:table-row>
                    <fo:table-cell><fo:block><xsl:value-of select="$strings/Start_Time"/>:</fo:block></fo:table-cell>
                    <fo:table-cell><fo:block xsl:use-attribute-sets="value"><xsl:value-of select="time:formatMediumDateMediumTime(tsl:starttime, $locale)"/></fo:block></fo:table-cell>
	                <fo:table-cell><fo:block><xsl:value-of select="$strings/inconclusiveTestCases"/>:</fo:block></fo:table-cell>
	                <fo:table-cell><fo:block xsl:use-attribute-sets="value"><xsl:value-of select="tsl:testcasesinconclusive"/></fo:block></fo:table-cell>
                </fo:table-row>
                <!-- End Time -->
	            <!-- Blocked Test Cases -->
                <fo:table-row>
                    <fo:table-cell><fo:block><xsl:value-of select="$strings/End_Time"/>:</fo:block></fo:table-cell>
                    <fo:table-cell><fo:block xsl:use-attribute-sets="value"><xsl:value-of select="time:formatMediumDateMediumTime(tsl:endtime, $locale)"/></fo:block></fo:table-cell>
	                <fo:table-cell><fo:block><xsl:value-of select="$strings/blockedTestCases"/>:</fo:block></fo:table-cell>
	                <fo:table-cell><fo:block xsl:use-attribute-sets="value"><xsl:value-of select="tsl:testcasesblocked"/></fo:block></fo:table-cell>
                </fo:table-row>
                <!-- Total Run Time -->
	            <!-- In Progress Test Cases -->
                <fo:table-row>                
                	<fo:table-cell><fo:block><xsl:value-of select="$strings/Total_Run_Time"/>:</fo:block></fo:table-cell>
                    <fo:table-cell><fo:block><xsl:value-of select="time:formatTimeEstimateNoMillis(tsl:starttime, tsl:endtime, $locale)"/></fo:block></fo:table-cell>
	                <fo:table-cell><fo:block><xsl:value-of select="$strings/inProgressTestCases"/>:</fo:block></fo:table-cell>
	                <fo:table-cell><fo:block xsl:use-attribute-sets="value"><xsl:value-of select="tsl:testcasesinprogress"/></fo:block></fo:table-cell>
                </fo:table-row>
                <!-- Execution Mode -->
                <!--    Not yet implemented -->
                <!-- Execution Variables -->
                <!--    Not yet implemented, tracked by work item 50365 -->
                <!-- Was Suite Execution Halted? -->
	            <!-- Not Started Test Cases -->
                <fo:table-row>
                    <fo:table-cell><fo:block><xsl:value-of select="$strings/Was_Suite_Execution_Halted"/>:</fo:block></fo:table-cell>
                    <fo:table-cell><fo:block xsl:use-attribute-sets="value"><xsl:value-of select="rqm:haltOnFailure"/></fo:block></fo:table-cell>
	                <fo:table-cell><fo:block><xsl:value-of select="$strings/notStartedTestCases"/>:</fo:block></fo:table-cell>
	                <fo:table-cell><fo:block xsl:use-attribute-sets="value"><xsl:value-of select="tsl:testcasesnotstarted"/></fo:block></fo:table-cell>
                </fo:table-row>
                <!-- Test Plan -->
	            <!-- Total Test Cases -->
                <fo:table-row>
                    <fo:table-cell><fo:block><xsl:value-of select="$strings/Test_Plan"/>:</fo:block></fo:table-cell>
                    <fo:table-cell>
                        <fo:block xsl:use-attribute-sets="value">
                            <xsl:if test="rqm:testplan">
                                <fo:basic-link external-destination="{rqm:testplan/@href}" xsl:use-attribute-sets="simple-link"><xsl:value-of select="document(concat('rest://', rqm:testplan/@href))/rqm:testplan/dc:title"/></fo:basic-link>
                            </xsl:if>
                        </fo:block>
                    </fo:table-cell>
	                <fo:table-cell><fo:block><xsl:value-of select="$strings/totalTestCases"/>:</fo:block></fo:table-cell>
	                <fo:table-cell><fo:block xsl:use-attribute-sets="value"><xsl:value-of select="count(tsl:suiteelements)"/></fo:block></fo:table-cell>
                </fo:table-row>
                <!-- Build -->
                <fo:table-row>
                    <fo:table-cell><fo:block><xsl:value-of select="$strings/Build"/>:</fo:block></fo:table-cell>
                    <fo:table-cell>
                        <fo:block>
                            <xsl:choose>
                                <xsl:when test="rqm:buildrecord">
                                    <fo:basic-link xsl:use-attribute-sets="simple-link" external-destination="{rqm:buildrecord/dc:identifier}">
                                        <xsl:choose>
                                			<xsl:when test="rqm:builddefinition">
                                				<xsl:value-of select="concat(rqm:buildrecord/rqm:webId, ': ', rqm:builddefinition/dc:title, ' : ', rqm:buildrecord/dc:title)"/>    
                                        	</xsl:when>
                                			<xsl:otherwise>
                                        		<xsl:value-of select="concat(rqm:buildrecord/rqm:webId, ': ', rqm:buildrecord/dc:title)"/>    
                                			</xsl:otherwise>
			                            </xsl:choose>
                                    </fo:basic-link>
                                </xsl:when>
                                <xsl:otherwise>
                                    <fo:inline xsl:use-attribute-sets="value">
                                        <xsl:value-of select="$strings/Unassigned"/>
                                    </fo:inline>
                                </xsl:otherwise>
                            </xsl:choose>
                        </fo:block>
                    </fo:table-cell>
                </fo:table-row>
                <!-- Iteration -->   
                <fo:table-row>
                	<fo:table-cell>
						<fo:block xsl:use-attribute-sets="label"><xsl:value-of select="$strings/Iteration"/>:</fo:block>
					</fo:table-cell>
					<fo:table-cell>
						<fo:block xsl:use-attribute-sets="value">
							<xsl:choose>
								<xsl:when test="rqm:testphase">
									<xsl:value-of select="rqm:testphase/dc:title"/>
								</xsl:when>
								<xsl:otherwise>
									<xsl:value-of select="$strings/Unassigned"/>
								</xsl:otherwise>
							</xsl:choose>
						</fo:block>
					</fo:table-cell>
                </fo:table-row>
                <!-- Test Suite -->
                <fo:table-row>
                    <fo:table-cell><fo:block><xsl:value-of select="$strings/Test_Suite"/>:</fo:block></fo:table-cell>
                    <fo:table-cell><fo:block xsl:use-attribute-sets="value"><fo:basic-link external-destination="{rqm:testsuite/@href}" xsl:use-attribute-sets="simple-link"><xsl:value-of select="document(concat('rest://', rqm:testsuite/@href))/rqm:testsuite/dc:title"/></fo:basic-link></fo:block></fo:table-cell>
                </fo:table-row>
                <!-- Originator -->
                <fo:table-row>
                    <fo:table-cell><fo:block><xsl:value-of select="$strings/Originator"/>:</fo:block></fo:table-cell>
                    <fo:table-cell><fo:block xsl:use-attribute-sets="value"><xsl:apply-templates select="prep:creator"/></fo:block></fo:table-cell>
                </fo:table-row>
                <!-- Owner -->
                <fo:table-row>
                    <fo:table-cell><fo:block><xsl:value-of select="$strings/Owner"/>:</fo:block></fo:table-cell>
                    <fo:table-cell><fo:block xsl:use-attribute-sets="value"><xsl:apply-templates select="prep:owner"/></fo:block></fo:table-cell>
                </fo:table-row>
                <fo:table-row height="8pt">
                    <fo:table-cell><fo:block linefeed-treatment="preserve"/></fo:table-cell>
                </fo:table-row>
                <!-- Weight -->
                <fo:table-row>
                    <fo:table-cell><fo:block><xsl:value-of select="$strings/Weight"/>:</fo:block></fo:table-cell>
                    <fo:table-cell><fo:block xsl:use-attribute-sets="value"><xsl:value-of select="tsl:weight"/></fo:block></fo:table-cell>
                </fo:table-row>
            </fo:table-body>
        </fo:table>
        
        <fo:block xsl:use-attribute-sets="artifact-description"><xsl:value-of select="$strings/Description"/>: <fo:inline xsl:use-attribute-sets="value"><xsl:apply-templates select="rqm:executionresult/dc:description"/></fo:inline></fo:block>
    </xsl:template>
    
    <xsl:template name="_buildWeightDistribution">
        <!-- section header -->
        <fo:block xsl:use-attribute-sets="section-heading"><xsl:value-of select="$strings/Weight_Distribution"/></fo:block>
        
        <!-- section content -->
        <fo:block><xsl:value-of select="$strings/Pass"/>: <xsl:value-of select="tsl:pointspassed"/></fo:block>
        <fo:block><xsl:value-of select="$strings/Fail"/>: <xsl:value-of select="tsl:pointsfailed"/></fo:block>
        <fo:block><xsl:value-of select="$strings/Inconclusive"/>: <xsl:value-of select="tsl:pointsinconclusive"/></fo:block>
        <fo:block><xsl:value-of select="$strings/PermFailed"/>: <xsl:value-of select="rqm:pointspermfailed"/></fo:block>
        <fo:block><xsl:value-of select="$strings/Deferred"/>: <xsl:value-of select="rqm:pointsdeferred"/></fo:block>
        <fo:block><xsl:value-of select="$strings/Blocked"/>: <xsl:value-of select="tsl:pointsblocked"/></fo:block>
        <fo:block><xsl:value-of select="$strings/Attempted"/>: <xsl:value-of select="tsl:pointsattempted"/></fo:block>        
    </xsl:template>
    
    <xsl:template name="buildSuiteResultDetailsTable">
        <xsl:param name="results"/>
        <xsl:param name="configuration"/>
        <fo:table xsl:use-attribute-sets="artifact-table">
            <fo:table-header xsl:use-attribute-sets="artifact-table-header">
                <!-- ID -->
                <fo:table-cell xsl:use-attribute-sets="artifact-table-header-cell id-header">
                    <fo:block><xsl:value-of select="$strings/ID"/></fo:block>
                </fo:table-cell>
                <!-- Test Case -->
                <fo:table-cell xsl:use-attribute-sets="artifact-table-header-cell">
                    <fo:block><xsl:value-of select="$strings/Test_Case"/></fo:block>
                </fo:table-cell>
                <!-- Test Enviroment -->
                <fo:table-cell xsl:use-attribute-sets="artifact-table-header-cell">
                    <fo:block><xsl:value-of select="$strings/Test_Environment"/></fo:block>
                </fo:table-cell>
                <!-- Test Script -->
                <fo:table-cell xsl:use-attribute-sets="artifact-table-header-cell">
                    <fo:block><xsl:value-of select="$strings/Test_Script"/></fo:block>
                </fo:table-cell>
                <!-- Owner -->
                <fo:table-cell xsl:use-attribute-sets="artifact-table-header-cell">
                    <fo:block><xsl:value-of select="$strings/Owner"/></fo:block>
                </fo:table-cell>
                <!-- Host Name -->
                <fo:table-cell xsl:use-attribute-sets="artifact-table-header-cell">
                    <fo:block><xsl:value-of select="$strings/Host_Name"/></fo:block>
                </fo:table-cell>
                <!-- Weight -->
                <fo:table-cell xsl:use-attribute-sets="artifact-table-header-cell">
                    <fo:block><xsl:value-of select="$strings/Weight"/></fo:block>
                </fo:table-cell>
                <!-- Actual Result -->
                <fo:table-cell xsl:use-attribute-sets="artifact-table-header-cell">
                    <fo:block><xsl:value-of select="$strings/Actual_Result"/></fo:block>
                </fo:table-cell>
            </fo:table-header>
            
            <fo:table-body>
                <xsl:choose>
                    <xsl:when test="$results">
                        <xsl:variable name="configuration-title">
                            <xsl:if test="$configuration/@href">
                                <xsl:value-of select="document(concat('rest://',$configuration/@href))/rqm:configuration/dc:title"/>
                            </xsl:if>
                        </xsl:variable>
                        
                        <xsl:for-each select="$results">
                            <fo:table-row>
                                <xsl:variable name="ter-result" select="document(concat('rest://', @href))/rqm:executionresult"/>

                                <!-- ID -->
                                <fo:table-cell xsl:use-attribute-sets="artifact-table-body-cell id-cell"><fo:block-container overflow="hidden"><fo:block><fo:basic-link external-destination="{$ter-result/dc:identifier}" xsl:use-attribute-sets="simple-link"><xsl:value-of select="$ter-result/rqm:webId"/></fo:basic-link></fo:block></fo:block-container></fo:table-cell>
                                <!-- Test Case -->
                                <fo:table-cell xsl:use-attribute-sets="artifact-table-body-cell"><fo:block-container overflow="hidden"><fo:block><xsl:if test="$ter-result/rqm:testcase/@href"><fo:basic-link external-destination="{$ter-result/rqm:testcase/@href}" xsl:use-attribute-sets="simple-link"><xsl:value-of select="document(concat('rest://',$ter-result/rqm:testcase/@href))/rqm:testcase/dc:title"/></fo:basic-link></xsl:if></fo:block></fo:block-container></fo:table-cell>
                                <!-- Test Enviroment -->
                                <fo:table-cell xsl:use-attribute-sets="artifact-table-body-cell"><fo:block-container overflow="hidden"><fo:block><xsl:choose><xsl:when test="$configuration/@href"><fo:basic-link external-destination="{$configuration/@href}" xsl:use-attribute-sets="simple-link"><xsl:value-of select="$configuration-title"/></fo:basic-link></xsl:when><xsl:otherwise><xsl:value-of select="$strings/Unassigned"/></xsl:otherwise></xsl:choose></fo:block></fo:block-container></fo:table-cell>
                                <!-- Test Script -->
                                <fo:table-cell xsl:use-attribute-sets="artifact-table-body-cell"><fo:block-container overflow="hidden"><fo:block><xsl:if test="$ter-result/rqm:testscript/@href"><fo:basic-link external-destination="{$ter-result/rqm:testscript/@href}" xsl:use-attribute-sets="simple-link"><xsl:value-of select="document(concat('rest://',$ter-result/rqm:testscript/@href))/rqm:testscript/dc:title"/></fo:basic-link></xsl:if></fo:block></fo:block-container></fo:table-cell>
                                <!-- Owner -->
                                <fo:table-cell xsl:use-attribute-sets="artifact-table-body-cell"><fo:block-container overflow="hidden"><fo:block><xsl:apply-templates select="$ter-result/alm:owner"/></fo:block></fo:block-container></fo:table-cell>
                                <!-- Host Name -->
                                <fo:table-cell xsl:use-attribute-sets="artifact-table-body-cell"><fo:block-container overflow="hidden"><fo:block><xsl:value-of select="$ter-result/ewir:machine"/></fo:block></fo:block-container></fo:table-cell>
                                <!-- Weight -->
                                <fo:table-cell xsl:use-attribute-sets="artifact-table-body-cell"><fo:block-container overflow="hidden"><fo:block><xsl:value-of select="$ter-result/ewir:weight"/></fo:block></fo:block-container></fo:table-cell>
                                <!-- Actual Result -->
                                <fo:table-cell xsl:use-attribute-sets="artifact-table-body-cell"><fo:block-container overflow="hidden"><fo:block><xsl:apply-templates select="$ter-result/alm:state"/></fo:block></fo:block-container></fo:table-cell>
                            </fo:table-row>
                        </xsl:for-each>
                    </xsl:when>
                    <xsl:otherwise>
                        <fo:table-row>
                            <!-- ID -->
                            <fo:table-cell><fo:block/></fo:table-cell>
                            <!-- Test Case -->
                            <fo:table-cell><fo:block/></fo:table-cell>
                            <!-- Test Enviroment -->
                            <fo:table-cell><fo:block/></fo:table-cell>
                            <!-- Test Script -->
                            <fo:table-cell><fo:block/></fo:table-cell>
                            <!-- Owner -->
                            <fo:table-cell><fo:block/></fo:table-cell>
                            <!-- Host Name -->
                            <fo:table-cell><fo:block/></fo:table-cell>
                            <!-- Weight -->
                            <fo:table-cell><fo:block/></fo:table-cell>
                            <!-- Actual Result -->
                            <fo:table-cell><fo:block/></fo:table-cell>
                        </fo:table-row>
                    </xsl:otherwise>
                </xsl:choose>
                
            </fo:table-body>
        </fo:table>
    </xsl:template>
    
    
    <xsl:template match="tsl:result|alm:state">
    	<xsl:variable name="stateId" select="."/>
        <xsl:variable name="customStateName" select="utils:getCustomResultStateName(current(), $projectArea)"/>
        
         <fo:inline>
            <xsl:attribute name="color">
                <xsl:choose>
                    <xsl:when test="current()='com.ibm.rqm.execution.common.state.passed'">green</xsl:when>
                    <xsl:when test="current()='com.ibm.rqm.execution.common.state.failed'">red</xsl:when>
                    <xsl:when test="current()='com.ibm.rqm.execution.common.state.inprogress'">orange</xsl:when>
                    <xsl:when test="current()='com.ibm.rqm.execution.common.state.notrun'">orange</xsl:when>
                    <xsl:when test="current()='com.ibm.rqm.execution.common.state.incomplete'">orange</xsl:when>
                    <xsl:when test="current()='com.ibm.rqm.execution.common.state.inconclusive'">orange</xsl:when>
                    <xsl:when test="current()='com.ibm.rqm.execution.common.state.part_blocked'">orange</xsl:when>
                    <xsl:when test="current()='com.ibm.rqm.execution.common.state.error'">red</xsl:when>
                    <xsl:when test="current()='com.ibm.rqm.execution.common.state.blocked'">orange</xsl:when>
                    <xsl:when test="current()='com.ibm.rqm.execution.common.state.perm_failed'">grey</xsl:when>
                    <xsl:when test="current()='com.ibm.rqm.execution.common.state.deferred'">rgb(100,100,175)</xsl:when>
                    <xsl:otherwise>black</xsl:otherwise>
                </xsl:choose>
            </xsl:attribute>
            
            <xsl:choose>
        		<xsl:when test="$customStateName">
        	 		<xsl:value-of select="$customStateName"/></xsl:when>
        		<xsl:otherwise>
        			<xsl:value-of select="$strings/*[name()=$stateId]"/></xsl:otherwise>
        	</xsl:choose>
        </fo:inline>
    </xsl:template>
    
    <xsl:template match="rqm:approvals">
        <!-- section header -->
        <fo:block xsl:use-attribute-sets="section-heading"><xsl:value-of select="$strings/Formal_Review"/></fo:block>
        
        <!-- section content -->
        <xsl:apply-templates select="rqm:approvalDescriptor"/>
        <xsl:call-template   name="_buildSuiteResultsApprovalComments"></xsl:call-template>
    </xsl:template>
     
    <xsl:template name="_buildEnvironmentSection">
        <!-- section header -->
        <fo:block xsl:use-attribute-sets="section-heading"><xsl:value-of select="$strings/Test_Environment"/></fo:block>
        
        <!-- section content -->
        <xsl:if test="rqm:channel">
        	<xsl:variable name="channel" select="proxy:rest($item, rqm:channel/@href)"/>

        	<fo:block>
            	<fo:inline xsl:use-attribute-sets="label"><xsl:value-of select="$strings/Channel"/>:</fo:inline>
            	<xsl:text> </xsl:text>
            	<fo:inline xsl:use-attribute-sets="value"><xsl:value-of select="$channel/dc:title"/></fo:inline>
        	</fo:block>
        </xsl:if>
    </xsl:template> 
    <xsl:template name="_buildAuditSection">
        <xsl:variable name="testsuitestateatexecutionstateid" select="tsl:testsuitestateatexecution"/>
        <xsl:variable name="testplanstateatexecutionstateid" select="tsl:testplanstateatexecution"/>
        <xsl:if test="$testsuitestateatexecutionstateid">
	        <!-- section header -->
	        <fo:block xsl:use-attribute-sets="section-heading"><xsl:value-of select="$strings/Audit"/></fo:block>
	        
	        <!-- section content -->
            <fo:block><xsl:value-of select="$strings/DateWhenRun"/><xsl:value-of select="time:formatMediumDateMediumTime(tsl:starttime, $locale)"/></fo:block>
            <xsl:if test="$testplanstateatexecutionstateid">
                <xsl:variable name="testplanstateatexecutionstatename" select="$strings/*[name()=$testplanstateatexecutionstateid]"/>    
				<fo:block>
			    	<xsl:value-of select="$strings/TestPlanStateAtExecution"/>: 
			        <xsl:text> </xsl:text>
		            <xsl:choose>
			        	<xsl:when test="$testplanstateatexecutionstatename">
			            	<xsl:value-of select="$testplanstateatexecutionstatename"/>
			            </xsl:when>
			            <xsl:otherwise>
			            	<xsl:variable name="workflowType" select="'TESTPLAN'"/>    
			                <xsl:value-of select="utils:getStateName($projectArea, $workflowType, $testplanstateatexecutionstateid)"/>
			            </xsl:otherwise>
			        </xsl:choose>
				</fo:block>			
            </xsl:if>
			<xsl:variable name="testsuitestateatexecutionstatename" select="$strings/*[name()=$testsuitestateatexecutionstateid]"/>    
			<fo:block>
				<xsl:value-of select="$strings/TestSuiteStateAtExecution"/>: 
			    <xsl:text> </xsl:text>
		        <xsl:choose>
			    	<xsl:when test="$testsuitestateatexecutionstatename">
			        	<xsl:value-of select="$testsuitestateatexecutionstatename"/>
			        </xsl:when>
			        <xsl:otherwise>
			        	<xsl:variable name="workflowType" select="'TESTSUITE'"/>    
			            <xsl:value-of select="utils:getStateName($projectArea, $workflowType, $testsuitestateatexecutionstateid)"/>
					</xsl:otherwise>
				</xsl:choose>
			</fo:block>
	    </xsl:if>        
    </xsl:template>  
    
</xsl:stylesheet>